@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2014 Edgewall Software
+# Copyright (C) 2007-2018 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -14,12 +14,12 @@
 
 """Example macro."""
 
-revision = "$Rev: 12605 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/HelloWorld.py $"
-
 from trac.util.translation import cleandoc_
 from trac.wiki.macros import WikiMacroBase
 
+revision = "$Rev: 16771 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/HelloWorld.py $"
+
 
 class HelloWorldMacro(WikiMacroBase):
     _description = cleandoc_(
