@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2014 Edgewall Software
+# Copyright (C) 2007-2018 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -14,15 +14,13 @@
 
 """Inserts the current time (in seconds) into the wiki page."""
 
-revision = "$Rev: 14481 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/Timestamp.py $"
-
-from datetime import datetime
-
-from genshi.builder import tag
 from trac.util.datefmt import datetime_now, format_datetime, utc
+from trac.util.html import tag
 from trac.wiki.macros import WikiMacroBase
 
+revision = "$Rev: 16771 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/Timestamp.py $"
+
 
 class TimestampMacro(WikiMacroBase):
     _description = "Inserts the current time (in seconds) into the wiki page."
