@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2018 Edgewall Software
 # Copyright (C) 2007 Eli Carter <retracile@gmail.com>
 # All rights reserved.
 #
@@ -18,8 +18,8 @@ from trac.core import Component, implements
 from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.perm import IPermissionRequestor
 
-revision = "$Rev: 14533 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/workflow/StatusFixer.py $"
+revision = "$Rev: 16771 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/workflow/StatusFixer.py $"
 
 class StatusFixerActionController(Component):
     """Provides the admin with a way to correct a ticket's status.
