@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2010 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,10 +12,9 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import cmd
-import locale
 import os.path
 import pkg_resources
 from shlex import shlex
@@ -24,24 +23,28 @@ import sys
 import traceback
 
 from trac import __version__ as VERSION
-from trac.admin import AdminCommandError, AdminCommandManager
+from trac.admin.api import AdminCommandError, AdminCommandManager, \
+                           get_console_locale
+from trac.config import Configuration
 from trac.core import TracError
 from trac.env import Environment
 from trac.ticket.model import *
-from trac.util import translation
+from trac.util import translation, warn_setuptools_issue
 from trac.util.html import html
 from trac.util.text import console_print, exception_to_unicode, printout, \
                            printerr, raw_input, to_unicode, \
                            getpreferredencoding
-from trac.util.translation import _, ngettext, get_negotiated_locale, \
-                                  has_babel, cleandoc_
+from trac.util.translation import _, ngettext, has_babel, cleandoc_
 from trac.versioncontrol.api import RepositoryManager
+from trac.web.chrome import default_mainnav_order, default_metanav_order
 from trac.wiki.admin import WikiAdmin
+from trac.wiki.formatter import MacroError
 from trac.wiki.macros import WikiMacroBase
 
+
 TRAC_VERSION = pkg_resources.get_distribution('Trac').version
 rl_completion_suppress_append = None
-LANG = os.environ.get('LANG')
+
 
 def find_readline_lib():
     """Return the name (and possibly the full path) of the readline library
@@ -109,20 +112,21 @@ class TracAdmin(cmd.Cmd):
             rv = cmd.Cmd.onecmd(self, line) or 0
         except SystemExit:
             raise
-        except AdminCommandError, e:
+        except AdminCommandError as e:
             printerr(_("Error: %(msg)s", msg=to_unicode(e)))
             if e.show_usage:
-                print
+                print()
                 self.do_help(e.cmd or self.arg_tokenize(line)[0])
             rv = 2
-        except TracError, e:
+        except TracError as e:
             printerr(exception_to_unicode(e))
             rv = 2
-        except Exception, e:
+        except Exception as e:
             printerr(exception_to_unicode(e))
             rv = 2
             if self.env_check():
-                self.env.log.error("Exception in trac-admin command: %s",
+                self.env.log.error("Exception in trac-admin command: %r%s",
+                                   line,
                                    exception_to_unicode(e, traceback=True))
         if not self.interactive:
             return rv
@@ -131,10 +135,10 @@ class TracAdmin(cmd.Cmd):
         self.interactive = True
         printout(_("""Welcome to trac-admin %(version)s
 Interactive Trac administration console.
-Copyright (C) 2003-2013 Edgewall Software
+Copyright (C) %(year)s Edgewall Software
 
 Type:  '?' or 'help' for help on commands.
-        """, version=TRAC_VERSION))
+        """, version=TRAC_VERSION, year='2003-2018'))
         self.cmdloop()
 
     ##
@@ -161,17 +165,17 @@ Type:  '?' or 'help' for help on commands.
             if not self.__env:
                 self._init_env()
             return self.__env
-        except Exception, e:
+        except Exception as e:
             printerr(_("Failed to open environment: %(err)s",
                        err=exception_to_unicode(e, traceback=True)))
             sys.exit(1)
 
     def _init_env(self):
         self.__env = env = Environment(self.envname)
+        negotiated = None
         # fixup language according to env settings
         if has_babel:
-            default = env.config.get('trac', 'default_language', '')
-            negotiated = get_negotiated_locale([LANG, default])
+            negotiated = get_console_locale(env)
             if negotiated:
                 translation.activate(negotiated)
 
@@ -179,6 +183,10 @@ Type:  '?' or 'help' for help on commands.
     ## Utility methods
     ##
 
+    @property
+    def cmd_mgr(self):
+        return AdminCommandManager(self.env)
+
     def arg_tokenize(self, argstr):
         """`argstr` is an `unicode` string
 
@@ -240,10 +248,9 @@ Type:  '?' or 'help' for help on commands.
         if line and line[-1] == ' ':    # Space starts new argument
             args.append('')
         if self.env_check():
-            cmd_mgr = AdminCommandManager(self.env)
             try:
-                comp = cmd_mgr.complete_command(args, cmd_only)
-            except Exception, e:
+                comp = self.cmd_mgr.complete_command(args, cmd_only)
+            except Exception as e:
                 printerr()
                 printerr(_('Completion error: %(err)s',
                            err=exception_to_unicode(e)))
@@ -270,19 +277,18 @@ Type:  '?' or 'help' for help on commands.
                 self._init_env()
             if self.needs_upgrade is None:
                 self.needs_upgrade = self.__env.needs_upgrade()
-        except TracError, e:
+        except TracError as e:
             raise AdminCommandError(to_unicode(e))
-        except Exception, e:
+        except Exception as e:
             raise AdminCommandError(exception_to_unicode(e))
         args = self.arg_tokenize(line)
         if args[0] == 'upgrade':
             self.needs_upgrade = None
         elif self.needs_upgrade:
-            raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
-                              'Run "trac-admin %(path)s upgrade"',
+            raise TracError(_('The Trac Environment needs to be upgraded. '
+                              'Run:\n\n  trac-admin "%(path)s" upgrade',
                               path=self.envname))
-        cmd_mgr = AdminCommandManager(self.env)
-        return cmd_mgr.execute_command(*args)
+        return self.cmd_mgr.execute_command(*args)
 
     ##
     ## Available Commands
@@ -304,9 +310,10 @@ Type:  '?' or 'help' for help on commands.
     def do_help(self, line=None):
         arg = self.arg_tokenize(line)
         if arg[0]:
+            cmd_mgr = None
             doc = getattr(self, "_help_" + arg[0], None)
             if doc is None and self.env_check():
-                cmd_mgr = AdminCommandManager(self.env)
+                cmd_mgr = self.cmd_mgr
                 doc = cmd_mgr.get_command_help(arg)
             if doc:
                 self.print_doc(doc)
@@ -314,7 +321,9 @@ Type:  '?' or 'help' for help on commands.
                 printerr(_("No documentation found for '%(cmd)s'."
                            " Use 'help' to see the list of commands.",
                            cmd=' '.join(arg)))
-                cmds = cmd_mgr.get_similar_commands(arg[0])
+                cmds = None
+                if cmd_mgr:
+                    cmds = cmd_mgr.get_similar_commands(arg[0])
                 if cmds:
                     printout('')
                     printout(ngettext("Did you mean this?",
@@ -326,7 +335,7 @@ Type:  '?' or 'help' for help on commands.
             printout(_("trac-admin - The Trac Administration Console "
                        "%(version)s", version=TRAC_VERSION))
             if not self.interactive:
-                print
+                print()
                 printout(_("Usage: trac-admin </path/to/projenv> "
                            "[command [subcommand] [option ...]]\n")
                     )
@@ -342,7 +351,7 @@ Type:  '?' or 'help' for help on commands.
     _help_EOF = _help_quit
 
     def do_quit(self, line):
-        print
+        print()
         sys.exit()
 
     do_exit = do_quit # Alias
@@ -355,7 +364,7 @@ Type:  '?' or 'help' for help on commands.
          """Create and initialize a new environment
 
          If no arguments are given, then the required parameters are requested
-         interactively.
+         interactively unless the optional argument `--config` is specified.
 
          One or more optional arguments --inherit=PATH can be used to specify
          the "[inherit] file" option at environment creation time, so that only
@@ -363,6 +372,12 @@ Type:  '?' or 'help' for help on commands.
          files are written to the conf/trac.ini file of the newly created
          environment. Relative paths are resolved relative to the "conf"
          directory of the new environment.
+
+         The optional argument --config=PATH can be used to specify a
+         configuration file that is used to populate the environment
+         configuration. The arguments <projectname>, <db> and any other
+         arguments passed in the invocation are optional, but if specified
+         will override values in the configuration file.
          """)]
 
     def do_initdb(self, line):
@@ -385,20 +400,20 @@ in order to initialize and prepare the project database.
         printout(_("""
  Please specify the connection string for the database to use.
  By default, a local SQLite database is created in the environment
- directory. It is also possible to use an already existing
+ directory. It is also possible to use an existing MySQL or
  PostgreSQL database (check the Trac documentation for the exact
  connection string syntax).
 """))
         ddb = 'sqlite:db/trac.db'
         prompt = _("Database connection string [%(default)s]> ", default=ddb)
         returnvals.append(raw_input(prompt).strip() or ddb)
-        print
+        print()
         return returnvals
 
     def do_initenv(self, line):
         def initenv_error(msg):
             printerr(_("Initenv for '%(env)s' failed.", env=self.envname),
-                     "\n" + msg)
+                     "\n%s" % msg)
         if self.env_check():
             initenv_error(_("Does an environment already exist?"))
             return 2
@@ -415,20 +430,37 @@ in order to initialize and prepare the project database.
 
         arg = self.arg_tokenize(line)
         inherit_paths = []
+        config_file_path = None
         i = 0
         while i < len(arg):
             item = arg[i]
             if item.startswith('--inherit='):
                 inherit_paths.append(arg.pop(i)[10:])
+            elif item.startswith('--config='):
+                config_file_path = arg.pop(i)[9:]
             else:
                 i += 1
+        config = None
+        if config_file_path:
+            if not os.path.exists(config_file_path):
+                initenv_error(_("The file specified in the --config argument "
+                                "does not exist: %(path)s.",
+                                path=config_file_path))
+                return 2
+            try:
+                config = Configuration(config_file_path)
+            except TracError as e:
+                initenv_error(e)
+                return 2
         arg = arg or [''] # Reset to usual empty in case we popped the only one
-        project_name = None
-        db_str = None
         repository_type = None
         repository_dir = None
-        if len(arg) == 1 and not arg[0]:
+        if len(arg) == 1 and not arg[0] and not config:
             project_name, db_str = self.get_initenv_args()
+        elif len(arg) < 2 and config:
+            project_name = db_str = None
+            if arg[0]:
+                project_name = arg[0]
         elif len(arg) == 2:
             project_name, db_str = arg
         elif len(arg) == 4:
@@ -439,14 +471,26 @@ in order to initialize and prepare the project database.
 
         try:
             printout(_("Creating and Initializing Project"))
-            options = [
-                ('project', 'name', project_name),
-                ('trac', 'database', db_str),
-            ]
+            options = []
+            if config:
+                for section in config.sections(defaults=False):
+                    options.extend((section, option, value)
+                                   for option, value
+                                   in config.options(section))
+            if project_name is not None:
+                options.append(('project', 'name', project_name))
+            if db_str is not None:
+                options.append(('trac', 'database', db_str))
+
+            def add_nav_order_options(section, default):
+                for i, name in enumerate(default, 1):
+                    options.append((section, name + '.order', float(i)))
+            add_nav_order_options('mainnav', default_mainnav_order)
+            add_nav_order_options('metanav', default_metanav_order)
             if repository_dir:
                 options.extend([
-                    ('trac', 'repository_type', repository_type),
-                    ('trac', 'repository_dir', repository_dir),
+                    ('repositories', '.type', repository_type),
+                    ('repositories', '.dir', repository_dir),
                 ])
             if inherit_paths:
                 options.append(('inherit', 'file',
@@ -454,7 +498,7 @@ in order to initialize and prepare the project database.
             try:
                 self.__env = Environment(self.envname, create=True,
                                          options=options)
-            except Exception, e:
+            except Exception as e:
                 initenv_error(_('Failed to create environment.'))
                 printerr(e)
                 traceback.print_exc()
@@ -472,20 +516,20 @@ in order to initialize and prepare the project database.
                     if repos:
                         printout(_(" Indexing default repository"))
                         repos.sync(self._resync_feedback)
-                except TracError, e:
+                except TracError as e:
                     printerr(_("""
 ---------------------------------------------------------------------
 Warning: couldn't index the default repository.
 
 This can happen for a variety of reasons: wrong repository type,
-no appropriate third party library for this repository type,
-no actual repository at the specified repository path...
+no appropriate third party library for this repository type, 
+no repository at the specified repository path...
 
 You can nevertheless start using your Trac environment, but
-you'll need to check again your trac.ini file and the [trac]
-repository_type and repository_path settings.
+you'll need to check your `.type` and `.dir` option values in 
+the [repositories] section of your trac.ini file.
 """))
-        except Exception, e:
+        except Exception as e:
             initenv_error(to_unicode(e))
             traceback.print_exc()
             return 2
@@ -516,7 +560,7 @@ website:
 Congratulations!
 """, project_name=project_name, project_path=self.envname,
            project_dir=os.path.basename(self.envname),
-           config_path=os.path.join(self.envname, 'conf', 'trac.ini')))
+           config_path=self.__env.config_file_path))
 
     def _resync_feedback(self, rev):
         sys.stdout.write(' [%s]\r' % rev)
@@ -545,30 +589,34 @@ class TracAdminHelpMacro(WikiMacroBase):
                 cmd_mgr = AdminCommandManager(self.env)
                 doc = cmd_mgr.get_command_help(arg)
             if not doc:
-                raise TracError('Unknown trac-admin command "%s"' % content)
+                raise MacroError(_('Unknown trac-admin command '
+                                   '"%(command)s"', command=content))
         else:
             doc = TracAdmin.all_docs(self.env)
         buf = StringIO.StringIO()
         TracAdmin.print_doc(doc, buf, long=True)
-        return html.PRE(buf.getvalue(), class_='wiki')
+        return html.PRE(buf.getvalue().decode('utf-8'), class_='wiki')
 
 
-def run(args=None):
-    """Main entry point."""
+def _quote_args(args):
+    def quote(arg):
+        if arg.isalnum():
+            return arg
+        return '"\'"'.join("'%s'" % v for v in arg.split("'"))
+    return [quote(arg) for arg in args]
+
+
+def _run(args):
     if args is None:
         args = sys.argv[1:]
-    locale = None
-    if has_babel:
-        import babel
-        try:
-            locale = get_negotiated_locale([LANG]) or babel.Locale.default()
-        except babel.UnknownLocaleError:
-            pass
-        translation.activate(locale)
+    warn_setuptools_issue()
+    if sys.flags.optimize != 0:
+        printerr(_("Python with optimizations is not supported."))
+        return 2
     admin = TracAdmin()
-    if len(args) > 0:
+    if args:
         if args[0] in ('-h', '--help', 'help'):
-            return admin.onecmd(' '.join(['help'] + args[1:]))
+            return admin.onecmd(' '.join(_quote_args(['help'] + args[1:])))
         elif args[0] in ('-v','--version'):
             printout(os.path.basename(sys.argv[0]), TRAC_VERSION)
         else:
@@ -578,12 +626,10 @@ def run(args=None):
             except UnicodeDecodeError:
                 printerr(_("Non-ascii environment path '%(path)s' not "
                            "supported.", path=to_unicode(env_path)))
-                sys.exit(2)
+                return 2
             admin.env_set(env_path)
             if len(args) > 1:
-                s_args = ' '.join(["'%s'" % c for c in args[2:]])
-                command = args[1] + ' ' + s_args
-                return admin.onecmd(command)
+                return admin.onecmd(' '.join(_quote_args(args[1:])))
             else:
                 while True:
                     try:
@@ -594,6 +640,15 @@ def run(args=None):
         return admin.onecmd("help")
 
 
+def run(args=None):
+    """Main entry point."""
+    translation.activate(get_console_locale())
+    try:
+        return _run(args)
+    finally:
+        translation.deactivate()
+
+
 if __name__ == '__main__':
     pkg_resources.require('Trac==%s' % VERSION)
     sys.exit(run())
