@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2005-2009 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -16,6 +16,7 @@
 
 import os
 import re
+from pkg_resources import DistributionNotFound
 
 from genshi import Markup
 
@@ -37,10 +38,8 @@ try:
     from psycopg2.extensions import register_type, UNICODE, \
                                     register_adapter, AsIs, QuotedString
 except ImportError:
-    has_psycopg = False
-    psycopg2_version = None
+    raise DistributionNotFound('psycopg2>=2.0 or psycopg2-binary', ['Trac'])
 else:
-    has_psycopg = True
     register_type(UNICODE)
     register_adapter(Markup, lambda markup: QuotedString(unicode(markup)))
     register_adapter(type(empty), lambda empty: AsIs("''"))
@@ -87,9 +86,6 @@ class PostgreSQLConnector(Component):
     pg_dump_path = Option('trac', 'pg_dump_path', 'pg_dump',
         """Location of pg_dump for Postgres database backups""")
 
-    def __init__(self):
-        self.error = None
-
     # ISystemInfoProvider methods
 
     def get_system_info(self):
@@ -99,13 +95,12 @@ class PostgreSQLConnector(Component):
     # IDatabaseConnector methods
 
     def get_supported_schemes(self):
-        if not has_psycopg:
-            self.error = _("Cannot load Python bindings for PostgreSQL")
-        yield 'postgres', -1 if self.error else 1
+        yield 'postgres', 1
 
     def get_connection(self, path, log=None, user=None, password=None,
                        host=None, port=None, params={}):
         self.required = True
+        params.setdefault('schema', 'public')
         return PostgreSQLConnection(path, log, user, password, host, port,
                                     params)
 
@@ -117,7 +112,7 @@ class PostgreSQLConnector(Component):
         cnx = self.get_connection(path, log, user, password, host, port,
                                   params)
         cursor = cnx.cursor()
-        if cnx.schema:
+        if cnx.schema and cnx.schema != 'public':
             cursor.execute('CREATE SCHEMA ' + _quote(cnx.schema))
             cursor.execute('SET search_path TO %s', (cnx.schema,))
         if schema is None:
@@ -131,7 +126,11 @@ class PostgreSQLConnector(Component):
                    port=None, params={}):
         cnx = self.get_connection(path, log, user, password, host, port,
                                   params)
-        cnx.execute('DROP SCHEMA %s CASCADE' % _quote(cnx.schema))
+        if cnx.schema and cnx.schema != 'public':
+            cnx.execute('DROP SCHEMA %s CASCADE' % _quote(cnx.schema))
+        else:
+            for table in cnx.get_table_names():
+                cnx.execute('DROP TABLE %s' % _quote(table))
         cnx.commit()
 
     def db_exists(self, path, log=None, user=None, password=None, host=None,
@@ -192,6 +191,7 @@ class PostgreSQLConnector(Component):
         db_url = self.env.config.get('trac', 'database')
         scheme, db_prop = parse_connection_uri(db_url)
         db_params = db_prop.setdefault('params', {})
+        db_params.setdefault('schema', 'public')
         db_name = os.path.basename(db_prop['path'])
 
         args = [self.pg_dump_path, '-C', '--inserts', '-x', '-Z', '8']
@@ -203,12 +203,11 @@ class PostgreSQLConnector(Component):
             if '/' not in host:
                 args.extend(['-p', str(db_prop.get('port', '5432'))])
 
-        if 'schema' in db_params:
-            # Need quote for -n (--schema) option in PostgreSQL 8.2+
-            if re.search(r' 8\.[01]\.', self._version()):
-                args.extend(['-n', db_params['schema']])
-            else:
-                args.extend(['-n', '"%s"' % db_params['schema']])
+        # Need quote for -n (--schema) option in PostgreSQL 8.2+
+        if re.search(r' 8\.[01]\.', self._version()):
+            args.extend(['-n', db_params['schema']])
+        else:
+            args.extend(['-n', '"%s"' % db_params['schema']])
 
         dest_file += ".gz"
         args.extend(['-f', dest_file, db_name])
@@ -258,13 +257,13 @@ class PostgreSQLConnection(ConnectionBase, ConnectionWrapper):
                                               port))
 
         cnx.set_client_encoding('UNICODE')
-        self.schema = None
-        if 'schema' in params:
-            self.schema = params['schema']
+        self.schema = params.get('schema', 'public')
+        if self.schema != 'public':
             try:
                 cnx.cursor().execute('SET search_path TO %s', (self.schema,))
                 cnx.commit()
             except (DataError, ProgrammingError):
+                # probably the schema doesn't exist
                 cnx.rollback()
         ConnectionWrapper.__init__(self, cnx, log)
 
@@ -293,8 +292,8 @@ class PostgreSQLConnection(ConnectionBase, ConnectionWrapper):
         if (self._version or '').startswith(('8.0.', '8.1.')):
             cursor = self.cursor()
             cursor.execute("""SELECT table_name FROM information_schema.tables
-                              WHERE table_schema=current_schema()
-                              AND table_name=%s""", (table,))
+                              WHERE table_schema=%s AND table_name=%s
+                              """, (self.schema, table))
             for row in cursor:
                 if row[0] == table:
                     self.execute("DROP TABLE " + self.quote(table))
@@ -341,8 +340,6 @@ class PostgreSQLConnection(ConnectionBase, ConnectionWrapper):
         return _quote(identifier)
 
     def reset_tables(self):
-        if not self.schema:
-            return []
         # reset sequences
         # information_schema.sequences view is available in
         # PostgreSQL 8.2+ however Trac supports PostgreSQL 8.0+, uses
