@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,8 +13,8 @@
 
 import unittest
 
+from trac.mimeview.tests import api, patch, pygments, rst, txtl
 from trac.mimeview.tests.functional import functionalSuite
-from trac.mimeview.tests import api, patch, pygments
 
 
 def test_suite():
@@ -22,6 +22,8 @@ def test_suite():
     suite.addTest(api.test_suite())
     suite.addTest(patch.test_suite())
     suite.addTest(pygments.test_suite())
+    suite.addTest(rst.test_suite())
+    suite.addTest(txtl.test_suite())
     return suite
 
 
