@@ -14,17 +14,18 @@
 import unittest
 from email import message_from_string
 
-from genshi.builder import tag
-
 import trac.tests.compat
 from trac.core import Component, implements
 from trac.notification.api import (
-    IEmailSender, INotificationFormatter, INotificationSubscriber,
-    NotificationEvent, NotificationSystem,
+    IEmailAddressResolver, IEmailSender, INotificationFormatter,
+    INotificationSubscriber, NotificationEvent, NotificationSystem,
 )
+from trac.notification.mail import RecipientMatcher
 from trac.notification.model import Subscription
 from trac.test import EnvironmentStub
+from trac.ticket.model import _fixup_cc_list
 from trac.util.datefmt import datetime_now, utc
+from trac.util.html import escape
 from trac.web.session import DetachedSession
 
 
@@ -60,7 +61,7 @@ class TestFormatter(Component):
         if style == 'text/html':
             if 'raise-text-html' in text:
                 raise ValueError()
-            return unicode(tag.p(text))
+            return u'<p>%s</p>' % escape(text)
 
 
 class TestSubscriber(Component):
@@ -86,6 +87,15 @@ class TestSubscriber(Component):
         return ()
 
 
+class TestEmailAddressResolver(Component):
+
+    implements(IEmailAddressResolver)
+
+    def get_address_for_session(self, sid, authenticated):
+        if authenticated == 1:
+            return '%s@example.net' % sid
+
+
 class TestNotificationEvent(NotificationEvent): pass
 
 
@@ -101,19 +111,25 @@ class EmailDistributorTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(enable=['trac.*', TestEmailSender,
-                                           TestFormatter, TestSubscriber])
-        config = self.env.config
+                                           TestFormatter, TestSubscriber,
+                                           TestEmailAddressResolver])
+        self.config = config = self.env.config
         config.set('notification', 'smtp_from', 'trac@example.org')
         config.set('notification', 'smtp_enabled', 'enabled')
         config.set('notification', 'smtp_always_cc', 'cc@example.org')
         config.set('notification', 'smtp_always_bcc', 'bcc@example.org')
         config.set('notification', 'email_sender', 'TestEmailSender')
+        config.set('notification', 'email_address_resolvers',
+                   'SessionEmailResolver,TestEmailAddressResolver')
         self.sender = TestEmailSender(self.env)
         self.notsys = NotificationSystem(self.env)
         with self.env.db_transaction:
             self._add_session('foo', email='foo@example.org')
             self._add_session('bar', email='bar@example.org',
                               name=u"Bäŕ's name")
+            self._add_session('baz', name='Baz')
+            self._add_session('qux', tz='UTC')
+            self._add_session('corge', email='corge-mail')
 
     def tearDown(self):
         self.env.reset_db()
@@ -124,8 +140,10 @@ class EmailDistributorTestCase(unittest.TestCase):
                                       time or datetime_now(utc), author=author)
         self.notsys.notify(event)
 
-    def _add_session(self, sid, **attrs):
+    def _add_session(self, sid, values=None, **attrs):
         session = DetachedSession(self.env, sid)
+        if values is not None:
+            attrs.update(values)
         for name, value in attrs.iteritems():
             session[name] = value
         session.save()
@@ -169,6 +187,8 @@ class EmailDistributorTestCase(unittest.TestCase):
         with self.env.db_transaction:
             self._add_subscription(sid='foo')
             self._add_subscription(sid='bar')
+            self._add_subscription(sid='baz')
+            self._add_subscription(sid='qux')
         self._notify_event('blah')
 
         history = self.sender.history
@@ -177,6 +197,7 @@ class EmailDistributorTestCase(unittest.TestCase):
         from_addr, recipients, message = history[0]
         self.assertEqual('trac@example.org', from_addr)
         self.assertEqual(set(('foo@example.org', 'bar@example.org',
+                              'baz@example.net', 'qux@example.net',
                               'cc@example.org', 'bcc@example.org')),
                          set(recipients))
         self._assert_mail(message, 'text/plain', 'blah')
@@ -219,6 +240,41 @@ class EmailDistributorTestCase(unittest.TestCase):
                 self.assertEqual(['bar@example.org'], recipients)
                 self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
 
+    def test_formats_in_session_and_tracini(self):
+        self.config.set('notification', 'smtp_always_cc', 'bar,quux')
+        self.config.set('notification', 'smtp_always_bcc', '')
+        self.config.set('notification', 'default_format.email', 'text/html')
+        with self.env.db_transaction:
+            for user in ('foo', 'bar', 'baz', 'qux', 'quux'):
+                self._add_session(user, email='%s@example.org' % user)
+            self._add_subscription(sid='foo', format='text/plain')
+            # bar - no subscriptions
+            self._add_session('bar',
+                              {'notification.format.email': 'text/plain'})
+            self._add_subscription(sid='baz', format='text/plain')
+            self._add_session('baz',
+                              {'notification.format.email': 'text/html'})
+            self._add_subscription(sid='qux', format='text/html')
+            self._add_session('qux',
+                              {'notification.format.email': 'text/plain'})
+            # quux - no subscriptions and no preferred format in session
+        self._notify_event('blah')
+
+        history = self.sender.history
+        self.assertNotEqual([], history)
+        self.assertEqual(2, len(history))
+        for from_addr, recipients, message in history:
+            self.assertEqual('trac@example.org', from_addr)
+            recipients = sorted(recipients)
+            if 'bar@example.org' in recipients:
+                self.assertEqual(['bar@example.org', 'baz@example.org',
+                                  'foo@example.org'], recipients)
+                self._assert_mail(message, 'text/plain', 'blah')
+            if 'qux@example.org' in recipients:
+                self.assertEqual(['quux@example.org', 'qux@example.org'],
+                                 recipients)
+                self._assert_alternative_mail(message, 'blah', '<p>blah</p>')
+
     def test_broken_plain_formatter(self):
         with self.env.db_transaction:
             self._add_subscription(sid='foo', format='text/plain')
@@ -348,6 +404,108 @@ class EmailDistributorTestCase(unittest.TestCase):
         self.assertEqual('"Trac" <trac@example.org>', message['From'])
         self.assertEqual(1, len(history))
 
+    def test_ignore_domains(self):
+        config = self.env.config
+        config.set('notification', 'smtp_always_cc',
+                   'cc@example.org, cc@example.net')
+        config.set('notification', 'smtp_always_bcc',
+                   'bcc@example.org, bcc@example.net')
+        config.set('notification', 'ignore_domains',
+                   'example.org, example.com')
+
+        with self.env.db_transaction:
+            self._add_subscription(sid='foo')
+            self._add_subscription(sid='bar')
+            self._add_subscription(sid='baz')
+            self._add_subscription(sid='qux')
+        self._notify_event('blah')
+
+        history = self.sender.history
+        self.assertNotEqual([], history)
+        self.assertEqual(1, len(history))
+        from_addr, recipients, message = history[0]
+        self.assertEqual('trac@example.org', from_addr)
+        self.assertEqual(set(('baz@example.net', 'qux@example.net',
+                              'cc@example.net', 'bcc@example.net')),
+                         set(recipients))
+
+    def _test_without_domain(self, use_short_addr='disabled',
+                             smtp_default_domain=''):
+        config = self.env.config
+        config.set('notification', 'use_short_addr', use_short_addr)
+        config.set('notification', 'smtp_default_domain', smtp_default_domain)
+        config.set('notification', 'smtp_from', 'from-trac')
+        config.set('notification', 'smtp_always_cc', 'qux, cc@example.org')
+        config.set('notification', 'smtp_always_bcc', 'bcc1@example.org, bcc2')
+        config.set('notification', 'email_address_resolvers',
+                   'SessionEmailResolver')
+        with self.env.db_transaction:
+            self._add_subscription(sid='foo')
+            self._add_subscription(sid='baz')
+            self._add_subscription(sid='corge')
+        self._notify_event('blah')
+        history = self.sender.history
+        self.assertNotEqual([], history)
+        self.assertEqual(1, len(history))
+        return history
+
+    def _assert_equal_sets(self, expected, actual):
+        expected = set(expected)
+        actual = set(actual)
+        if expected != actual:
+            self.fail('%r != %r' % ((expected - actual, actual - expected)))
+
+    def _cclist(self, cc):
+        return _fixup_cc_list(cc).split(', ')
+
+    def test_use_short_addr(self):
+        history = self._test_without_domain(use_short_addr='enabled')
+        from_addr, recipients, message = history[0]
+        self.assertEqual('from-trac', from_addr)
+        self.assertEqual('"My Project" <from-trac>', message['From'])
+        self._assert_equal_sets(['qux', 'cc@example.org', 'bcc1@example.org',
+                                 'bcc2', 'foo@example.org', 'baz',
+                                 'corge-mail'], recipients)
+        self._assert_equal_sets(['qux', 'cc@example.org'],
+                                self._cclist(message['Cc']))
+
+    def test_smtp_default_domain(self):
+        history = self._test_without_domain(smtp_default_domain='example.com')
+        from_addr, recipients, message = history[0]
+        self.assertEqual('from-trac@example.com', from_addr)
+        self.assertEqual('"My Project" <from-trac@example.com>',
+                         message['From'])
+        self._assert_equal_sets(['qux@example.com', 'cc@example.org',
+                                 'bcc1@example.org', 'bcc2@example.com',
+                                 'foo@example.org', 'baz@example.com',
+                                 'corge-mail@example.com'], recipients)
+        self._assert_equal_sets(['qux@example.com', 'cc@example.org'],
+                                self._cclist(message['Cc']))
+
+    def test_username_is_email(self):
+        config = self.env.config
+        config.set('notification', 'email_address_resolvers',
+                   'SessionEmailResolver')
+        with self.env.db_transaction:
+            self._add_session(sid='foo@example.com')
+            self._add_session(sid='bar@example.com',
+                              email='foo@bar.example.org')
+            self._add_subscription(sid='foo@example.com')
+            self._add_subscription(sid='bar@example.com')
+            self._add_subscription(sid='baz@example.com')  # no session
+        self._notify_event('blah')
+        history = self.sender.history
+        self.assertNotEqual([], history)
+        self.assertEqual(1, len(history))
+        from_addr, recipients, message = history[0]
+        self.assertEqual('trac@example.org', from_addr)
+        self.assertEqual('"My Project" <trac@example.org>', message['From'])
+        self.assertEqual(set(['foo@example.com', 'foo@bar.example.org',
+                              'baz@example.com', 'cc@example.org',
+                              'bcc@example.org']), set(recipients))
+        self._assert_equal_sets(['cc@example.org'],
+                                self._cclist(message['Cc']))
+
 
 def test_suite():
     suite = unittest.TestSuite()
