@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -35,6 +35,7 @@ from trac.wiki.formatter import extract_link
 
 
 class SearchModule(Component):
+    """Controller for the search sub-system"""
 
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
                ITemplateProvider, IWikiSyntaxProvider)
@@ -88,7 +89,7 @@ class SearchModule(Component):
             return ('opensearch.xml', {},
                     'application/opensearchdescription+xml')
 
-        query = req.args.get('q')
+        query = req.args.getfirst('q')
         available_filters = []
         for source in self.search_sources:
             available_filters.extend(source.get_search_filters(req) or [])
