@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2014 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -20,9 +20,11 @@
 import abc
 import doctest
 import inspect
+import numbers
 import os
 import shutil
 import sys
+import time
 import types
 import unittest
 import StringIO
@@ -34,9 +36,6 @@ except ImportError:
 else:
     locale_en = Locale.parse('en_US')
 
-import trac.db.mysql_backend
-import trac.db.postgres_backend
-import trac.db.sqlite_backend
 from trac.config import Configuration
 from trac.core import ComponentManager, ComponentMeta, TracError
 from trac.db.api import DatabaseManager, parse_connection_uri
@@ -45,7 +44,7 @@ from trac.perm import PermissionCache
 from trac.ticket.default_workflow import load_workflow_config_snippet
 from trac.util import translation
 from trac.util.datefmt import time_now, utc
-from trac.web.api import _RequestArgs, Request
+from trac.web.api import _RequestArgs, Request, arg_list_to_args
 from trac.web.session import Session
 
 
@@ -173,8 +172,25 @@ def MockRequest(env, **kwargs):
     else:
         perm = PermissionCache(env, authname)
 
-    args = _RequestArgs()
-    args.update(kwargs.get('args', {}))
+    def convert(val):
+        if isinstance(val, bool):
+            return unicode(int(val))
+        elif isinstance(val, numbers.Real):
+            return unicode(val)
+        elif isinstance(val, (list, tuple)):
+            return [convert(v) for v in val]
+        else:
+            return val
+
+    if 'arg_list' in kwargs:
+        arg_list = [(k, convert(v)) for k, v in kwargs['arg_list']]
+        args = arg_list_to_args(arg_list)
+    else:
+        args = _RequestArgs()
+        args.update((k, convert(v))
+                    for k, v in kwargs.get('args', {}).iteritems())
+        arg_list = [(name, value) for name in args
+                                  for value in args.getlist(name)]
 
     environ = {
         'trac.base_url': env.abs_href(),
@@ -206,21 +222,19 @@ def MockRequest(env, **kwargs):
 
     from trac.web.chrome import Chrome
     req.callbacks.update({
-        'arg_list': None,
+        'arg_list': lambda req: arg_list,
         'args': lambda req: args,
         'authname': lambda req: authname,
         'chrome': Chrome(env).prepare_request,
         'form_token': lambda req: kwargs.get('form_token'),
-        'languages': Request._parse_languages,
         'lc_time': lambda req: kwargs.get('lc_time', locale_en),
         'locale': lambda req: kwargs.get('locale'),
-        'incookie': Request._parse_cookies,
         'perm': lambda req: perm,
         'session': lambda req: Session(env, req),
         'tz': lambda req: kwargs.get('tz', utc),
-        'use_xsendfile': False,
-        'xsendfile_header': None,
-        '_inheaders': Request._parse_headers
+        'use_xsendfile': lambda req: False,
+        'xsendfile_header': lambda req: None,
+        'configurable_headers': lambda req: [],
     })
 
     return req
@@ -282,8 +296,11 @@ def get_dburi():
                 not db_prop.get('params', {}).get('synchronous'):
             # Speed-up tests with SQLite database
             dburi += ('&' if '?' in dburi else '?') + 'synchronous=off'
-        return dburi
-    return 'sqlite::memory:'
+    else:
+        scheme = 'sqlite'
+        dburi = '%s::memory:' % scheme
+    __import__('trac.db.%s_backend' % scheme)
+    return dburi
 
 
 def reset_sqlite_db(env, db_prop):
@@ -374,10 +391,13 @@ class EnvironmentStub(Environment):
         for name_or_class in disable or ():
             config_key = self._component_name(name_or_class)
             self.config.set('components', config_key, 'disabled')
+        self.config.set('trac', 'permission_policies',
+                        'DefaultPermissionPolicy, LegacyAttachmentPolicy')
 
         # -- logging
         from trac.log import logger_handler_factory
         self.log, self._log_handler = logger_handler_factory('test')
+        self.log.addHandler(self._log_handler)
 
         # -- database
         self.dburi = get_dburi()
@@ -476,7 +496,7 @@ class EnvironmentStub(Environment):
         """
         self.reset_db()
         self.shutdown() # really closes the db connections
-        shutil.rmtree(self.path)
+        rmtree(self.env.path)
         if self._old_registry is not None:
             self.restore_component_registry()
 
@@ -528,6 +548,34 @@ def locate(fn):
     return None
 
 
+def rmtree(path):
+    import errno
+    def onerror(function, path, excinfo, retry=1):
+        # `os.remove` fails for a readonly file on Windows.
+        # Then, it attempts to be writable and remove.
+        if function != os.remove:
+            raise
+        e = excinfo[1]
+        if isinstance(e, OSError) and e.errno == errno.EACCES:
+            mode = os.stat(path).st_mode
+            os.chmod(path, mode | 0666)
+            try:
+                function(path)
+            except Exception:
+                # print "%d: %s %o" % (retry, path, os.stat(path).st_mode)
+                if retry > 10:
+                    raise
+                time.sleep(0.1)
+                onerror(function, path, excinfo, retry + 1)
+        else:
+            raise
+    if os.name == 'nt' and isinstance(path, str):
+        # Use unicode characters in order to allow non-ansi characters
+        # on Windows.
+        path = unicode(path, sys.getfilesystemencoding())
+    shutil.rmtree(path, onerror=onerror)
+
+
 INCLUDE_FUNCTIONAL_TESTS = True
 
 
