@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,6 +14,7 @@
 import os
 import tempfile
 import unittest
+import zipfile
 from datetime import datetime
 from StringIO import StringIO
 
@@ -23,7 +24,7 @@ from trac.perm import IPermissionPolicy, PermissionCache
 from trac.resource import IResourceManager, Resource, resource_exists
 from trac.test import EnvironmentStub, MockRequest
 from trac.util.datefmt import utc, to_utimestamp
-from trac.web.api import HTTPBadRequest
+from trac.web.api import HTTPBadRequest, RequestDone
 
 
 hashes = {
@@ -333,8 +334,40 @@ class AttachmentModuleTestCase(unittest.TestCase):
 
     def test_resource_doesnt_exist(self):
         """Non-existent resource returns False from resource_exists."""
-        r = Resource('wiki', 'WikiStart').child('attachment', 'file.txt')
-        self.assertFalse(AttachmentModule(self.env).resource_exists(r))
+        parent = Resource('parent_realm', 'parent_id')
+        self.assertTrue(resource_exists(self.env, parent))
+        r = parent.child('attachment', 'file.txt')
+        self.assertFalse(resource_exists(self.env, r))
+
+    def test_download_zip(self):
+        att = Attachment(self.env, 'parent_realm', 'parent_id')
+        att.description = 'Blah blah'
+        att.insert('foo.txt', StringIO('foo'), 3,
+                   datetime(2016, 9, 23, 12, 34, 56, tzinfo=utc))
+        att = Attachment(self.env, 'parent_realm', 'parent_id')
+        att.insert('bar.jpg', StringIO('bar'), 3,
+                   datetime(2016, 12, 14, 23, 56, 30, tzinfo=utc))
+        module = AttachmentModule(self.env)
+        req = MockRequest(self.env, args={'format': 'zip'},
+                          path_info='/attachment/parent_realm/parent_id/')
+
+        self.assertTrue(module.match_request(req))
+        self.assertRaises(RequestDone, module.process_request, req)
+        z = zipfile.ZipFile(req.response_sent, 'r')
+        self.assertEqual(['bar.jpg', 'foo.txt'],
+                         sorted(i.filename for i in z.infolist()))
+
+        zinfo = z.getinfo('foo.txt')
+        self.assertEqual('foo', z.read('foo.txt'))
+        self.assertEqual(3, zinfo.file_size)
+        self.assertEqual((2016, 9, 23, 12, 34, 56), zinfo.date_time)
+        self.assertEqual('Blah blah', zinfo.comment)
+
+        zinfo = z.getinfo('bar.jpg')
+        self.assertEqual('bar', z.read('bar.jpg'))
+        self.assertEqual(3, zinfo.file_size)
+        self.assertEqual((2016, 12, 14, 23, 56, 30), zinfo.date_time)
+        self.assertEqual('', zinfo.comment)
 
 
 def test_suite():
