@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,8 +11,6 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
 import tempfile
 import unittest
 
@@ -186,20 +184,20 @@ def email_never_obfuscate_setup(tc):
     tc.env.config.set('trac', 'never_obfuscate_mailto', True)
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(formatter.suite(SEARCH_TEST_CASES, file=__file__))
-    suite.addTest(formatter.suite(ATTACHMENT_TEST_CASES, file=__file__,
-                                  context=('wiki', 'WikiStart'),
-                                  setup=attachment_setup,
-                                  teardown=attachment_teardown))
-    suite.addTest(formatter.suite(EMAIL_TEST_CASE_DEFAULT, file=__file__,
-                                  context=email_default_context()))
-    suite.addTest(formatter.suite(EMAIL_TEST_CASE_NEVER_OBFUSCATE,
-                                  file=__file__,
-                                  context=email_default_context(),
-                                  setup=email_never_obfuscate_setup))
+    suite.addTest(formatter.test_suite(SEARCH_TEST_CASES, file=__file__))
+    suite.addTest(formatter.test_suite(ATTACHMENT_TEST_CASES, file=__file__,
+                                       context=('wiki', 'WikiStart'),
+                                       setup=attachment_setup,
+                                       teardown=attachment_teardown))
+    suite.addTest(formatter.test_suite(EMAIL_TEST_CASE_DEFAULT, file=__file__,
+                                       context=email_default_context()))
+    suite.addTest(formatter.test_suite(EMAIL_TEST_CASE_NEVER_OBFUSCATE,
+                                       file=__file__,
+                                       context=email_default_context(),
+                                       setup=email_never_obfuscate_setup))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
