@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2014 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
@@ -1187,8 +1187,13 @@ class Report(object):
         self.id = None
 
     def insert(self):
-        """Insert a new report."""
+        """Insert a new report.
+
+        :raises TracError: if `query` is empty
+        """
         assert not self.exists, "Cannot insert existing report"
+        if not self.query:
+            raise TracError(_("Query cannot be empty."))
 
         self.env.log.debug("Creating new report '%s'", self.id)
         with self.env.db_transaction as db:
@@ -1199,6 +1204,12 @@ class Report(object):
             self.id = db.get_last_id(cursor, 'report')
 
     def update(self):
+        """Update a report.
+
+        :raises TracError: if `query` is empty
+        """
+        if not self.query:
+            raise TracError(_("Query cannot be empty."))
         self.env.db_transaction("""
             UPDATE report SET title=%s, query=%s, description=%s
             WHERE id=%s
