@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2009 Edgewall Software
+# Copyright (C) 2004-2018 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -47,7 +47,7 @@ from trac.web.chrome import (INavigationContributor, Chrome,
                              add_script_data, add_stylesheet, add_warning,
                              web_context)
 from trac.wiki.api import IWikiSyntaxProvider
-from trac.wiki.formatter import system_message
+from trac.wiki.formatter import MacroError
 from trac.wiki.macros import WikiMacroBase
 
 
@@ -222,20 +222,17 @@ class Query(object):
             if col in cols:
                 cols.remove(col)
                 cols.append(col)
+        constrained_fields = set(self.constraint_cols)
 
-        def sort_columns(col1, col2):
-            constrained_fields = self.constraint_cols.keys()
-            if 'id' in (col1, col2):
-                # Ticket ID is always the first column
-                return -1 if col1 == 'id' else 1
-            elif 'summary' in (col1, col2):
-                # Ticket summary is always the second column
-                return -1 if col1 == 'summary' else 1
-            elif col1 in constrained_fields or col2 in constrained_fields:
-                # Constrained columns appear before other columns
-                return -1 if col1 in constrained_fields else 1
-            return 0
-        cols.sort(sort_columns)
+        def sort_columns(name):
+            if name == 'id':
+                return 1  # Ticket ID is always the first column
+            if name == 'summary':
+                return 2  # Ticket summary is always the second column
+            if name in constrained_fields:
+                return 3  # Constrained columns appear before other columns
+            return 4
+        cols.sort(key=sort_columns)
         return cols
 
     def get_default_columns(self):
@@ -273,6 +270,11 @@ class Query(object):
     def count(self, req=None, cached_ids=None, authname=None, tzinfo=None,
               locale=None):
         """Get the number of matching tickets for the present query.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
         """
         sql, args = self.get_sql(req, cached_ids, authname, tzinfo, locale)
         return self._count(sql, args)
@@ -287,6 +289,11 @@ class Query(object):
     def execute(self, req=None, cached_ids=None, authname=None, tzinfo=None,
                 href=None, locale=None):
         """Retrieve the list of matching tickets.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
         """
         if req is not None:
             href = req.href
@@ -315,11 +322,9 @@ class Query(object):
             columns = get_column_names(cursor)
             fields = [self.fields.by_name(column, None) for column in columns]
 
-            column_indices = range(len(columns))
             for row in cursor:
                 result = {}
-                for i in column_indices:
-                    name, field, val = columns[i], fields[i], row[i]
+                for name, field, val in zip(columns, fields, row):
                     if name == 'reporter':
                         val = val or 'anonymous'
                     elif name == 'id':
@@ -327,7 +332,7 @@ class Query(object):
                         if href is not None:
                             result['href'] = href.ticket(val)
                     elif name in self.time_fields:
-                        val = from_utimestamp(long(val)) if val else ''
+                        val = from_utimestamp(long(val)) if val else None
                     elif field and field['type'] == 'checkbox':
                         val = as_bool(val)
                     elif val is None:
@@ -409,11 +414,15 @@ class Query(object):
 
     def get_sql(self, req=None, cached_ids=None, authname=None, tzinfo=None,
                 locale=None):
-        """Return a (sql, params) tuple for the query."""
+        """Return a (sql, params) tuple for the query.
+
+        :since 1.0.17: the `tzinfo` parameter is deprecated and will be
+            removed in version 1.5.1
+        :since 1.0.17: the `locale` parameter is deprecated and will be
+            removed in version 1.5.1
+        """
         if req is not None:
             authname = req.authname
-            tzinfo = req.tz
-            locale = req.locale
         self.get_columns()
 
         # Build the list of actual columns to query
@@ -428,12 +437,14 @@ class Query(object):
         if self.rows:
             add_cols('reporter', *self.rows)
         add_cols('status', 'priority', 'time', 'changetime', self.order)
-        cols.extend([c for c in self.constraint_cols if c not in cols])
+        add_cols(*list(self.constraint_cols))
 
         custom_fields = set(f['name'] for f in self.fields if f.get('custom'))
         list_fields = set(f['name'] for f in self.fields
                                     if f['type'] == 'text' and
                                        f.get('format') == 'list')
+        cols_custom = [k for k in cols if k in custom_fields]
+        use_joins = len(cols_custom) <= 1
         enum_columns = [col for col in ('resolution', 'priority', 'severity',
                                         'type')
                             if col not in custom_fields and
@@ -441,9 +452,6 @@ class Query(object):
         joined_columns = [col for col in ('milestone', 'version')
                               if col not in custom_fields and
                                  col in (self.order, self.group)]
-        # 31 is max of joins in SQLite 32-bit
-        use_joins = (len(set(cols) & custom_fields) +
-                     len(enum_columns) + len(joined_columns)) <= 31
 
         sql = []
         sql.append("SELECT " + ",".join('t.%s AS %s' % (c, c) for c in cols
@@ -454,25 +462,25 @@ class Query(object):
         with self.env.db_query as db:
             if use_joins:
                 # Use LEFT OUTER JOIN for ticket_custom table
-                sql.extend(",%s.value AS %s" % ((db.quote(k),) * 2)
-                           for k in cols if k in custom_fields)
+                sql.extend(",%(qk)s.value AS %(qk)s" % {'qk': db.quote(k)}
+                           for k in cols_custom)
                 sql.append("\nFROM ticket AS t")
                 sql.extend("\n  LEFT OUTER JOIN ticket_custom AS %(qk)s ON "
                            "(%(qk)s.ticket=t.id AND %(qk)s.name='%(k)s')"
-                            % {'qk': db.quote(k), 'k': k}
-                            for k in cols if k in custom_fields)
+                            % {'qk': db.quote(k), 'k': k} for k in cols_custom)
             else:
-                # Use subquery for ticket_custom table
-                sql.extend(",%s AS %s" % ((db.quote(k),) * 2)
-                           for k in cols if k in custom_fields)
-                sql.append('\nFROM (\n  SELECT ')
-                sql.append(','.join('t.%s AS %s' % (c, c)
-                                    for c in cols if c not in custom_fields))
-                sql.extend(",\n  (SELECT c.value FROM ticket_custom c "
-                           "WHERE c.ticket=t.id AND c.name='%s') AS %s"
-                           % (k, db.quote(k))
-                           for k in cols if k in custom_fields)
-                sql.append("\n  FROM ticket AS t) AS t")
+                # Use MAX(CASE ... END) ... GROUP BY ... for ticket_custom
+                # table
+                sql.extend(",c.%(qk)s AS %(qk)s" % {'qk': db.quote(k)}
+                           for k in cols_custom)
+                sql.append("\nFROM ticket AS t"
+                           "\n  LEFT OUTER JOIN (SELECT\n    ticket AS id")
+                sql.extend(",\n    MAX(CASE WHEN name='%s' THEN value END) "
+                           "AS %s" % (k, db.quote(k)) for k in cols_custom)
+                sql.append("\n    FROM ticket_custom AS tc")
+                sql.append("\n    WHERE name IN (%s)" %
+                           ','.join("'%s'" % k for k in cols_custom))
+                sql.append("\n    GROUP BY tc.ticket) AS c ON c.id=t.id")
 
             # Join with the enum table for proper sorting
             sql.extend("\n  LEFT OUTER JOIN enum AS %(col)s ON "
@@ -499,7 +507,7 @@ class Query(object):
                 elif use_joins:
                     col = db.quote(name) + '.value'
                 else:
-                    col = 't.' + db.quote(name)
+                    col = 'c.' + db.quote(name)
                 value = value[len(mode) + neg:]
 
                 if name in self.time_fields:
@@ -526,8 +534,14 @@ class Query(object):
                     else:
                         return None
 
+                def split_words(splittable):
+                    return [w.strip() for wl in
+                            ([x[1:-1]] if x[0] == x[-1] == '"' else x.split()
+                             for x in re.split('("[^"]+")', splittable) if x)
+                            for w in wl]
+
                 if mode == '~' and name in list_fields:
-                    words = value.split()
+                    words = split_words(value)
                     clauses, args = [], []
                     for word in words:
                         cneg = ''
@@ -605,7 +619,7 @@ class Query(object):
                         elif use_joins:
                             col = db.quote(k) + '.value'
                         else:
-                            col = 't.' + db.quote(k)
+                            col = 'c.' + db.quote(k)
                         clauses.append("COALESCE(%s,'') %sIN (%s)"
                                        % (col, 'NOT ' if neg else '',
                                           ','.join('%s' for val in v)))
@@ -651,15 +665,19 @@ class Query(object):
                 elif use_joins:
                     col = db.quote(name) + '.value'
                 else:
-                    col = 't.' + db.quote(name)
+                    col = 'c.' + db.quote(name)
                 desc = ' DESC' if desc else ''
                 # FIXME: This is a somewhat ugly hack.  Can we also have the
                 #        column type for this?  If it's an integer, we do
                 #        first one, if text, we do 'else'
-                if name == 'id' or name in self.time_fields:
-                    sql.append("COALESCE(%s,0)=0%s," % (col, desc))
+                if name in custom_fields:
+                    coalesce_arg = "''"
+                elif name == 'id' or name in self.time_fields:
+                    coalesce_arg = '0'
                 else:
-                    sql.append("COALESCE(%s,'')=''%s," % (col, desc))
+                    coalesce_arg = "''"
+                sql.append("COALESCE(%(col)s,%(arg)s)=%(arg)s%(desc)s," %
+                           {'col': col, 'arg': coalesce_arg, 'desc': desc})
                 if name in enum_columns:
                     # These values must be compared as ints, not as strings
                     sql.append(db.cast(col, 'int') + desc)
@@ -750,7 +768,8 @@ class Query(object):
                 ], key=operator.itemgetter('name'))
                 field['options'].insert(0, {'name': '$USER',
                                             'value': '$USER'})
-            if name == 'milestone':
+            if name == 'milestone' and not field.get('custom'):
+                field = field.copy()
                 milestones = [Milestone(self.env, opt)
                               for opt in field['options']]
                 milestones = [m for m in milestones
@@ -765,13 +784,13 @@ class Query(object):
         groups = {}
         groupsequence = []
         for ticket in tickets:
-            if orig_list:
+            if orig_list and orig_time:
                 # Mark tickets added or changed since the query was first
                 # executed
-                if ticket['time'] > orig_time:
-                    ticket['added'] = True
-                elif ticket['changetime'] > orig_time:
-                    ticket['changed'] = True
+                if ticket['time'] and ticket['time'] > orig_time:
+                    ticket['_added'] = True
+                elif ticket['changetime'] and ticket['changetime'] > orig_time:
+                    ticket['_changed'] = True
             if self.group:
                 group_key = ticket[self.group]
                 groups.setdefault(group_key, []).append(ticket)
@@ -910,7 +929,7 @@ class QueryModule(Component):
 
         constraints = self._get_constraints(req)
         args = req.args
-        if not constraints and 'order' not in req.args:
+        if not constraints:
             # If no constraints are given in the URL, use the default ones.
             if req.authname and req.authname != 'anonymous':
                 qstring = self.default_query
@@ -928,12 +947,12 @@ class QueryModule(Component):
                 constraints = self._get_constraints(arg_list=arg_list)
             else:
                 query = Query.from_string(self.env, qstring)
-                args = {'order': query.order, 'group': query.group,
-                        'col': query.cols, 'max': query.max}
-                if query.desc:
-                    args['desc'] = '1'
-                if query.groupdesc:
-                    args['groupdesc'] = '1'
+                args.setdefault('col', query.cols)
+                args.setdefault('desc', query.desc)
+                args.setdefault('group', query.group)
+                args.setdefault('groupdesc', query.groupdesc)
+                args.setdefault('max', query.max)
+                args.setdefault('order', query.order)
                 constraints = query.constraints
 
             # Substitute $USER, or ensure no field constraints that depend
@@ -967,12 +986,13 @@ class QueryModule(Component):
         group = args.get('group')
         if isinstance(group, (list, tuple)):
             group = group[0] if group else None
+        page = args.get('page')
+        if isinstance(page, (list, tuple)):
+            page = page[0] if page else None
         query = Query(self.env, report_id,
-                      constraints, cols, order, 'desc' in args, group,
-                      'groupdesc' in args, 'verbose' in args,
-                      rows,
-                      args.get('page'),
-                      max)
+                      constraints, cols, order, as_bool(args.get('desc')),
+                      group, as_bool(args.get('groupdesc')),
+                      as_bool(args.get('verbose')), rows, page, max)
 
         if 'update' in req.args:
             # Reset session vars
@@ -1051,7 +1071,7 @@ class QueryModule(Component):
                                     for (start, end) in zip(vals, ends)]
                     if k in remove_constraints:
                         idx = remove_constraints[k]
-                        if idx >= 0:
+                        if 0 <= idx < len(vals):
                             del vals[idx]
                             if not vals:
                                 continue
@@ -1370,7 +1390,10 @@ class TicketQueryMacro(WikiMacroBase):
             query_string += '&'
 
         query_string += '&'.join('%s=%s' % item for item in kwargs.iteritems())
-        query = Query.from_string(self.env, query_string)
+        try:
+            query = Query.from_string(self.env, query_string)
+        except QuerySyntaxError as e:
+            raise MacroError(e)
 
         if format in ('count', 'rawcount'):
             cnt = query.count(req)
@@ -1386,8 +1409,7 @@ class TicketQueryMacro(WikiMacroBase):
         try:
             tickets = query.execute(req)
         except QueryValueError as e:
-            self.log.warn(e)
-            return system_message(_("Error executing TicketQuery macro"), e)
+            raise MacroError(e)
 
         if format == 'table':
             data = query.template_data(formatter.context, tickets,
@@ -1415,6 +1437,7 @@ class TicketQueryMacro(WikiMacroBase):
                 args = {}
                 if q.group:
                     args[q.group] = group_value
+                    q.groupdesc = 0  # avoid groupdesc=1 in query string
                 q.group = extra_args.get('group')
                 if 'status' in extra_args:
                     args['status'] = extra_args['status']
@@ -1455,6 +1478,8 @@ class TicketQueryMacro(WikiMacroBase):
 
             groups = grouped_stats_data(self.env, stats_provider, tickets, by,
                                         per_group_stats_data)
+            if query.groupdesc:
+                groups.reverse()
             data = {
                 'groups': groups, 'grouped_by': by,
                 'summary': _("Ticket completion status for each %(group)s",
