@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2013 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
@@ -169,6 +169,8 @@ class ReportModule(Component):
                 self._do_delete(req, id)
             elif action == 'edit':
                 self._do_save(req, id)
+            if action == 'clear':
+                self._do_clear(req)
             else:
                 raise HTTPBadRequest(_("Invalid request arguments."))
         elif action in ('copy', 'edit', 'new'):
@@ -182,11 +184,6 @@ class ReportModule(Component):
             template, data, content_type = self._render_list(req)
             if content_type:  # i.e. alternate format
                 return template, data, content_type
-            if action == 'clear':
-                if 'query_href' in req.session:
-                    del req.session['query_href']
-                if 'query_tickets' in req.session:
-                    del req.session['query_tickets']
         else:
             template, data, content_type = self._render_view(req, id)
             if content_type:  # i.e. alternate format
@@ -254,6 +251,12 @@ class ReportModule(Component):
             add_notice(req, _("Your changes have been saved."))
         req.redirect(req.href.report(id))
 
+    def _do_clear(self, req):
+        for name in ('query_href', 'query_tickets'):
+            if name in req.session:
+                del req.session[name]
+        req.redirect(req.href.report())
+
     def _render_confirm_delete(self, req, id):
         req.perm(self.realm, id).require('REPORT_DELETE')
 
@@ -292,11 +295,11 @@ class ReportModule(Component):
     def _render_list(self, req):
         """Render the list of available reports."""
         sort = req.args.get('sort', 'report')
-        asc = req.args.getbool('asc', True)
+        asc = req.args.getint('asc', 1, min=0, max=1)
         format = req.args.get('format')
 
         rows = [(report.id, report.title, report.description)
-                for report in Report.select(self.env, sort, asc)
+                for report in Report.select(self.env, sort, bool(asc))
                 if 'REPORT_VIEW' in req.perm(self.realm, report.id)]
 
         if format == 'rss':
@@ -312,8 +315,8 @@ class ReportModule(Component):
                            filename='reports.tsv')
 
         def report_href(**kwargs):
-            return req.href.report(sort=req.args.get('sort'),
-                                   asc=1 if asc else 0, **kwargs)
+            return req.href.report(sort=req.args.get('sort'), asc=asc,
+                                   **kwargs)
 
         add_link(req, 'alternate',
                  auth_link(req, report_href(format='rss')),
@@ -327,7 +330,7 @@ class ReportModule(Component):
                     'REPORT_MODIFY' in req.perm(self.realm, id),
                     'REPORT_DELETE' in req.perm(self.realm, id))
                    for id, title, description in rows]
-        data = {'reports': reports, 'sort': sort, 'asc': 1 if asc else 0}
+        data = {'reports': reports, 'sort': sort, 'asc': asc}
 
         return 'report_list.html', data, None
 
@@ -338,10 +341,6 @@ class ReportModule(Component):
         """Retrieve the report results and pre-process them for rendering."""
         r = Report(self.env, id)
         title, description, sql = r.title, r.description, r.query
-        try:
-            args = self.get_var_args(req)
-        except ValueError as e:
-            raise TracError(_("Report failed: %(error)s", error=e))
 
         # If this is a saved custom query, redirect to the query module
         #
@@ -392,7 +391,8 @@ class ReportModule(Component):
         offset = (page - 1) * limit
 
         sort_col = req.args.get('sort', '')
-        asc = req.args.getbool('asc', True)
+        asc = req.args.getint('asc', 1, min=0, max=1)
+        args = {}
 
         def report_href(**kwargs):
             """Generate links to this report preserving user variables,
@@ -410,23 +410,33 @@ class ReportModule(Component):
 
         data = {'action': 'view',
                 'report': {'id': id, 'resource': report_resource},
-                'context': context,
-                'title': sub_vars(title, args),
-                'description': sub_vars(description or '', args),
+                'context': context, 'title': title, 'description': description,
                 'max': limit, 'args': args, 'show_args_form': False,
                 'message': None, 'paginator': None,
-                'report_href': report_href,
-                }
+                'report_href': report_href}
 
-        res = self.execute_paginated_report(req, id, sql, args, limit, offset)
+        try:
+            args = self.get_var_args(req)
+        except ValueError as e:
+            data['message'] = _("Report failed: %(error)s", error=e)
+            return 'report_view.html', data, None
+        data.update({'args': args, 'title': sub_vars(title, args),
+                     'description': sub_vars(description or '', args)})
 
-        if len(res) == 2:
-            e, sql = res
-            data['message'] = \
-                tag_("Report execution failed: %(error)s %(sql)s",
-                     error=tag.pre(exception_to_unicode(e)),
-                     sql=tag(tag.hr(),
-                             tag.pre(sql, style="white-space: pre")))
+        try:
+            res = self.execute_paginated_report(req, id, sql, args, limit,
+                                                offset)
+        except TracError as e:
+            data['message'] = _("Report failed: %(error)s", error=e)
+        else:
+            if len(res) == 2:
+                e, sql = res
+                data['message'] = \
+                    tag_("Report execution failed: %(error)s %(sql)s",
+                         error=tag.pre(exception_to_unicode(e)),
+                         sql=tag(tag.hr(),
+                                 tag.pre(sql, style="white-space: pre")))
+        if data['message']:
             return 'report_view.html', data, None
 
         cols, results, num_items, missing_args, limit_offset = res
@@ -478,7 +488,7 @@ class ReportModule(Component):
             }
 
             if col == sort_col:
-                header['asc'] = asc
+                header['asc'] = bool(asc)
                 if not paginator and need_reorder:
                     # this dict will have enum values for sorting
                     # and will be used in sortkey(), if non-empty:
@@ -503,7 +513,7 @@ class ReportModule(Component):
                         if isinstance(val, basestring):
                             val = val.lower()
                         return val
-                    results = sorted(results, key=sortkey, reverse=(not asc))
+                    results = sorted(results, key=sortkey, reverse=not asc)
 
             header_group = header_groups[-1]
 
@@ -707,7 +717,7 @@ class ReportModule(Component):
 
             # The ORDER BY columns are inserted
             sort_col = req.args.get('sort', '')
-            asc = req.args.getbool('asc', True)
+            asc = req.args.getint('asc', 1, min=0, max=1)
             self.log.debug("%r %s (%s)", cols, sort_col, '^' if asc else 'v')
             order_cols = []
             if sort_col and sort_col not in cols:
