@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2013 Edgewall Software
+# Copyright (C) 2012-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -14,8 +14,9 @@
 import unittest
 from datetime import timedelta
 
-from trac.perm import DefaultPermissionPolicy, DefaultPermissionStore,\
-                      PermissionCache, PermissionSystem
+from trac.core import Component, implements
+from trac.perm import DefaultPermissionPolicy, DefaultPermissionStore, \
+                      PermissionSystem
 from trac.test import EnvironmentStub, MockRequest
 from trac.ticket import default_workflow, api, web_ui
 from trac.ticket.batch import BatchModifyModule
@@ -35,9 +36,8 @@ class BatchModifyTestCase(unittest.TestCase):
                     web_ui.TicketModule])
         self.env.config.set('trac', 'permission_policies',
                             'DefaultPermissionPolicy')
-        self.req = MockRequest(self.env)
-        self.req.session = {}
-        self.req.perm = PermissionCache(self.env)
+        self.req = MockRequest(self.env, authname='anonymous',
+                               path_info='/query')
 
     def assertCommentAdded(self, ticket_id, comment):
         ticket = Ticket(self.env, int(ticket_id))
@@ -453,6 +453,70 @@ class ProcessRequestTestCase(unittest.TestCase):
         self.assertFieldChanged(1, 'reporter', 'user1')
         self.assertFieldChanged(2, 'reporter', 'user1')
 
+    def test_post_process_request_add_template_data(self):
+        """Template data added by post_process_request."""
+        self._insert_ticket("Ticket 1", status='new')
+        self._insert_ticket("Ticket 2", status='new')
+        req = MockRequest(self.env, path_info='/query')
+        req.session['query_href'] = '/query?status=!closed'
+        batch = BatchModifyModule(self.env)
+        data_in = {'tickets': [{'id': 1}, {'id': 2}]}
+
+        data_out = batch.post_process_request(req, 'query.html', data_in,
+                                              'text/html')[1]
+
+        self.assertTrue(data_out['batch_modify'])
+        self.assertEqual(['leave', 'resolve', 'reassign', 'accept'],
+                         [a[0] for a in data_out['action_controls']])
+
+    def test_actions_added_by_additional_ticket_action_controllers(self):
+        """Actions added by custom ticket action controller.
+
+        Regression test for #12938.
+        """
+        class TestOperation(Component):
+            """TicketActionController that directly provides an action."""
+            implements(api.ITicketActionController)
+
+            def get_ticket_actions(self, req, ticket):
+                return [(0, 'test')]
+
+            def get_all_status(self):
+                return []
+
+            def render_ticket_action_control(self, req, ticket, action):
+                return "test", '', "This is a null action."
+
+            def get_ticket_changes(self, req, ticket, action):
+                return {}
+
+            def apply_action_side_effects(self, req, ticket, action):
+                pass
+
+        self._insert_ticket("Ticket 1", status='new')
+        self._insert_ticket("Ticket 2", status='new')
+        req = MockRequest(self.env, path_info='/query')
+        req.session['query_href'] = '/query?status=!closed'
+        batch = BatchModifyModule(self.env)
+        data_in = {'tickets': [{'id': 1}, {'id': 2}]}
+        self.env.config.set('ticket', 'workflow',
+                            'ConfigurableTicketWorkflow, TestOperation')
+        self.env.enable_component(TestOperation)
+
+        data_out = batch.post_process_request(req, 'query.html', data_in,
+                                              'text/html')[1]
+
+        self.assertEqual(['leave', 'test', 'resolve', 'reassign', 'accept'],
+                         [a[0] for a in data_out['action_controls']])
+
+    def test_post_process_request_error_handling(self):
+        """Exception not raised in post_process_request error handling.
+        """
+        req = MockRequest(self.env, path_info='/query')
+        batch = BatchModifyModule(self.env)
+        self.assertEqual((None, None, None),
+                         batch.post_process_request(req, None, None, None))
+
 
 def test_suite():
     suite = unittest.TestSuite()
