@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2015 Edgewall Software
+# Copyright (C) 2008-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -826,6 +826,15 @@ class TestAdminVersionDetailTime(FunctionalTwillTestCaseSetup):
         tc.find(name + '(<[^>]*>|\\s)*<[^>]* name="default" value="%s"'
                 % name, 's')
 
+        # Empty time value is not automatically populated.
+        tc.follow(name)
+        tc.find('<input type="text" id="releaseddate"')
+        tc.notfind('<input type="text" id="releaseddate"[^>]*value="')
+        tc.submit('save', formname="edit")
+        tc.url(version_admin + '$')
+        tc.find(name + '(<[^>]*>|\\s)*<[^>]* name="default" value="%s"'
+                % name, 's')
+
 
 class TestAdminVersionDetailCancel(FunctionalTwillTestCaseSetup):
     def runTest(self):
