@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2015 Edgewall Software
+# Copyright (C) 2008-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -1761,6 +1761,42 @@ class RegressionTestTicket11996(FunctionalTwillTestCaseSetup):
             self._testenv.set_config('ticket', 'default_milestone', milestone)
 
 
+class RegressionTestTicket12801(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        """Textarea fields with plain format preserve newlines.
+        Test for regression http://trac.edgewall.org/ticket/12801
+        """
+        env = self._testenv.get_trac_environment()
+        try:
+            env.config.set('ticket-custom', 't12801_plain', 'textarea')
+            env.config.set('ticket-custom', 't12801_plain.format', '')
+            env.config.set('ticket-custom', 't12801_wiki', 'textarea')
+            env.config.set('ticket-custom', 't12801_wiki.format', 'wiki')
+            env.config.save()
+
+            tkt = self._tester.create_ticket(
+                info={'description': "//**description field**//\n",
+                      't12801_plain': "- '''plain 1'''\n- ''plain 2''\n",
+                      't12801_wiki': "- ''wiki 1''\n- '''wiki 2'''\n"})
+            tc.find(r'<em><strong>description field</strong></em>')
+            tc.find(r"- '''plain 1'''\s*<br />- ''plain 2''")
+            tc.find(r'<li><em>wiki 1</em>\s*'
+                    r'</li><li><strong>wiki 2</strong>\s*</li>')
+
+            tc.go('%s/query?id=%s&&row=description&row=t12801_plain&'
+                  'row=t12801_wiki' % (self._tester.url, tkt))
+            tc.find(r'<em><strong>description field</strong></em>')
+            tc.find(r"- '''plain 1'''\s*<br />- ''plain 2''")
+            tc.find(r'<li><em>wiki 1</em>\s*'
+                    r'</li><li><strong>wiki 2</strong>\s*</li>')
+
+        finally:
+            for name in ('t12801_plain', 't12801_plain.format',
+                         't12801_wiki', 't12801_wiki.format'):
+                env.config.remove('ticket-custom', name)
+            env.config.save()
+
+
 def functionalSuite(suite=None):
     if not suite:
         import trac.tests.functional
@@ -1835,6 +1871,7 @@ def functionalSuite(suite=None):
     suite.addTest(RegressionTestTicket11176())
     suite.addTest(RegressionTestTicket11590())
     suite.addTest(RegressionTestTicket11996())
+    suite.addTest(RegressionTestTicket12801())
 
     return suite
 
