@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
@@ -31,7 +31,7 @@ from trac.test import EnvironmentStub, MockRequest
 from trac.tests.notification import SMTP_TEST_PORT, SMTPThreadedServer, \
                                     parse_smtp_message
 from trac.ticket.model import Ticket
-from trac.ticket.notification import BatchTicketNotifyEmail, \
+from trac.ticket.notification import BatchTicketNotifyEmail, Subscription, \
                                      TicketChangeEvent, TicketNotifyEmail
 from trac.ticket.web_ui import TicketModule
 from trac.util.datefmt import datetime_now, utc
@@ -207,7 +207,7 @@ class RecipientTestCase(unittest.TestCase):
             now = datetime_now(utc)
             ticket.save_changes('joe@example.org', 'this is my comment',
                                 when=now)
-            notify_ticket_created(self.env, ticket)
+            notify_ticket_changed(self.env, ticket)
             recipients = notifysuite.smtpd.get_recipients()
             if enabled:
                 self.assertEqual(1, len(recipients))
@@ -229,7 +229,7 @@ class RecipientTestCase(unittest.TestCase):
             now = datetime_now(utc)
             ticket.save_changes('joe@example.org', 'this is my comment',
                                 when=now)
-            notify_ticket_created(self.env, ticket)
+            notify_ticket_changed(self.env, ticket)
             recipients = notifysuite.smtpd.get_recipients()
             if enabled:
                 self.assertEqual(1, len(recipients))
@@ -240,6 +240,49 @@ class RecipientTestCase(unittest.TestCase):
         for enable in False, True:
             _test_reporter(enable)
 
+    def test_notify_new_tickets(self):
+        """Notification to NewTicketSubscribers."""
+        def _test_new_ticket():
+            ticket = Ticket(self.env)
+            ticket['reporter'] = 'user4'
+            ticket['owner'] = 'user5'
+            ticket['summary'] = 'New Ticket Subscribers'
+            ticket.insert()
+            notify_ticket_created(self.env, ticket)
+            return ticket
+
+        def _add_new_ticket_subscriber(sid, authenticated):
+            Subscription.add(self.env, {
+                'sid': sid, 'authenticated': authenticated,
+                'distributor': 'email', 'format': None, 'adverb': 'always',
+                'class': 'NewTicketSubscriber'
+            })
+
+        _test_new_ticket()
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(0, len(recipients))
+
+        self.env.insert_users([
+            ('user1', 'User One', 'joe@example.org', 1),
+            ('user2', 'User Two', 'bar@example.org', 1),
+            ('58bd3a', 'User Three', 'jim@example.org', 0),
+        ])
+        _add_new_ticket_subscriber('user1', 1)
+        _add_new_ticket_subscriber('58bd3a', 0)
+
+        ticket = _test_new_ticket()
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(2, len(recipients))
+        self.assertIn('joe@example.org', recipients)
+        self.assertIn('jim@example.org', recipients)
+
+        ticket.save_changes('user4', 'this is my comment',
+                            when=datetime_now(utc))
+        notify_ticket_changed(self.env, ticket)
+
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(0, len(recipients))
+
     def test_no_duplicates(self):
         """Email addresses should be found only once in the recipient list."""
         self.env.config.set('notification', 'smtp_always_cc',
@@ -261,7 +304,7 @@ class RecipientTestCase(unittest.TestCase):
         ticket = Ticket(self.env)
         ticket['reporter'] = '"Joe" <joe.user@example.com>'
         ticket['owner'] = 'Joe <joe.user@example.net>'
-        ticket['cc'] = 'Joe < joe.user@example.org >'
+        ticket['cc'] = u' \u00a0 Jóe \u3000 < joe.user@example.org > \u00a0 '
         ticket['summary'] = 'Long form'
         ticket.insert()
         notify_ticket_created(self.env, ticket)
@@ -322,6 +365,7 @@ class NotificationTestCase(unittest.TestCase):
         self.assertEqual('<073.8a48f9c2ab2dc64e820e391f8f784a04@localhost>',
                          headers['Message-ID'])
         self.assertIn('From', headers)
+        self.assertIn('\n-- \nTicket URL: <', body)
 
     def test_date(self):
         """Date format compliance (RFC822)
@@ -395,71 +439,82 @@ class NotificationTestCase(unittest.TestCase):
 
     def test_short_login(self):
         """Email addresses without a FQDN"""
-        def _test_short_login(enabled):
+        def _test_short_login(use_short_addr, username, address):
             config_subscriber(self.env, reporter=True)
             ticket = Ticket(self.env)
-            ticket['reporter'] = 'joeuser'
+            ticket['reporter'] = username
             ticket['summary'] = 'This is a summary'
             ticket.insert()
             # Be sure that at least one email address is valid, so that we
             # send a notification even if other addresses are not valid
             self.env.config.set('notification', 'smtp_always_cc',
-                                'joe.bar@example.net')
-            self.env.config.set('notification', 'use_short_addr', enabled)
+                                'joe.bar@example.net, john')
+            self.env.config.set('notification', 'use_short_addr',
+                                'enabled' if use_short_addr else 'disabled')
             notify_ticket_created(self.env, ticket)
             message = notifysuite.smtpd.get_message()
+            recipients = set(notifysuite.smtpd.get_recipients())
             headers, body = parse_smtp_message(message)
             # Msg should always have a 'To' field
             self.assertEqual('undisclosed-recipients: ;', headers['To'])
             # Msg should have a 'Cc' field
             self.assertIn('Cc', headers)
-            cclist = [addr.strip() for addr in headers['Cc'].split(',')]
-            if enabled:
+            cclist = set(addr.strip() for addr in headers['Cc'].split(','))
+            if use_short_addr:
                 # Msg should be delivered to the reporter
-                self.assertIn(ticket['reporter'], cclist)
+                self.assertEqual(set([address, 'joe.bar@example.net',
+                                      'john']), cclist)
+                self.assertEqual(set([address, 'joe.bar@example.net',
+                                      'john']), recipients)
             else:
-                # Msg should not be delivered to joeuser
-                self.assertNotIn(ticket['reporter'], cclist)
-            # Msg should still be delivered to the always_cc list
-            self.assertIn(self.env.config.get('notification',
-                                              'smtp_always_cc'), cclist)
+                # Msg should not be delivered to the reporter
+                self.assertEqual(set(['joe.bar@example.net']), cclist)
+                self.assertEqual(set(['joe.bar@example.net']), recipients)
+
         # Validate with and without the short addr option enabled
-        for enable in False, True:
-            _test_short_login(enable)
+        self.env.insert_users([('bar', 'Bar User', ''),
+                               ('qux', 'Qux User', 'qux-mail')])
+        for use_short_addr in (False, True):
+            _test_short_login(use_short_addr, 'foo', 'foo')
+            _test_short_login(use_short_addr, 'bar', 'bar')
+            _test_short_login(use_short_addr, 'qux', 'qux-mail')
 
     def test_default_domain(self):
         """Default domain name"""
-        def _test_default_domain(enabled):
+        def _test_default_domain(enable):
             config_subscriber(self.env)
             self.env.config.set('notification', 'smtp_always_cc', '')
             ticket = Ticket(self.env)
-            ticket['cc'] = 'joenodom, joewithdom@example.com'
+            ticket['cc'] = 'joenodom, foo, bar, qux, joewithdom@example.com'
             ticket['summary'] = 'This is a summary'
             ticket.insert()
             # Be sure that at least one email address is valid, so that we
             # send a notification even if other addresses are not valid
             self.env.config.set('notification', 'smtp_always_cc',
                                 'joe.bar@example.net')
-            if enabled:
-                self.env.config.set('notification', 'smtp_default_domain',
-                                    'example.org')
+            self.env.config.set('notification', 'smtp_default_domain',
+                                'example.org' if enable else '')
             notify_ticket_created(self.env, ticket)
             message = notifysuite.smtpd.get_message()
             headers, body = parse_smtp_message(message)
             # Msg should always have a 'Cc' field
             self.assertIn('Cc', headers)
-            cclist = [addr.strip() for addr in headers['Cc'].split(',')]
-            self.assertIn('joewithdom@example.com', cclist)
-            self.assertIn('joe.bar@example.net', cclist)
-            if enabled:
-                self.assertEqual(3, len(cclist))
-                self.assertIn('joenodom@example.org', cclist)
+            cclist = set(addr.strip() for addr in headers['Cc'].split(','))
+            if enable:
+                self.assertEqual(set(['joenodom@example.org',
+                                      'foo@example.org',
+                                      'bar@example.org',
+                                      'qux-mail@example.org',
+                                      'joewithdom@example.com',
+                                      'joe.bar@example.net']), cclist)
             else:
-                self.assertEqual(2, len(cclist))
-                self.assertNotIn('joenodom', cclist)
+                self.assertEqual(set(['joewithdom@example.com',
+                                      'joe.bar@example.net']), cclist)
 
         # Validate with and without a default domain
-        for enable in False, True:
+        self.env.insert_users([('bar', 'Bar User', ''),
+                               ('qux', 'Qux User', 'qux-mail')])
+        for enable in (False, True):
             _test_default_domain(enable)
 
     def test_email_map(self):
@@ -1691,6 +1746,7 @@ class BatchTicketNotificationTestCase(unittest.TestCase):
         self.assertIn('Batch modification to #3, #10, #4, #11, #5, #12, #6, '
                       '#13, #7, #14, #1, #2, #8, #9 by author@example.org:',
                       body)
+        self.assertIn('-- ', body)
         self.assertIn('Tickets URL: <http://example.org/trac.cgi/query?id=3'
                       '%2C10%2C4%2C11%2C5%2C12%2C6%2C13%2C7%2C14%2C1%2C2%2C8'
                       '%2C9>', body)
