@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2013 Edgewall Software
+# Copyright (C) 2004-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,7 +13,6 @@
 
 from datetime import datetime, timedelta
 import difflib
-import re
 import unittest
 
 import trac.tests.compat
@@ -40,7 +39,11 @@ class QueryTestCase(unittest.TestCase):
         in creating a useful diff between two SQL statements."""
         pretty = []
         for line in sql.split('\n'):
-            pretty.extend(['%s,\n' % x for x in line.split(',')])
+            if line.startswith('SELECT '):
+                pretty.extend(x + '\n'
+                              for x in ',\n'.join(line.split(',')).split('\n'))
+            else:
+                pretty.append(line + '\n')
         return pretty
 
     def assertEqualSQL(self, sql, correct_sql):
@@ -481,64 +484,6 @@ ORDER BY COALESCE(t.id,0)=0,t.id""")
         self.assertEqual(1, len(tickets))
         self.assertEqual(ticket.id, tickets[0]['id'])
 
-    def _get_join_tables(self, sql):
-        return sorted(match.group(1)
-                      for match in re.finditer(r'\bLEFT OUTER JOIN (\w+)',
-                                               sql))
-
-    def test_query_using_joins(self):
-        fields = ['col_%02d' % i for i in xrange(100)]
-        for f in fields:
-            self.env.config.set('ticket-custom', f, 'text')
-        with self.env.db_transaction as db:
-            ticket = Ticket(self.env)
-            ticket['reporter'] = 'joe'
-            ticket['summary'] = 'Foo'
-            for idx, f in enumerate(fields):
-                ticket[f] = '%d.%s' % (idx, f)
-            ticket.insert()
-
-        query = Query.from_string(
-            self.env, 'col_12=12.col_12&' +
-                      'order=resolution&group=severity&col=id&col=summary' +
-                      ''.join('&col=col_%02d' % idx for idx in xrange(28)))
-        sql, args = query.get_sql()
-        self.assertEqual(['enum'] * 3 + ['ticket_custom'] * 28,
-                         self._get_join_tables(sql))
-        tickets = query.execute(self.req)
-        self.assertEqual(1, len(tickets))
-
-        query = Query.from_string(
-            self.env, 'col_12=12.col_12&' +
-                      'order=milestone&group=version&col=id&col=summary' +
-                      ''.join('&col=col_%02d' % idx for idx in xrange(28)))
-        sql, args = query.get_sql()
-        self.assertEqual(['enum', 'milestone'] + ['ticket_custom'] * 28 +
-                         ['version'],
-                         self._get_join_tables(sql))
-        tickets = query.execute(self.req)
-        self.assertEqual(1, len(tickets))
-
-        query = Query.from_string(
-            self.env, 'col_12=12.col_12&' +
-                      'order=resolution&group=severity&col=id&col=summary' +
-                      ''.join('&col=col_%02d' % idx for idx in xrange(29)))
-        sql, args = query.get_sql()
-        self.assertEqual(['enum'] * 3, self._get_join_tables(sql))
-        tickets = query.execute(self.req)
-        self.assertEqual(ticket.id, tickets[0]['id'])
-        self.assertEqual(1, len(tickets))
-
-        query = Query.from_string(
-            self.env, 'col_12=12.col_12&' +
-                      'order=milestone&group=version&col=id&col=summary' +
-                      ''.join('&col=col_%02d' % idx for idx in xrange(29)))
-        sql, args = query.get_sql()
-        self.assertEqual(['enum', 'milestone', 'version'],
-                         self._get_join_tables(sql))
-        tickets = query.execute(self.req)
-        self.assertEqual(1, len(tickets))
-
     def test_too_many_custom_fields(self):
         fields = ['col_%02d' % i for i in xrange(100)]
         for f in fields:
@@ -852,6 +797,22 @@ ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': like})
                           'foo bar baz', ''],
                          [t['keywords'] for t in tickets])
 
+    def test_constrained_by_keyword_phrase(self):
+        query = Query.from_string(self.env, 'keywords~="bar baz" -foo',
+                                  order='id')
+        sql, args = query.get_sql()
+        with self.env.db_query as db:
+            like = db.like()
+        self.assertEqualSQL(sql,
+"""SELECT t.id AS id,t.summary AS summary,t.keywords AS keywords,t.owner AS owner,t.type AS type,t.status AS status,t.priority AS priority,t.time AS time,t.changetime AS changetime,priority.value AS _priority_value
+FROM ticket AS t
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
+WHERE (((COALESCE(t.keywords,'') %(like)s AND COALESCE(t.keywords,'') NOT %(like)s)))
+ORDER BY COALESCE(t.id,0)=0,t.id""" % {'like': like})
+        self.assertEqual(['%bar baz%', '%foo%'], args)
+        tickets = query.execute(self.req)
+        self.assertEqual(['bar baz'], [t['keywords'] for t in tickets])
+
     def test_constrained_by_milestone_or_version(self):
         query = Query.from_string(self.env, 'milestone=milestone1&or&version=version1', order='id')
         sql, args = query.get_sql()
@@ -1050,69 +1011,20 @@ WHERE ((COALESCE(t.status,'')!=%%s) AND COALESCE(%(version)s.value,'') IN (%%s,%
 ORDER BY COALESCE(%(version)s.value,'')='',%(version)s.value,t.id""" % quoted)
         self.assertEqual(['closed', 'foo', 'blah'], args)
 
-    def test_without_enums_with_many_custom_fields(self):
-        ncols = 32 - 5
-        columns = ('priority', 'resolution', 'type', 'milestone', 'version')
-        for name in columns:
-            self.env.config.set('ticket-custom', name, 'text')
-        for idx in xrange(ncols):
-            self.env.config.set('ticket-custom', 'col_%02d' % idx, 'text')
-        with self.env.db_transaction as db:
-            db("DELETE FROM enum")
-            db("DELETE FROM milestone")
-            db("DELETE FROM version")
-        tktsys = TicketSystem(self.env)
-        tktsys.reset_ticket_fields()
-        del tktsys.custom_fields
-        quoted = {}
-        with self.env.db_transaction as db:
-            for value in ('foo', 'bar', 'baz', 'blah'):
-                t = Ticket(self.env)
-                t['reporter'] = 'joe'
-                t['summary'] = 'Summary "%s"' % value
-                for name in columns:
-                    t[name] = '%s-%s' % (value, name)
-                for idx in xrange(ncols):
-                    t['col_%02d' % idx] = 'v'
-                t.insert()
-            for name in columns:
-                quoted[name] = db.quote(name)
-            for idx in xrange(ncols):
-                name = 'col_%02d' % idx
-                quoted[name] = db.quote(name)
-
-        query = Query.from_string(self.env,
-            'status!=closed&'
-            'priority=foo-priority&priority=blah-priority&'
-            'resolution=foo-resolution&resolution=blah-resolution&'
-            'type=foo-type&type=blah-type&'
-            'milestone=foo-milestone&milestone=blah-milestone&'
-            'version=foo-version&version=blah-version&'
-            'col=id&col=summary&col=priority&col=resolution&col=type&'
-            'col=milestone&col=version' +
-            (''.join('&col=col_%02d' % idx for idx in xrange(ncols))) +
-            '&order=resolution')
+    def test_invalid_id_custom_field(self):
+        self.env.config.set('ticket-custom', 'id', 'text')
+        ticket = Ticket(self.env)
+        ticket.populate({'summary': 'test_invalid_id_custom_field',
+                         'reporter': 'anonymous', 'status': 'new',
+                         'id': 'blah'})
+        ticket.insert()
+        query = Query.from_string(
+            self.env, 'summary=test_invalid_id_custom_field&col=id')
         tickets = query.execute(self.req)
-        self.assertEqual(['Summary "blah"', 'Summary "foo"'],
-                         [t['summary'] for t in tickets])
-        sql, args = query.get_sql(req=self.req)
-        self.assertEqual(['blah-milestone', 'blah-priority', 'blah-resolution',
-                          'blah-type', 'blah-version', 'closed',
-                          'foo-milestone', 'foo-priority', 'foo-resolution',
-                          'foo-type', 'foo-version'], sorted(args))
-        for col in ('priority', 'resolution', 'type', 'milestone', 'version'):
-            self.assertIn(" (SELECT c.value FROM ticket_custom c WHERE "
-                          "c.ticket=t.id AND c.name='%s') AS %s" %
-                          (col, quoted[col]), sql)
-            self.assertNotIn(' LEFT OUTER JOIN ticket_custom AS %s ON ' %
-                             quoted[col], sql)
-        for idx in xrange(ncols):
-            col = 'col_%02d' % idx
-            self.assertIn(" (SELECT c.value FROM ticket_custom c WHERE "
-                          "c.ticket=t.id AND c.name='%s') AS %s" %
-                          (col, quoted[col]), sql)
-            self.assertNotIn(' LEFT OUTER JOIN ticket_custom AS %s ON ' %
-                             quoted[col], sql)
+        self.assertEqual(ticket.id, tickets[0]['id'])
+        self.assertEqual('new', tickets[0]['status'])
+        self.assertEqual('test_invalid_id_custom_field', tickets[0]['summary'])
+        self.assertEqual(1, len(tickets))
 
     def test_csv_cols_are_labels(self):
         self.env.config.set('ticket-custom', 'custom1', 'text')
@@ -1131,6 +1043,63 @@ ORDER BY COALESCE(%(version)s.value,'')='',%(version)s.value,t.id""" % quoted)
                          u'1,joe@example.org,milestone1,val1\r\n',
                          content.decode('utf-8'))
 
+    def test_columns_in_ticket_custom_as_custom_field(self):
+        fields = ('ticket', 'name', 'value')
+        for field in fields:
+            self.env.config.set('ticket-custom', field, 'text')
+        tktids = []
+        with self.env.db_transaction as db:
+            for idx in xrange(3):
+                ticket = Ticket(self.env)
+                ticket.populate({'summary': 'test_ticket_custom_field',
+                                 'reporter': 'anonymous', 'status': 'new',
+                                 'ticket': 'ticket-%d' % idx,
+                                 'name': 'name-%d' % idx,
+                                 'value': 'value-%d' % idx})
+                tktid = ticket.insert()
+                tktids.append(tktid)
+                if idx == 1:
+                    db("""DELETE FROM ticket_custom
+                          WHERE ticket=%s AND name='name'""", (tktid,))
+                elif idx == 2:
+                    db("""DELETE FROM ticket_custom
+                          WHERE ticket=%s AND name='value'""", (tktid,))
+            quoted = dict((field, db.quote(field)) for field in fields)
+            quoted['like'] = db.like()
+        query = Query.from_string(self.env, 'ticket=^ticket-&order=ticket&'
+                                            'col=summary&col=ticket&col=name&'
+                                            'col=value')
+        sql, args = query.get_sql(self.req)
+        self.assertEqualSQL(sql, """\
+SELECT t.id AS id,t.summary AS summary,t.status AS status,\
+t.priority AS priority,t.time AS time,t.changetime AS changetime,\
+priority.value AS _priority_value,\
+c.%(ticket)s AS %(ticket)s,c.%(name)s AS %(name)s,c.%(value)s AS %(value)s
+FROM ticket AS t
+  LEFT OUTER JOIN (SELECT
+    ticket AS id,
+    MAX(CASE WHEN name='ticket' THEN value END) AS %(ticket)s,
+    MAX(CASE WHEN name='name' THEN value END) AS %(name)s,
+    MAX(CASE WHEN name='value' THEN value END) AS %(value)s
+    FROM ticket_custom AS tc
+    WHERE name IN ('ticket','name','value')
+    GROUP BY tc.ticket) AS c ON c.id=t.id
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
+WHERE ((COALESCE(c.%(ticket)s,'') %(like)s))
+ORDER BY COALESCE(c.%(ticket)s,'')='',c.%(ticket)s,t.id""" % quoted)
+        tickets = query.execute(self.req)
+        self.assertEqual(tktids, [t['id'] for t in tickets])
+        self.assertEqual(set(['new']), set(t['status'] for t in tickets))
+        self.assertEqual(['ticket-0', 'ticket-1', 'ticket-2'],
+                         sorted(t['ticket'] for t in tickets))
+        self.assertEqual(['', 'name-0', 'name-2'],
+                         sorted(t['name'] for t in tickets))
+        self.assertEqual(['', 'value-0', 'value-1'],
+                         sorted(t['value'] for t in tickets))
+        self.assertEqual(set(['test_ticket_custom_field']),
+                         set(t['summary'] for t in tickets))
+        self.assertEqual(3, len(tickets))
+
     def test_csv_escape(self):
         query = Mock(get_columns=lambda: ['id', 'col1'],
                      execute=lambda r: [{'id': 1,
@@ -1178,6 +1147,127 @@ ORDER BY COALESCE(%(version)s.value,'')='',%(version)s.value,t.id""" % quoted)
         data = query.template_data(context, tickets)
         self.assertEqual(['$USER'], data['clauses'][0]['owner']['values'])
 
+    def test_properties_script_data(self):
+        req = MockRequest(self.env, path_info='/query')
+        mod = QueryModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        template, data, content_type = mod.process_request(req)
+        prop = req.chrome['script_data']['properties']['milestone']
+        self.assertEqual('select', prop['type'])
+        self.assertEqual('Milestone', prop['label'])
+        self.assertEqual([], prop['options'])
+        self.assertEqual([{'label': 'Open (by due date)',
+                           'options': ['milestone1', 'milestone2']},
+                          {'label': 'Open (no due date)',
+                           'options': ['milestone3', 'milestone4']},
+                          {'label': 'Closed', 'options': []}],
+                         prop['optgroups'])
+
+    def test_properties_script_data_with_no_milestones(self):
+        self.env.db_transaction("DELETE FROM milestone")
+        self.env.config.set('ticket-custom', 'milestone', 'text')
+        req = MockRequest(self.env, path_info='/query')
+        mod = QueryModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        template, data, content_type = mod.process_request(req)
+        prop = req.chrome['script_data']['properties']['milestone']
+        self.assertEqual({'label': 'Milestone', 'type': 'text',
+                          'format': 'plain'}, prop)
+
+    def test_null_time_and_changetime_with_saved_query_tickets(self):
+        with self.env.db_transaction as db:
+            n = self.n_tickets / 2
+            db("UPDATE ticket SET time=NULL WHERE id<%s", (n,))
+            db("UPDATE ticket SET changetime=NULL WHERE id>%s", (n,))
+        req = MockRequest(self.env, path_info='/query', args={'id': '!0'})
+        mod = QueryModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        mod.process_request(req)
+        self.assertNotEqual('', req.session['query_tickets'])
+        mod.process_request(req)  # TypeError not raised (#12029)
+
+    def test_time_fields(self):
+        when = datetime(2017, 11, 9, 12, 56, 34, 654321, utc)
+        due = datetime(2017, 9, 18, 12, 34, 56, 876543, utc)
+        self.env.config.set('ticket-custom', 'due', 'time')
+        t1 = Ticket(self.env, 1)
+        t1['due'] = due
+        t1.save_changes(when=when)
+        t2 = Ticket(self.env, 2)
+        t2['due'] = ''  # clear the field
+        t2.save_changes(when=when + timedelta(seconds=1))
+        req = MockRequest(self.env, path_info='/query',
+                          args={'id': '1-3', 'order': 'id', 'col': 'due'})
+        mod = QueryModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        data = mod.process_request(req)[1]
+        tickets = data['tickets']
+
+        t1 = tickets[0]
+        self.assertEqual('2008-07-01T12:34:56.987654+00:00',
+                         t1['time'].isoformat())
+        self.assertEqual('2017-11-09T12:56:34.654321+00:00',
+                         t1['changetime'].isoformat())
+        self.assertEqual('2017-09-18T12:34:56.876543+00:00',
+                         t1['due'].isoformat())
+
+        t2 = tickets[1]
+        self.assertEqual('2008-07-11T12:34:56.987654+00:00',
+                         t2['time'].isoformat())
+        self.assertEqual('2017-11-09T12:56:35.654321+00:00',
+                         t2['changetime'].isoformat())
+        self.assertEqual(None, t2['due'])
+
+        t3 = tickets[2]
+        self.assertEqual('2008-07-21T12:34:56.987654+00:00',
+                         t3['time'].isoformat())
+        self.assertEqual('2008-07-22T12:34:56.987654+00:00',
+                         t3['changetime'].isoformat())
+        self.assertEqual(None, t3['due'])
+
+    def test_time_fields_order(self):
+        dt = datetime(2018, 4, 25, 12, 34, 56, 987654, utc)
+        self.env.config.set('ticket-custom', 'due', 'time')
+        tkt = Ticket(self.env, 2)
+        tkt['due'] = dt
+        tkt.save_changes(when=dt)
+
+        with self.env.db_query as db:
+            quoted = {'due': db.quote('due')}
+        query = Query.from_string(self.env, 'col=due&order=due')
+        sql, args = query.get_sql()
+        self.assertEqualSQL(sql, """\
+SELECT t.id AS id,t.status AS status,t.priority AS priority,t.time AS time,\
+t.changetime AS changetime,priority.value AS _priority_value,\
+%(due)s.value AS %(due)s
+FROM ticket AS t
+  LEFT OUTER JOIN ticket_custom AS %(due)s ON (%(due)s.ticket=t.id AND %(due)s.name='due')
+  LEFT OUTER JOIN enum AS priority ON (priority.type='priority' AND priority.name=t.priority)
+ORDER BY COALESCE(%(due)s.value,'')='',%(due)s.value,t.id""" % quoted)
+
+        def process_request(req):
+            mod = QueryModule(self.env)
+            self.assertTrue(mod.match_request(req))
+            return mod.process_request(req)
+
+        req = MockRequest(self.env, path_info='/query',
+                          args={'id': '1-3', 'order': 'due'})
+        data = process_request(req)[1]
+        tickets = data['tickets']
+        self.assertEqual([2, 1, 3], [t['id'] for t in tickets])
+
+        req = MockRequest(self.env, path_info='/query',
+                          args={'id': '1-3', 'order': 'time'})
+        data = process_request(req)[1]
+        tickets = data['tickets']
+        self.assertEqual([1, 2, 3], [t['id'] for t in tickets])
+
+        req = MockRequest(self.env, path_info='/query',
+                          args={'id': '1-3', 'order': 'changetime'})
+        data = process_request(req)[1]
+        tickets = data['tickets']
+        self.assertEqual([1, 3, 2], [t['id'] for t in tickets])
+
 
 class QueryLinksTestCase(unittest.TestCase):
 
@@ -1242,6 +1332,31 @@ class QueryLinksTestCase(unittest.TestCase):
                          set(value for value, tickets in data['groups']))
         self.assertEqual(2, len(data['groups']))
 
+    def test_max_only_argument(self):
+        for _ in range(0, 4):
+            self._insert_ticket(status='new')
+        data = self._process_request('max=3')
+        self.assertEqual(3, data['query'].max)
+        self.assertEqual(3, len(data['tickets']))
+
+    def test_parameter_overrides_default_query(self):
+        self.env.config.set('query', 'default_anonymous_query',
+                            'status!=closed&order=milestone'
+                            'cols=id&cols=summary&cols=status&cols=owner&'
+                            'group=milestone&max=4&groupdesc=0&desc=1')
+
+        data = self._process_request(
+            'order=status&col=id&col=summary&col=status&col=type&'
+            'group=status&max=3&groupdesc=1&desc=0')
+
+        self.assertEqual('status', data['query'].order)
+        self.assertEqual(['id', 'summary', 'status', 'type'],
+                         data['query'].cols)
+        self.assertEqual('status', data['query'].group)
+        self.assertEqual(3, data['query'].max)
+        self.assertTrue(data['query'].groupdesc)
+        self.assertFalse(data['query'].desc)
+
 
 class TicketQueryMacroTestCase(unittest.TestCase):
 
@@ -1317,9 +1432,15 @@ QUERY_TEST_CASES = u"""
 [[TicketQuery(created=...)]]
 ------------------------------
 <p>
-</p><div class="system-message"><strong>Error executing TicketQuery macro</strong><pre>Invalid query constraint value</pre></div><p>
+<div class="system-message"><strong>Macro TicketQuery(created=...) failed</strong><pre>Invalid query constraint value</pre></div>
 </p>
 ------------------------------
+============================== TicketQuery(keywords~=x&y)
+[[TicketQuery(keywords~=x&y)]]
+------------------------------
+<p>
+<div class="system-message"><strong>Macro TicketQuery(keywords~=x&amp;y) failed</strong><pre>Query filter requires field and constraints separated by a "="</pre></div>
+</p>
 ============================== TicketQuery(format=progress)
 [[TicketQuery(format=progress)]]
 ------------------------------
@@ -1466,6 +1587,65 @@ QUERY_TEST_CASES = u"""
 
 
 
+      </td>
+    </tr>
+  </table>
+</div><p>
+</p>
+------------------------------
+============================== TicketQuery(format=progress, group=project, groupdesc=1)
+[[TicketQuery(format=progress, group=project, groupdesc=1)]]
+------------------------------
+<p>
+</p><div class="trac-groupprogress">
+  <table xmlns="http://www.w3.org/1999/xhtml" summary="Ticket completion status for each project">
+    <tr>
+      <th scope="row">
+
+
+        <a href="/query?project=xmas&amp;max=0&amp;order=id">xmas</a>
+      </th>
+      <td>
+
+
+  <table class="progress" style="width: 80%">
+    <tr>
+      <td class="closed" style="width: 50%">
+        <a href="/query?project=xmas&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="1/2 closed"></a>
+      </td><td class="open" style="width: 50%">
+        <a href="/query?project=xmas&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/2 active"></a>
+      </td>
+    </tr>
+  </table>
+
+  <p class="percent">1 / 2</p>
+
+
+
+      </td>
+    </tr><tr>
+      <th scope="row">
+        <i><a href="/query?project=&amp;max=0&amp;order=id">(none)</a></i>
+
+
+      </th>
+      <td>
+
+
+  <table class="progress" style="width: 40%">
+    <tr>
+      <td class="closed" style="display: none">
+        <a href="/query?project=&amp;status=closed&amp;group=resolution&amp;max=0&amp;order=time" title="0/1 closed"></a>
+      </td><td class="open" style="width: 100%">
+        <a href="/query?project=&amp;status=assigned&amp;status=new&amp;status=accepted&amp;status=reopened&amp;max=0&amp;order=id" title="1/1 active"></a>
+      </td>
+    </tr>
+  </table>
+
+  <p class="percent">0 / 1</p>
+
+
+
       </td>
     </tr>
   </table>
