@@ -14,6 +14,7 @@
 from datetime import datetime, timedelta
 import unittest
 
+from trac.core import TracError
 from trac.perm import PermissionSystem
 from trac.resource import ResourceNotFound
 from trac.test import EnvironmentStub, MockRequest
@@ -53,7 +54,8 @@ class TicketModuleTestCase(unittest.TestCase):
         ticket = Ticket(self.env)
         for k, v in kw.items():
             ticket[k] = v
-        return ticket.insert()
+        ticket.insert()
+        return ticket
 
     def test_ticket_module_as_default_handler(self):
         """The New Ticket mainnav entry is active when TicketModule is the
@@ -74,10 +76,10 @@ class TicketModuleTestCase(unittest.TestCase):
         """Full name of reporter and owner are used in ticket properties."""
         self.env.insert_users([('user1', 'User One', ''),
                                ('user2', 'User Two', '')])
-        tkt_id = self._insert_ticket(reporter='user1', owner='user2')
+        ticket = self._insert_ticket(reporter='user1', owner='user2')
         PermissionSystem(self.env).grant_permission('user2', 'TICKET_VIEW')
         req = MockRequest(self.env, authname='user2', method='GET',
-                          args={'id': tkt_id, 'replyto': '1'})
+                          args={'id': ticket.id, 'replyto': '1'})
 
         data = self.ticket_module.process_request(req)[1]
 
@@ -231,6 +233,14 @@ class TicketModuleTestCase(unittest.TestCase):
         self.assertRaises(ResourceNotFound,
                           self.ticket_module.process_request, req)
 
+    def test_edit_comment_cnum_missing_raises(self):
+        ticket = self._insert_ticket()
+        req = MockRequest(
+            self.env, method='POST', path_info='/ticket/%d' % ticket.id,
+            args={'edit_comment': 'Submit changes', 'cnum_edit': '42'})
+        self.assertTrue(self.ticket_module.match_request(req))
+        self.assertRaises(TracError, self.ticket_module.process_request, req)
+
     def _test_template_data_for_time_field(self, req, value, expected, format):
         self.env.config.set('ticket-custom', 'timefield', 'time')
         if format:
@@ -469,6 +479,39 @@ class TicketModuleTestCase(unittest.TestCase):
     def test_newticket_with_version_as_custom_field(self):
         self._test_newticket_with_enum_as_custom_field('version')
 
+    def test_add_comment_requires_ticket_append(self):
+        """Adding a ticket comment requires TICKET_APPEND."""
+        ps = PermissionSystem(self.env)
+        ps.grant_permission('user1', 'TICKET_VIEW')
+        ps.grant_permission('user1', 'TICKET_APPEND')
+        ps.grant_permission('user2', 'TICKET_VIEW')
+        ps.grant_permission('user2', 'TICKET_CHGPROP')
+        ticket = self._insert_ticket(summary='the summary')
+        comment = 'the comment'
+
+        def make_req(authname):
+            change_time = Ticket(self.env, 1)['changetime']
+            return MockRequest(
+                self.env, authname=authname,
+                method='POST', path_info='/ticket/1',
+                args={'comment': comment, 'action': 'leave', 'submit': True,
+                      'view_time': unicode(to_utimestamp(change_time))})
+
+        req = make_req('user1')
+        self.assertTrue(self.ticket_module.match_request(req))
+        self.assertRaises(RequestDone, self.ticket_module.process_request,
+                          req)
+        self.assertEqual([], req.chrome['warnings'])
+        self.assertEqual(comment,
+                         ticket.get_change(1)['fields']['comment']['new'])
+
+        req = make_req('user2')
+        self.assertTrue(self.ticket_module.match_request(req))
+        self.ticket_module.process_request(req)
+        self.assertEqual(1, len(req.chrome['warnings']))
+        self.assertEqual("No permissions to add a comment.",
+                         unicode(req.chrome['warnings'][0]))
+
 
 def test_suite():
     suite = unittest.TestSuite()
