@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -158,7 +158,7 @@ class TimelineModule(Component):
         filters = [f[0] for f in available_filters if f[0] in req.args]
         if not filters and format != 'rss':
             filters = [f[0] for f in available_filters
-                       if req.session.get('timeline.filter.' + f[0]) == '1']
+                            if req.session.as_int('timeline.filter.' + f[0])]
         if not filters:
             filters = [f[0] for f in available_filters if len(f) == 2 or f[2]]
 
