@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2006-2009 Edgewall Software
+# Copyright (C) 2006-2018 Edgewall Software
 # Copyright (C) 2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -16,6 +16,7 @@
 tasks such as grouping or pagination.
 """
 
+from json import dumps
 from math import ceil
 import re
 
@@ -71,7 +72,7 @@ def group(iterable, num, predicate=None):
 
     >>> items = [1, 2, 3, 4]
     >>> for item in group(items, 2):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3, 4)
 
@@ -80,7 +81,7 @@ def group(iterable, num, predicate=None):
 
     >>> items = [1, 2, 3, 4, 5]
     >>> for item in group(items, 2):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3, 4)
     (5, None)
@@ -92,7 +93,7 @@ def group(iterable, num, predicate=None):
 
     >>> items = [1, 2, 3, 4]
     >>> for item in group(items, 2, lambda x: x != 3):
-    ...     print item
+    ...     print(item)
     (1, 2)
     (3,)
     (4, None)
@@ -246,7 +247,7 @@ class Paginator(object):
         return self.page > 0
 
     def get_shown_pages(self, page_index_count = 11):
-        if self.has_more_pages == False:
+        if not self.has_more_pages:
             return range(1, 2)
 
         min_page = 1
@@ -274,10 +275,11 @@ class Paginator(object):
                     start=self.span[0] + 1, stop=self.span[1], total=total)
 
 
-def separated(items, sep=','):
+def separated(items, sep=',', last=None):
     """Yield `(item, sep)` tuples, one for each element in `items`.
 
-    `sep` will be `None` for the last item.
+    The separator after the last item is specified by the `last` parameter,
+    which defaults to `None`. (Since 1.1.3)
 
     >>> list(separated([1, 2]))
     [(1, ','), (2, None)]
@@ -285,51 +287,27 @@ def separated(items, sep=','):
     >>> list(separated([1]))
     [(1, None)]
 
-    >>> list(separated("abc", ':'))
+    >>> list(separated('abc', ':'))
     [('a', ':'), ('b', ':'), ('c', None)]
+
+    >>> list(separated((1, 2, 3), sep=';', last='.'))
+    [(1, ';'), (2, ';'), (3, '.')]
     """
     items = iter(items)
-    last = items.next()
+    next = items.next()
     for i in items:
-        yield last, sep
-        last = i
-    yield last, None
+        yield next, sep
+        next = i
+    yield next, last
 
 
-try:
-    from json import dumps
+_js_quote = dict((c, '\\u%04x' % ord(c)) for c in '&<>')
+_js_quote_re = re.compile('[' + ''.join(_js_quote) + ']')
 
-    _js_quote = dict((c, '\\u%04x' % ord(c)) for c in '&<>')
-    _js_quote_re = re.compile('[' + ''.join(_js_quote) + ']')
 
-    def to_json(value):
-        """Encode `value` to JSON."""
-        def replace(match):
-            return _js_quote[match.group(0)]
-        text = dumps(value, sort_keys=True, separators=(',', ':'))
-        return _js_quote_re.sub(replace, text)
-
-except ImportError:
-    from trac.util.text import to_js_string
-
-    def to_json(value):
-        """Encode `value` to JSON."""
-        if isinstance(value, basestring):
-            return to_js_string(value)
-        elif value is None:
-            return 'null'
-        elif value is False:
-            return 'false'
-        elif value is True:
-            return 'true'
-        elif isinstance(value, (int, long)):
-            return str(value)
-        elif isinstance(value, float):
-            return repr(value)
-        elif isinstance(value, (list, tuple)):
-            return '[%s]' % ','.join(to_json(each) for each in value)
-        elif isinstance(value, dict):
-            return '{%s}' % ','.join('%s:%s' % (to_json(k), to_json(v))
-                                     for k, v in sorted(value.iteritems()))
-        else:
-            raise TypeError('Cannot encode type %s' % value.__class__.__name__)
+def to_json(value):
+    """Encode `value` to JSON."""
+    def replace(match):
+        return _js_quote[match.group(0)]
+    text = dumps(value, sort_keys=True, separators=(',', ':'))
+    return _js_quote_re.sub(replace, text)
