@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -371,7 +371,7 @@ class ToFragmentTestCase(unittest.TestCase):
     def _ioerror(self, filename):
         try:
             open(filename)
-        except IOError, e:
+        except IOError as e:
             return e
         else:
             self.fail('IOError not raised')
