@@ -22,10 +22,11 @@ from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.util import as_bool, is_path_below
 from trac.util.text import breakable_path, normalize_whitespace, print_table, \
-                           printout
+                           printerr, printout
 from trac.util.translation import _, ngettext, tag_
 from trac.versioncontrol import DbRepositoryProvider, InvalidRepository, \
                                 NoSuchChangeset, RepositoryManager, is_default
+from trac.versioncontrol.cache import CachedRepository
 from trac.web.chrome import Chrome, add_notice, add_warning
 
 
@@ -94,7 +95,8 @@ class VersionControlAdmin(Component):
         rm = RepositoryManager(self.env)
         errors = rm.notify('changeset_added', reponame, (first_rev,) + revs)
         for error in errors:
-            printout(error)
+            printerr(error)
+        return 2 if errors else 0
 
     def _do_changeset_modified(self, reponame, first_rev, *revs):
         if is_default(reponame):
@@ -102,7 +104,8 @@ class VersionControlAdmin(Component):
         rm = RepositoryManager(self.env)
         errors = rm.notify('changeset_modified', reponame, (first_rev,) + revs)
         for error in errors:
-            printout(error)
+            printerr(error)
+        return 2 if errors else 0
 
     def _do_list(self):
         rm = RepositoryManager(self.env)
@@ -129,22 +132,26 @@ class VersionControlAdmin(Component):
             if repos is None:
                 raise TracError(_("Repository '%(repo)s' not found",
                                   repo=reponame or '(default)'))
-            if rev is not None:
-                repos.sync_changeset(rev)
-                printout(_('%(rev)s resynced on %(reponame)s.', rev=rev,
-                           reponame=repos.reponame or '(default)'))
-                return
             repositories = [repos]
 
         for repos in sorted(repositories, key=lambda r: r.reponame):
-            printout(_('Resyncing repository history for %(reponame)s... ',
-                       reponame=repos.reponame or '(default)'))
-            repos.sync(self._sync_feedback, clean=clean)
-            for cnt, in self.env.db_query(
-                    "SELECT count(rev) FROM revision WHERE repos=%s",
-                    (repos.id,)):
-                printout(ngettext('%(num)s revision cached.',
-                                  '%(num)s revisions cached.', num=cnt))
+            pretty_name = repos.reponame or '(default)'
+            if not isinstance(repos, CachedRepository):
+                printout(_("%(reponame)s is not a cached repository.",
+                           reponame=pretty_name))
+            elif rev is not None:
+                repos.sync_changeset(rev)
+                printout(_('%(rev)s resynced on %(reponame)s.', rev=rev,
+                           reponame=pretty_name))
+            else:
+                printout(_('Resyncing repository history for %(reponame)s... ',
+                           reponame=pretty_name))
+                repos.sync(self._sync_feedback, clean=clean)
+                for cnt, in self.env.db_query(
+                        "SELECT count(rev) FROM revision WHERE repos=%s",
+                        (repos.id,)):
+                    printout(ngettext('%(num)s revision cached.',
+                                      '%(num)s revisions cached.', num=cnt))
         printout(_('Done.'))
 
     def _sync_feedback(self, rev):
@@ -219,7 +226,7 @@ class RepositoryAdminPanel(Component):
                         db_provider.modify_repository(reponame, changes)
                         add_notice(req, _('Your changes have been saved.'))
                         name = req.args.get('name')
-                        resync = tag.code('trac-admin %s repository resync '
+                        resync = tag.code('trac-admin "%s" repository resync '
                                           '"%s"' % (self.env.path,
                                                     name or '(default)'))
                         if 'dir' in changes:
@@ -233,7 +240,7 @@ class RepositoryAdminPanel(Component):
                                        resync=resync)
                             add_notice(req, msg)
                         if name and name != path_info and not 'alias' in info:
-                            cset_added = tag.code('trac-admin %s changeset '
+                            cset_added = tag.code('trac-admin "%s" changeset '
                                                   'added "%s" $REV'
                                                   % (self.env.path,
                                                      name or '(default)'))
@@ -271,13 +278,13 @@ class RepositoryAdminPanel(Component):
                         name = name or '(default)'
                         add_notice(req, _('The repository "%(name)s" has been '
                                           'added.', name=name))
-                        resync = tag.code('trac-admin %s repository resync '
+                        resync = tag.code('trac-admin "%s" repository resync '
                                            '"%s"' % (self.env.path, name))
                         msg = tag_('You should now run %(resync)s to '
                                    'synchronize Trac with the repository.',
                                    resync=resync)
                         add_notice(req, msg)
-                        cset_added = tag.code('trac-admin %s changeset '
+                        cset_added = tag.code('trac-admin "%s" changeset '
                                               'added "%s" $REV'
                                               % (self.env.path, name))
                         doc = tag.a(_("documentation"),
@@ -289,7 +296,6 @@ class RepositoryAdminPanel(Component):
                                    '%(doc)s for more information.',
                                    cset_added=cset_added, doc=doc)
                         add_notice(req, msg)
-                        req.redirect(req.href.admin(category, page))
 
                 # Add a repository alias
                 elif db_provider and req.args.get('add_alias'):
@@ -304,13 +310,13 @@ class RepositoryAdminPanel(Component):
                                               name=name or '(default)'))
                         add_notice(req, _('The alias "%(name)s" has been '
                                           'added.', name=name or '(default)'))
-                        req.redirect(req.href.admin(category, page))
-                    add_warning(req, _('Missing arguments to add an '
-                                       'alias.'))
+                    else:
+                        add_warning(req, _('Missing arguments to add an '
+                                           'alias.'))
 
                 # Refresh the list of repositories
                 elif req.args.get('refresh'):
-                    req.redirect(req.href.admin(category, page))
+                    pass
 
                 # Remove repositories
                 elif db_provider and req.args.get('remove'):
@@ -320,8 +326,10 @@ class RepositoryAdminPanel(Component):
                             db_provider.remove_repository(name)
                         add_notice(req, _('The selected repositories have '
                                           'been removed.'))
-                        req.redirect(req.href.admin(category, page))
-                    add_warning(req, _('No repositories were selected.'))
+                    else:
+                        add_warning(req, _('No repositories were selected.'))
+
+                req.redirect(req.href.admin(category, page))
 
             data = {'view': 'list'}
 
