@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2009 Edgewall Software
+# Copyright (C) 2004-2018 Edgewall Software
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -17,13 +17,18 @@
 import difflib
 import re
 
-from genshi import Markup, escape
-
+from trac.util.html import Markup, escape
 from trac.util.text import expandtabs
 
 __all__ = ['diff_blocks', 'get_change_extent', 'get_diff_options',
            'unified_diff']
 
+_whitespace_split = re.compile(r'\s+', re.UNICODE).split
+
+
+def _norm_space_changes(text):
+    return ' '.join(_whitespace_split(text))
+
 
 def get_change_extent(str1, str2):
     """Determines the extent of differences between two strings.
@@ -65,11 +70,16 @@ def get_filtered_hunks(fromlines, tolines, context=None,
     to filter out the results will come straight from the
     SequenceMatcher.
     """
+    if ignore_space_changes:
+        fromlines = map(_norm_space_changes, fromlines)
+        tolines = map(_norm_space_changes, tolines)
+    if ignore_case:
+        fromlines = [l.lower() for l in fromlines]
+        tolines = [l.lower() for l in tolines]
     hunks = get_hunks(fromlines, tolines, context)
-    if ignore_space_changes or ignore_case or ignore_blank_lines:
+    if ignore_blank_lines:
         hunks = filter_ignorable_lines(hunks, fromlines, tolines, context,
-                                       ignore_blank_lines, ignore_case,
-                                       ignore_space_changes)
+                                       ignore_blank_lines, False, False)
     return hunks
 
 
@@ -95,20 +105,19 @@ def filter_ignorable_lines(hunks, fromlines, tolines, context,
     See `get_filtered_hunks` for the parameter descriptions.
     """
     def is_ignorable(tag, fromlines, tolines):
-        if tag == 'delete' and ignore_blank_lines:
-            if ''.join(fromlines) == '':
-                return True
-        elif tag == 'insert' and ignore_blank_lines:
-            if ''.join(tolines) == '':
-                return True
-        elif tag == 'replace' and (ignore_case or ignore_space_changes):
+        if ignore_blank_lines:
+            if tag == 'delete':
+                return not any(fromlines)
+            if tag == 'insert':
+                return not any(tolines)
+        if (ignore_case or ignore_space_changes) and tag == 'replace':
             if len(fromlines) != len(tolines):
                 return False
             def f(str):
                 if ignore_case:
                     str = str.lower()
                 if ignore_space_changes:
-                    str = ' '.join(str.split())
+                    str = _norm_space_changes(str)
                 return str
             for i in range(len(fromlines)):
                 if f(fromlines[i]) != f(tolines[i]):
@@ -176,10 +185,6 @@ def filter_ignorable_lines(hunks, fromlines, tolines, context,
             yield hunk
 
 
-def hdf_diff(*args, **kwargs):
-    """:deprecated: use `diff_blocks` (will be removed in 1.1.1)"""
-    return diff_blocks(*args, **kwargs)
-
 def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
                 ignore_blank_lines=0, ignore_case=0, ignore_space_changes=0):
     """Return an array that is adequate for adding to the data dictionary
@@ -306,7 +311,7 @@ def get_diff_options(req):
     data = {'options': options_data}
 
     def get_bool_option(name, default=0):
-        pref = int(req.session.get('diff_' + name, default))
+        pref = req.session.as_int('diff_' + name, default)
         arg = int(name in req.args)
         if 'update' in req.args and arg != pref:
             req.session.set('diff_' + name, arg, default)
@@ -320,16 +325,13 @@ def get_diff_options(req):
         req.session.set('diff_style', style, 'inline')
     data['style'] = style
 
-    pref = int(req.session.get('diff_contextlines', 2))
-    try:
-        context = int(req.args.get('contextlines', pref))
-    except ValueError:
-        context = -1
+    pref = req.session.as_int('diff_contextlines', 2)
+    context = req.args.as_int('contextlines', pref)
     if 'update' in req.args and context != pref:
         req.session.set('diff_contextlines', context, 2)
     options_data['contextlines'] = context
 
-    arg = int(req.args.get('contextall', 0))
+    arg = req.args.as_int('contextall', 0)
     options_data['contextall'] = arg
     options = ['-U%d' % (-1 if arg else context)]
 
