@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -18,8 +18,8 @@
 
 import re
 
-from genshi.core import Markup
 from genshi.builder import tag
+from genshi.core import Markup
 
 from trac.config import IntOption, ListOption
 from trac.core import *
@@ -28,14 +28,14 @@ from trac.resource import ResourceNotFound
 from trac.util import Ranges
 from trac.util.text import to_unicode, wrap
 from trac.util.translation import _
-from trac.versioncontrol.api import (RepositoryManager, Changeset,
-                                     NoSuchChangeset)
+from trac.versioncontrol.api import (Changeset, NoSuchChangeset,
+                                     RepositoryManager)
 from trac.versioncontrol.web_ui.changeset import ChangesetModule
 from trac.versioncontrol.web_ui.util import *
-from trac.web import IRequestHandler
-from trac.web.chrome import (Chrome, INavigationContributor, add_ctxtnav,
-                             add_link, add_script, add_script_data,
-                             add_stylesheet, auth_link, web_context)
+from trac.web.api import IRequestHandler
+from trac.web.chrome import (INavigationContributor, add_ctxtnav, add_link,
+                             add_script, add_script_data, add_stylesheet,
+                             auth_link, web_context)
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
 
@@ -44,9 +44,11 @@ class LogModule(Component):
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
                IWikiSyntaxProvider)
 
+    realm = RepositoryManager.changeset_realm
+
     default_log_limit = IntOption('revisionlog', 'default_log_limit', 100,
         """Default value for the limit argument in the TracRevisionLog.
-        (''since 0.11'')""")
+        """)
 
     graph_colors = ListOption('revisionlog', 'graph_colors',
         ['#cc0', '#0c0', '#0cc', '#00c', '#c0c', '#c00'],
@@ -79,19 +81,23 @@ class LogModule(Component):
 
         mode = req.args.get('mode', 'stop_on_copy')
         path = req.args.get('path', '/')
-        rev = req.args.get('rev')
+        rev = req.args.getfirst('rev')
         stop_rev = req.args.get('stop_rev')
         revs = req.args.get('revs')
         format = req.args.get('format')
         verbose = req.args.get('verbose')
-        limit = int(req.args.get('limit') or self.default_log_limit)
+        limit = req.args.getint('limit', self.default_log_limit)
 
         rm = RepositoryManager(self.env)
         reponame, repos, path = rm.get_repository_by_path(path)
 
         if not repos:
-            raise ResourceNotFound(_("Repository '%(repo)s' not found",
-                                   repo=reponame))
+            if path == '/':
+                raise TracError(_("No repository specified and no default"
+                                  " repository configured."))
+            else:
+                raise ResourceNotFound(_("Repository '%(repo)s' not found",
+                                         repo=reponame or path.strip('/')))
 
         if reponame != repos.reponame:  # Redirect alias
             qs = req.query_string
@@ -99,65 +105,56 @@ class LogModule(Component):
                          + ('?' + qs if qs else ''))
 
         normpath = repos.normalize_path(path)
+
         # if `revs` parameter is given, then we're restricted to the
         # corresponding revision ranges.
         # If not, then we're considering all revisions since `rev`,
         # on that path, in which case `revranges` will be None.
-        revranges = None
         if revs:
-            try:
-                revranges = Ranges(revs)
-                rev = revranges.b
-            except ValueError:
-                pass
-        rev = unicode(repos.normalize_rev(rev))
-        display_rev = repos.display_rev
+            revranges = RevRanges(repos, revs, resolve=True)
+            rev = revranges.b
+        else:
+            revranges = None
+            rev = repos.normalize_rev(rev)
 
         # The `history()` method depends on the mode:
         #  * for ''stop on copy'' and ''follow copies'', it's `Node.history()`
         #    unless explicit ranges have been specified
         #  * for ''show only add, delete'' we're using
         #   `Repository.get_path_history()`
-        cset_resource = repos.resource.child('changeset')
+        cset_resource = repos.resource.child(self.realm)
         show_graph = False
+        curr_revrange = []
         if mode == 'path_history':
             def history():
                 for h in repos.get_path_history(path, rev):
                     if 'CHANGESET_VIEW' in req.perm(cset_resource(id=h[1])):
                         yield h
+
         elif revranges:
+            show_graph = path == '/' and not verbose \
+                         and not repos.has_linear_changesets \
+                         and len(revranges) == 1
+
             def history():
-                prevpath = path
-                expected_next_item = None
-                ranges = list(revranges.pairs)
-                ranges.reverse()
-                for (a, b) in ranges:
-                    a = repos.normalize_rev(a)
-                    b = repos.normalize_rev(b)
-                    while not repos.rev_older_than(b, a):
-                        node = get_existing_node(req, repos, prevpath, b)
-                        node_history = list(node.get_history(2))
-                        p, rev, chg = node_history[0]
+                separator = False
+                for a, b in reversed(revranges.pairs):
+                    curr_revrange[:] = (a, b)
+                    node = get_existing_node(req, repos, path, b)
+                    for p, rev, chg in node.get_history():
                         if repos.rev_older_than(rev, a):
-                            break # simply skip, no separator
+                            break
                         if 'CHANGESET_VIEW' in req.perm(cset_resource(id=rev)):
-                            if expected_next_item:
-                                # check whether we're continuing previous range
-                                np, nrev, nchg = expected_next_item
-                                if rev != nrev: # no, we need a separator
-                                    yield (np, nrev, None)
-                            yield node_history[0]
-                        prevpath = node_history[-1][0] # follow copy
-                        b = repos.previous_rev(rev)
-                        if len(node_history) > 1:
-                            expected_next_item = node_history[-1]
-                        else:
-                            expected_next_item = None
-                if expected_next_item:
-                    yield (expected_next_item[0], expected_next_item[1], None)
+                            separator = True
+                            yield p, rev, chg
+                    else:
+                        separator = False
+                    if separator:
+                        yield p, rev, None
         else:
             show_graph = path == '/' and not verbose \
                          and not repos.has_linear_changesets
+
             def history():
                 node = get_existing_node(req, repos, path, rev)
                 for h in node.get_history():
@@ -169,6 +166,7 @@ class LogModule(Component):
         depth = 1
         previous_path = normpath
         count = 0
+        history_remaining = True
         for old_path, old_rev, old_chg in history():
             if stop_rev and repos.rev_older_than(old_rev, stop_rev):
                 break
@@ -192,7 +190,7 @@ class LogModule(Component):
                     break
                 elif mode == 'path_history':
                     depth -= 1
-            if old_chg is None: # separator entry
+            if old_chg is None:  # separator entry
                 stop_limit = limit
             else:
                 count += 1
@@ -200,13 +198,17 @@ class LogModule(Component):
             if count >= stop_limit:
                 break
             previous_path = old_path
-        if info == []:
+        else:
+            history_remaining = False
+        if not info:
             node = get_existing_node(req, repos, path, rev)
             if repos.rev_older_than(stop_rev, node.created_rev):
                 # FIXME: we should send a 404 error here
                 raise TracError(_("The file or directory '%(path)s' doesn't "
-                    "exist at revision %(rev)s or at any previous revision.",
-                    path=path, rev=display_rev(rev)), _('Nonexistent path'))
+                                  "exist at revision %(rev)s or at any "
+                                  "previous revision.", path=path,
+                                  rev=repos.display_rev(rev)),
+                                _('Nonexistent path'))
 
         # Generate graph data
         graph = {}
@@ -231,22 +233,27 @@ class LogModule(Component):
             return req.href.log(repos.reponame or None, path, **params)
 
         if format in ('rss', 'changelog'):
-            info = [i for i in info if i['change']] # drop separators
+            info = [i for i in info if i['change']]  # drop separators
             if info and count > limit:
                 del info[-1]
-        elif info and count >= limit:
+        elif info and history_remaining and count >= limit:
             # stop_limit reached, there _might_ be some more
             next_rev = info[-1]['rev']
             next_path = info[-1]['path']
             next_revranges = None
-            if revranges:
-                next_revranges = str(revranges.truncate(next_rev))
+            if curr_revrange:
+                new_revrange = (curr_revrange[0], next_rev) \
+                               if info[-1]['change'] else None
+                next_revranges = revranges.truncate(curr_revrange,
+                                                    new_revrange)
+                next_revranges = unicode(next_revranges) or None
             if next_revranges or not revranges:
-                older_revisions_href = make_log_href(next_path, rev=next_rev,
-                                                     revs=next_revranges)
+                older_revisions_href = make_log_href(
+                    next_path, rev=next_rev, revs=next_revranges)
                 add_link(req, 'next', older_revisions_href,
-                    _('Revision Log (restarting at %(path)s, rev. %(rev)s)',
-                    path=next_path, rev=display_rev(next_rev)))
+                         _('Revision Log (restarting at %(path)s, rev. '
+                           '%(rev)s)', path=next_path,
+                           rev=repos.display_rev(next_rev)))
             # only show fully 'limit' results, use `change == None` as a marker
             info[-1]['change'] = None
 
@@ -274,18 +281,17 @@ class LogModule(Component):
             'context': web_context(req, 'source', path, parent=repos.resource),
             'reponame': repos.reponame or None, 'repos': repos,
             'path': path, 'rev': rev, 'stop_rev': stop_rev,
-            'display_rev': display_rev, 'revranges': revranges,
-            'mode': mode, 'verbose': verbose, 'limit' : limit,
+            'display_rev': repos.display_rev, 'revranges': revranges,
+            'mode': mode, 'verbose': verbose, 'limit': limit,
             'items': info, 'changes': changes, 'extra_changes': extra_changes,
             'graph': graph,
-            'wiki_format_messages':
-            self.config['changeset'].getbool('wiki_format_messages')
+            'wiki_format_messages': self.config['changeset']
+                                    .getbool('wiki_format_messages')
         }
 
         if format == 'changelog':
             return 'revisionlog.txt', data, 'text/plain'
         elif format == 'rss':
-            data['email_map'] = Chrome(self.env).get_email_map()
             data['context'] = web_context(req, 'source',
                                           path, parent=repos.resource,
                                           absurls=True)
@@ -294,8 +300,8 @@ class LogModule(Component):
         item_ranges = []
         range = []
         for item in info:
-            if item['change'] is None: # separator
-                if range: # start new range
+            if item['change'] is None:  # separator
+                if range:  # start new range
                     range.append(item)
                     item_ranges.append(range)
                     range = []
@@ -320,7 +326,8 @@ class LogModule(Component):
                  'application/rss+xml', 'rss')
         changelog_href = make_log_href(path, format='changelog', revs=revs,
                                        stop_rev=stop_rev)
-        add_link(req, 'alternate', changelog_href, _('ChangeLog'), 'text/plain')
+        add_link(req, 'alternate', changelog_href, _('ChangeLog'),
+                 'text/plain')
 
         add_ctxtnav(req, _('View Latest Revision'),
                     href=req.href.browser(repos.reponame or None, path))
@@ -334,8 +341,9 @@ class LogModule(Component):
 
     # IWikiSyntaxProvider methods
 
-    REV_RANGE = r"(?:%s|%s)" % (Ranges.RE_STR, ChangesetModule.CHANGESET_ID)
-    #                          int rev ranges or any kind of rev
+    # int rev ranges or any kind of rev range
+    REV_RANGE = r"(?:%(int)s|%(cset)s(?:[:-]%(cset)s)?)" % \
+                {'int': Ranges.RE_STR, 'cset': ChangesetModule.CHANGESET_ID}
 
     def get_wiki_syntax(self):
         yield (
@@ -352,6 +360,8 @@ class LogModule(Component):
     def get_link_resolvers(self):
         yield ('log', self._format_link)
 
+    LOG_LINK_RE = re.compile(r"([^@:]*)[@:]%s?" % REV_RANGE)
+
     def _format_link(self, formatter, ns, match, label, fullmatch=None):
         if ns == 'log1':
             groups = fullmatch.groupdict()
@@ -388,23 +398,19 @@ class LogModule(Component):
                     repos = rm.get_repository(reponame)
 
             if repos:
-                revranges = None
-                if any(c for c in ':-,' if c in revs):
-                    revranges = self._normalize_ranges(repos, path, revs)
-                    revs = None
-                if 'LOG_VIEW' in formatter.perm:
-                    if revranges:
-                        href = formatter.href.log(repos.reponame or None,
-                                                  path or '/',
-                                                  revs=str(revranges))
+                if 'LOG_VIEW' in formatter.perm(repos.resource):
+                    reponame = repos.reponame or None
+                    path = path or '/'
+                    revranges = RevRanges(repos, revs)
+                    if revranges.has_ranges():
+                        href = formatter.href.log(reponame, path,
+                                                  revs=unicode(revranges))
                     else:
-                        try:
-                            rev = repos.normalize_rev(revs)
-                        except NoSuchChangeset:
-                            rev = None
-                        href = formatter.href.log(repos.reponame or None,
-                                                  path or '/', rev=rev)
-                    if query and (revranges or revs):
+                        # try to resolve if single rev
+                        repos.normalize_rev(revs)
+                        href = formatter.href.log(reponame, path,
+                                                  rev=revs or None)
+                    if query and '?' in href:
                         query = '&' + query[1:]
                     return tag.a(label, class_='source',
                                  href=href + query + fragment)
@@ -413,24 +419,118 @@ class LogModule(Component):
                 errmsg = _("Repository '%(repo)s' not found", repo=reponame)
             else:
                 errmsg = _("No default repository defined")
-        except TracError, e:
+        except TracError as e:
             errmsg = to_unicode(e)
         return tag.a(label, class_='missing source', title=errmsg)
 
-    LOG_LINK_RE = re.compile(r"([^@:]*)[@:]%s?" % REV_RANGE)
 
-    def _normalize_ranges(self, repos, path, revs):
-        ranges = revs.replace(':', '-')
-        try:
-            # fast path; only numbers
-            return Ranges(ranges, reorder=True)
-        except ValueError:
-            # slow path, normalize each rev
-            splitted_ranges = re.split(r'([-,])', ranges)
+class RevRanges(object):
+
+    def __init__(self, repos, revs=None, resolve=False):
+        self.repos = repos
+        self.resolve = resolve
+        self.pairs = []
+        self.a = self.b = None
+        if revs:
+            self._append(revs)
+
+    def has_ranges(self):
+        n = len(self.pairs)
+        return n > 1 or n == 1 and self.a != self.b
+
+    def truncate(self, curr_pair, new_pair=None):
+        curr_pair = tuple(curr_pair)
+        if new_pair:
+            new_pair = tuple(new_pair)
+        revranges = RevRanges(self.repos, resolve=self.resolve)
+        pairs = revranges.pairs
+        for pair in self.pairs:
+            if pair == curr_pair:
+                if new_pair:
+                    pairs.append(new_pair)
+                break
+            pairs.append(pair)
+        if pairs:
+            revranges.a = pairs[0][0]
+            revranges.b = pairs[-1][1]
+        revranges._reduce()
+        return revranges
+
+    def _normrev(self, rev):
+        if not rev:
+            raise NoSuchChangeset(rev)
+        if self.resolve:
+            return self.repos.normalize_rev(rev)
+        elif self.repos.has_linear_changesets:
             try:
-                revs = [repos.normalize_rev(r) for r in splitted_ranges[::2]]
-            except NoSuchChangeset:
-                return None
-            seps = splitted_ranges[1::2] + ['']
-            ranges = ''.join([str(rev)+sep for rev, sep in zip(revs, seps)])
-            return Ranges(ranges)
+                return int(rev)
+            except (ValueError, TypeError):
+                return rev
+        else:
+            return rev
+
+    _cset_range_re = re.compile(r"""(?:
+        %(cset)s[:-]%(cset)s    |  # int or hexa revs
+        [0-9]+[:-][A-Za-z_0-9]+ |  # e.g. 42-head
+        [A-Za-z_0-9]+[:-][0-9]+ |  # e.g. head-42
+        [^:]+:[^:]+                # e.g. master:dev-42
+        )\Z
+        """ % {'cset': ChangesetModule.CHANGESET_ID}, re.VERBOSE)
+
+    def _append(self, revs):
+        if not revs:
+            return
+
+        pairs = []
+        for rev in re.split(u',\u200b?', revs):
+            a = b = None
+            if self._cset_range_re.match(rev):
+                for sep in ':-':
+                    if sep in rev:
+                        a, b = rev.split(sep)
+                        break
+            if a is None:
+                a = b = self._normrev(rev)
+            elif a == b:
+                a = b = self._normrev(a)
+            else:
+                a = self._normrev(a)
+                b = self._normrev(b)
+            pairs.append((a, b))
+        self.pairs.extend(pairs)
+        self._reduce()
+
+    def _reduce(self):
+        if all(isinstance(pair[0], (int, long)) and
+               isinstance(pair[1], (int, long))
+               for pair in self.pairs):
+            try:
+                ranges = Ranges(unicode(self), reorder=True)
+            except:
+                pass
+            else:
+                self.pairs[:] = ranges.pairs
+        else:
+            seen = set()
+            pairs = self.pairs[:]
+            for idx, pair in enumerate(pairs):
+                if pair in seen:
+                    pairs[idx] = None
+                else:
+                    seen.add(pair)
+            if len(pairs) != len(seen):
+                self.pairs[:] = filter(None, pairs)
+        if self.pairs:
+            self.a = self.pairs[0][0]
+            self.b = self.pairs[-1][1]
+        else:
+            self.a = self.b = None
+
+    def __len__(self):
+        return len(self.pairs)
+
+    def __unicode__(self):
+        sep = '-' if self.repos.has_linear_changesets else ':'
+        return ','.join(sep.join(map(unicode, pair)) if pair[0] != pair[1]
+                                                     else unicode(pair[0])
+                        for pair in self.pairs)
