@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2018 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -17,16 +17,35 @@
 #         Christian Boos <cboos@edgewall.org>
 
 from itertools import izip
+from tempfile import TemporaryFile
+from zipfile import ZipFile, ZIP_DEFLATED
 
 from genshi.builder import tag
 
 from trac.resource import ResourceNotFound
-from trac.util.datefmt import datetime, utc
+from trac.util import content_disposition, create_zipinfo
+from trac.util.datefmt import http_date
 from trac.util.translation import tag_, _
-from trac.versioncontrol.api import Changeset, NoSuchNode, NoSuchChangeset
+from trac.versioncontrol.api import EmptyChangeset, NoSuchChangeset, \
+                                    NoSuchNode
+from trac.web.api import RequestDone
 
-__all__ = ['get_changes', 'get_path_links', 'get_existing_node',
-           'get_allowed_node', 'make_log_graph']
+__all__ = ['content_closing', 'get_changes', 'get_path_links',
+           'get_existing_node', 'get_allowed_node', 'make_log_graph',
+           'render_zip']
+
+
+class content_closing(object):
+
+    def __init__(self, content):
+        self.content = content
+
+    def __enter__(self):
+        return self.content
+
+    def __exit__(self, *exc_info):
+        if hasattr(self.content, 'close'):
+            self.content.close()
 
 
 def get_changes(repos, revs, log=None):
@@ -37,10 +56,10 @@ def get_changes(repos, revs, log=None):
         try:
             changeset = repos.get_changeset(rev)
         except NoSuchChangeset:
-            changeset = Changeset(repos, rev, '', '',
-                                  datetime(1970, 1, 1, tzinfo=utc))
+            changeset = EmptyChangeset(repos, rev)
             if log is not None:
-                log.warning("Unable to get changeset [%s]", rev)
+                log.warning("Unable to get changeset [%s] in %s", rev,
+                            repos.reponame or '(default)')
         changes[rev] = changeset
     return changes
 
@@ -68,13 +87,13 @@ def get_path_links(href, reponame, path, rev, order=None, desc=None):
 def get_existing_node(req, repos, path, rev):
     try:
         return repos.get_node(path, rev)
-    except NoSuchNode, e:
+    except NoSuchNode as e:
         # TRANSLATOR: You can 'search' in the repository history... (link)
         search_a = tag.a(_("search"),
                          href=req.href.log(repos.reponame or None, path,
                                            rev=rev, mode='path_history'))
         raise ResourceNotFound(tag(
-            tag.p(e.message, class_="message"),
+            tag.p(e, class_="message"),
             tag.p(tag_("You can %(search)s in the repository history to see "
                        "if that path existed but was later removed",
                        search=search_a))))
@@ -132,13 +151,13 @@ def make_log_graph(repos, revs):
             column = active.index(rev)
             vertices.append((column, threads.index(active_thread[column])))
 
-            next_rev = revs.next() # Raises StopIteration when no more revs
+            next_rev = revs.next()  # Raises StopIteration when no more revs
             next = active[:]
             parents = list(repos.parent_revs(rev))
 
             # Replace current item with parents not already present
             new_parents = [p for p in parents if p not in active]
-            next[column : column + 1] = new_parents
+            next[column:column + 1] = new_parents
 
             # Add edges to parents
             for col, (r, thread) in enumerate(izip(active, active_thread)):
@@ -158,11 +177,89 @@ def make_log_graph(repos, revs):
             else:
                 base = len(threads)
                 threads.extend([[0, column + 1 + i, line + 1]]
-                                for i in xrange(len(new_parents) - 1))
-                active_thread[column + 1 : column + 1] = threads[base:]
+                               for i in xrange(len(new_parents) - 1))
+                active_thread[column + 1:column + 1] = threads[base:]
 
             active = next
             line += 1
     except StopIteration:
         pass
     return threads, vertices, columns
+
+
+def render_zip(req, filename, repos, root_node, iter_nodes):
+    """Send a ZIP file containing the data corresponding to the `nodes`
+    iterable.
+
+    :type root_node: `~trac.versioncontrol.api.Node`
+    :param root_node: optional ancestor for all the *nodes*
+
+    :param iter_nodes: callable taking the optional *root_node* as input
+                       and generating the `~trac.versioncontrol.api.Node`
+                       for which the content should be added into the zip.
+    """
+    req.send_response(200)
+    req.send_header('Content-Type', 'application/zip')
+    req.send_header('Content-Disposition',
+                    content_disposition('inline', filename))
+    if root_node:
+        req.send_header('Last-Modified', http_date(root_node.last_modified))
+        root_path = root_node.path.rstrip('/')
+    else:
+        root_path = ''
+    if root_path:
+        root_path += '/'
+        root_name = root_node.name + '/'
+    else:
+        root_name = ''
+    root_len = len(root_path)
+    req.end_headers()
+
+    def write_partial(fileobj, start):
+        end = fileobj.tell()
+        fileobj.seek(start, 0)
+        remaining = end - start
+        while remaining > 0:
+            chunk = fileobj.read(min(remaining, 4096))
+            req.write(chunk)
+            remaining -= len(chunk)
+        fileobj.seek(end, 0)
+        return end
+
+    pos = 0
+    fileobj = TemporaryFile(prefix='trac-', suffix='.zip')
+    try:
+        zipfile = ZipFile(fileobj, 'w', ZIP_DEFLATED)
+        for node in iter_nodes(root_node):
+            if node is root_node:
+                continue
+            path = node.path.strip('/')
+            assert path.startswith(root_path)
+            path = root_name + path[root_len:]
+            kwargs = {'mtime': node.last_modified}
+            data = None
+            if node.isfile:
+                with content_closing(
+                        node.get_processed_content(eol_hint='CRLF')) \
+                        as content:
+                    data = content.read()
+                properties = node.get_properties()
+                # Subversion specific
+                if 'svn:special' in properties and data.startswith('link '):
+                    data = data[5:]
+                    kwargs['symlink'] = True
+                if 'svn:executable' in properties:
+                    kwargs['executable'] = True
+            elif node.isdir and path:
+                kwargs['dir'] = True
+                data = ''
+            if data is not None:
+                zipfile.writestr(create_zipinfo(path, **kwargs), data)
+                pos = write_partial(fileobj, pos)
+    finally:
+        try:
+            zipfile.close()
+            write_partial(fileobj, pos)
+        finally:
+            fileobj.close()
+    raise RequestDone
