@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -24,7 +24,6 @@ import new
 import mimetypes
 import os
 import re
-import socket
 from StringIO import StringIO
 import sys
 import urlparse
@@ -452,6 +451,12 @@ class Request(object):
     This class provides a convenience API over WSGI.
     """
 
+    _disallowed_control_codes_re = re.compile(r'[\x00-\x08\x0a-\x1f\x7f]')
+    _reserved_headers = set(['content-type', 'content-length', 'location',
+                             'etag', 'pragma', 'cache-control', 'expires'])
+    # RFC7230 3.2 Header Fields
+    _valid_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
+
     def __init__(self, environ, start_response):
         """Create the request wrapper.
 
@@ -465,6 +470,7 @@ class Request(object):
         self._write = None
         self._status = '200 OK'
         self._response = None
+        self._content_type = None
 
         self._outheaders = []
         self._outcharset = None
@@ -595,6 +601,7 @@ class Request(object):
         """
         lower_name = name.lower()
         if lower_name == 'content-type':
+            self._content_type = value.split(';', 1)[0]
             ctpos = value.find('charset=')
             if ctpos >= 0:
                 self._outcharset = value[ctpos + 8:].strip()
@@ -606,6 +613,10 @@ class Request(object):
         """Must be called after all headers have been sent and before the
         actual content is written.
         """
+        if self.method == 'POST' and self._content_type == 'text/html':
+            # Disable XSS protection (#12926)
+            self.send_header('X-XSS-Protection', 0)
+        self._send_configurable_headers()
         self._send_cookie_headers()
         self._write = self._start_response(self._status, self._outheaders)
 
@@ -726,6 +737,7 @@ class Request(object):
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
         self.send_header('Content-Length', len(data))
+        self._send_configurable_headers()
         self._send_cookie_headers()
 
         self._write = self._start_response(self._status, self._outheaders,
@@ -835,11 +847,22 @@ class Request(object):
                     buf[:] = ()
             if bufsize > 0:
                 self._write(''.join(buf))
-        except (IOError, socket.error) as e:
-            if self._is_client_disconnected(e):
+        except IOError as e:
+            if is_client_disconnect_exception(e):
                 raise RequestDone
             raise
 
+    @classmethod
+    def is_valid_header(cls, name, value=None):
+        """Check whether the field name, and optionally the value, make
+        a valid HTTP header.
+        """
+        valid_name = name and name.lower() not in cls._reserved_headers and \
+                     bool(cls._valid_header_re.match(name))
+        valid_value = not cls._disallowed_control_codes_re.search(value) \
+                      if value else True
+        return valid_name & valid_value
+
     # Internal methods
 
     def _parse_arg_list(self):
@@ -865,8 +888,8 @@ class Request(object):
         try:
             fs = _FieldStorage(fp, environ=self.environ,
                                keep_blank_values=True)
-        except (IOError, socket.error) as e:
-            if self._is_client_disconnected(e):
+        except IOError as e:
+            if is_client_disconnect_exception(e):
                 raise HTTPBadRequest(
                     _("Exception caught while reading request: %(msg)s",
                       msg=exception_to_unicode(e)))
@@ -948,6 +971,12 @@ class Request(object):
         return urlparse.urlunparse((self.scheme, host, self.base_path, None,
                                     None, None))
 
+    def _send_configurable_headers(self):
+        sent_headers = [name.lower() for name, val in self._outheaders]
+        for name, val in getattr(self, 'configurable_headers', []):
+            if name.lower() not in sent_headers:
+                self.send_header(name, val)
+
     def _send_cookie_headers(self):
         for name in self.outcookie.keys():
             path = self.outcookie[name].get('path')
@@ -961,15 +990,5 @@ class Request(object):
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
-    def _is_client_disconnected(self, e):
-        if is_client_disconnect_exception(e):
-            return True
-        # Note that mod_wsgi raises an IOError with only a message
-        # if the client disconnects
-        if 'mod_wsgi.version' in self.environ:
-            return e.args[0] in ('failed to write data',
-                                 'client connection closed',
-                                 'request data read error')
-        return False
 
 __no_apidoc__ = _HTTPException_subclass_names
