@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2015 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,7 +12,6 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 
 import os.path
-import shutil
 import sys
 import tempfile
 import unittest
@@ -22,7 +21,7 @@ import trac.tests.compat
 from genshi.builder import tag
 from trac import perm
 from trac.core import TracError
-from trac.test import EnvironmentStub, Mock, MockPerm
+from trac.test import EnvironmentStub, Mock, MockPerm, rmtree
 from trac.util import create_file
 from trac.util.datefmt import utc
 from trac.util.text import shorten_line
@@ -246,7 +245,7 @@ class RequestTestCase(unittest.TestCase):
             status_sent.append(status)
             headers_sent.update(dict(headers))
         environ = _make_environ(method='HEAD')
-        req = Request(environ, start_response,)
+        req = Request(environ, start_response)
         req.session = Mock(save=lambda: None)
         self.assertRaises(RequestDone, req.redirect,
                           'http://example.com/trac/test')
@@ -265,7 +264,7 @@ class RequestTestCase(unittest.TestCase):
                 status_sent.append(status)
                 headers_sent.update(dict(headers))
             environ = _make_environ(method='POST', HTTP_USER_AGENT=ua)
-            req = Request(environ, start_response,)
+            req = Request(environ, start_response)
             req.session = Mock(save=lambda: None)
             self.assertRaises(RequestDone, req.redirect, url)
             self.assertEqual('303 See Other', status_sent[0])
@@ -510,6 +509,48 @@ ne\x00w\r\n\
         req = Request(environ, None)
         self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
 
+    def test_post_text_html_disables_xss(self):
+        """POST request with content-type text/html disables XSS
+        protection (#12926).
+        """
+        status_sent = []
+        headers_sent = {}
+        buf = StringIO()
+
+        def write(data):
+            buf.write(data)
+
+        def start_response(status, headers):
+            status_sent.append(status)
+            headers_sent.update(dict(headers))
+            return write
+
+        content_type = 'text/html'
+        content = "The content"
+        environ = _make_environ(method='POST',
+                                **{'wsgi.input': StringIO(content),
+                                   'CONTENT_LENGTH': str(len(content)),
+                                   'CONTENT_TYPE': content_type})
+        req = Request(environ, start_response)
+        self.assertRaises(RequestDone, req.send, content, content_type)
+        self.assertIn('0', headers_sent['X-XSS-Protection'])
+
+    def test_is_valid_header(self):
+        # Reserved headers not allowed.
+        for name in ('Content-Type', 'Content-Length', 'Location',
+                     'ETag', 'Pragma', 'Cache-Control', 'Expires'):
+            self.assertFalse(Request.is_valid_header(name))
+            self.assertFalse(Request.is_valid_header(name.lower()))
+        # Control code not allowed in header value.
+        self.assertFalse(Request.is_valid_header('X-Custom-1', '\x00custom1'))
+        self.assertFalse(Request.is_valid_header('X-Custom-1', 'cust\x0aom1'))
+        self.assertFalse(Request.is_valid_header('X-Custom-1', 'custom1\x7f'))
+        # Only a subset of special characters allowed in header name.
+        self.assertFalse(Request.is_valid_header('X-Custom-(2)', 'custom2'))
+        self.assertFalse(Request.is_valid_header('X-Custom-:2:', 'custom2'))
+        self.assertTrue(Request.is_valid_header('Aa0-!#$%&\'*+.^_`|~',
+                                                'custom2'))
+
 
 class RequestSendFileTestCase(unittest.TestCase):
 
@@ -526,7 +567,7 @@ class RequestSendFileTestCase(unittest.TestCase):
     def tearDown(self):
         if self.req and self.req._response:
             self.req._response.close()
-        shutil.rmtree(self.dir)
+        rmtree(self.dir)
 
     def _start_response(self, status, headers):
         self.status = status
@@ -663,7 +704,7 @@ class SendErrorTestCase(unittest.TestCase):
     def _send_error(self, admin_trac_url='.', perm=None,
                     error_klass=ValueError):
         self.env.config.set('project', 'admin_trac_url', admin_trac_url)
-        self.assertEquals(admin_trac_url, self.env.project_admin_trac_url)
+        self.assertEqual(admin_trac_url, self.env.project_admin_trac_url)
 
         content = StringIO()
         result = {'status': None, 'headers': []}
@@ -694,7 +735,7 @@ class SendErrorTestCase(unittest.TestCase):
                           data=data)
         content = content.getvalue().decode('utf-8')
         self.assertIn('<!DOCTYPE ', content)
-        self.assertEquals('500', result['status'].split()[0])
+        self.assertEqual('500', result['status'].split()[0])
         self.assertIn(('Content-Type', 'text/html;charset=utf-8'),
                       result['headers'])
         return content
