@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2013 Edgewall Software
+# Copyright (C) 2010-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -12,12 +12,13 @@
 # history and logs, available at http://trac.edgewall.org/log/.
 
 import os.path
+import re
 import sys
 import tempfile
-import types
 import unittest
 from subprocess import PIPE, Popen
 
+
 import trac.tests.compat
 from trac.config import ConfigurationError
 from trac.core import Component, ComponentManager, TracError, implements
@@ -140,6 +141,7 @@ class AuthenticateTestCase(unittest.TestCase):
                             'AuthenticateRequestHandler')
         authenticated = [0]
         req = MockRequest(self.env)
+        self.request_dispatcher.set_default_callbacks(req)
 
         self.assertEqual(1, len(self.request_dispatcher.authenticators))
         self.assertIsInstance(self.request_dispatcher.authenticators[0],
@@ -378,7 +380,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data, content_type):
                 return template, data, content_type
@@ -395,7 +397,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data, content_type):
                 return template, data, content_type
@@ -411,7 +413,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data,
                                      content_type, method=None):
@@ -429,7 +431,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data,
                                      content_type, method=None):
@@ -445,7 +447,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data,
                                      content_type, method=None):
@@ -461,7 +463,7 @@ class PostProcessRequestTestCase(unittest.TestCase):
         """
         class RequestFilter(Component):
             implements(IRequestFilter)
-            def pre_process_request(self, handler):
+            def pre_process_request(self, req, handler):
                 return handler
             def post_process_request(self, req, template, data,
                                      content_type, method=None):
@@ -472,6 +474,44 @@ class PostProcessRequestTestCase(unittest.TestCase):
         self.assertEqual(4, len(resp))
         self.assertEqual(args[:3] + ('xml',), resp)
 
+    def test_redirect_on_permission_error(self):
+        """The post_process_request method can redirect during exception
+        handling from an exception raised in process_request.
+        """
+        class RedirectOnPermissionErrorStub(Component):
+            implements(IRequestHandler, IRequestFilter)
+
+            def match_request(self, req):
+                return re.match(r'/perm-error', req.path_info)
+
+            def process_request(self, req):
+                req.entered_process_request = True
+                raise PermissionError("No permission to view")
+
+            def pre_process_request(self, req, handler):
+                return handler
+
+            def post_process_request(self, req, template, data, content_type):
+                if (template, data, content_type) == (None, None, None):
+                    req.entered_post_process_request = True
+                    req.redirect(req.href('/redirect-target'))
+                return template, data, content_type
+
+        dispatcher = RequestDispatcher(self.env)
+        req = MockRequest(self.env, method='GET', path_info='/perm-error')
+        req.entered_process_request = False
+        req.entered_post_process_request = False
+
+        try:
+            dispatcher.dispatch(req)
+        except RequestDone:
+            pass
+        else:
+            self.fail("RequestDone not raised")
+
+        self.assertTrue(req.entered_process_request)
+        self.assertTrue(req.entered_post_process_request)
+
 
 class RequestDispatcherTestCase(unittest.TestCase):
 
@@ -482,6 +522,9 @@ class RequestDispatcherTestCase(unittest.TestCase):
         filepath = os.path.join(self.env.templates_dir,
                                 TestStubRequestHandler.filename)
         create_file(filepath, TestStubRequestHandler.template)
+        self.filename = os.path.join(self.env.path, 'test.txt')
+        self.data = 'contents\n'
+        create_file(self.filename, self.data, 'wb')
 
     def tearDown(self):
         self.env.reset_db_and_disk()
@@ -493,6 +536,11 @@ class RequestDispatcherTestCase(unittest.TestCase):
         self.env.insert_users([(sid, name, email, 0)])
         return sid, name, email
 
+    def _content(self):
+        yield 'line1,'
+        yield 'line2,'
+        yield 'line3\n'
+
     def test_invalid_default_date_format_raises_exception(self):
         self.env.config.set('trac', 'default_date_format', u'ĭšo8601')
 
@@ -507,6 +555,7 @@ class RequestDispatcherTestCase(unittest.TestCase):
         req = MockRequest(self.env, path_info='/test-stub',
                           cookie='trac_session=%s;' % sid)
         request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
 
         self.assertRaises(RequestDone, request_dispatcher.dispatch, req)
 
@@ -521,9 +570,9 @@ class RequestDispatcherTestCase(unittest.TestCase):
     def test_get_session_returns_fake_session(self):
         """Fake session is returned when database is not reachable."""
         sid = self._insert_session()[0]
-        _get_session = RequestDispatcher._get_session
+        request_dispatcher = RequestDispatcher(self.env)
 
-        def get_session(self, req):
+        def get_session(req):
             """Simulates an unreachable database."""
             _get_connector = DatabaseManager.get_connector
 
@@ -532,15 +581,13 @@ class RequestDispatcherTestCase(unittest.TestCase):
 
             DatabaseManager.get_connector = get_connector
             DatabaseManager(self.env).shutdown()
-            session = _get_session(self, req)
+            session = request_dispatcher._get_session(req)
             DatabaseManager.get_connector = _get_connector
             return session
 
         req = MockRequest(self.env, path_info='/test-stub',
                           cookie='trac_session=%s;' % sid)
-        request_dispatcher = RequestDispatcher(self.env)
-        request_dispatcher._get_session = \
-            types.MethodType(get_session, request_dispatcher)
+        req.callbacks['session'] = get_session
 
         self.assertRaises(RequestDone, request_dispatcher.dispatch, req)
 
@@ -552,6 +599,119 @@ class RequestDispatcherTestCase(unittest.TestCase):
         self.assertEqual('200 Ok', req.status_sent[0])
         self.assertIn('<h1>Hello World</h1>', req.response_sent.getvalue())
 
+    def test_invalid_session_id_returns_fake_session(self):
+        """Fake session is returned when session id is invalid."""
+        sid = 'a' * 23 + '$'  # last char invalid, sid must be alphanumeric.
+        req = MockRequest(self.env, path_info='/test-stub',
+                          cookie='trac_session=%s;' % sid)
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+
+        self.assertRaises(RequestDone, request_dispatcher.dispatch, req)
+
+        self.assertIsInstance(req.session, FakeSession)
+        self.assertIsNone(req.session.sid)
+        self.assertEqual('200 Ok', req.status_sent[0])
+        self.assertIn('<h1>Hello World</h1>', req.response_sent.getvalue())
+
+    def test_set_valid_xsendfile_header(self):
+        """Send file using xsendfile header."""
+        self.env.config.set('trac', 'use_xsendfile', True)
+        self.env.config.set('trac', 'xsendfile_header', 'X-Accel-Redirect')
+
+        req = MockRequest(self.env)
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+
+        # File is sent using xsendfile.
+        self.assertRaises(RequestDone, req.send_file, self.filename)
+        self.assertEqual(['200 Ok'], req.status_sent)
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(self.filename, req.headers_sent['X-Accel-Redirect'])
+        self.assertNotIn('X-Sendfile', req.headers_sent)
+        self.assertEqual(None, req._response)
+        self.assertEqual('', req.response_sent.getvalue())
+
+    def _test_file_not_sent_using_xsendfile_header(self, xsendfile_header):
+        req = MockRequest(self.env)
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+
+        # File is not sent using xsendfile.
+        self.assertRaises(RequestDone, req.send_file, self.filename)
+        self.assertEqual(['200 Ok'], req.status_sent)
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertNotIn(xsendfile_header, req.headers_sent)
+        self.assertEqual('_FileWrapper', type(req._response).__name__)
+        self.assertEqual('', req.response_sent.getvalue())
+
+    def test_set_invalid_xsendfile_header(self):
+        """Not sent by xsendfile header because header is invalid."""
+        xsendfile_header = '(X-SendFile)'
+        self.env.config.set('trac', 'use_xsendfile', True)
+        self.env.config.set('trac', 'xsendfile_header', xsendfile_header)
+
+        self._test_file_not_sent_using_xsendfile_header(xsendfile_header)
+
+    def test_xsendfile_is_disabled(self):
+        """Not sent by xsendfile header because xsendfile is disabled."""
+        xsendfile_header = 'X-SendFile'
+        self.env.config.set('trac', 'use_xsendfile', False)
+        self.env.config.set('trac', 'xsendfile_header', xsendfile_header)
+
+        self._test_file_not_sent_using_xsendfile_header(xsendfile_header)
+
+    def _test_configurable_headers(self, method):
+        # Reserved headers not allowed.
+        content_type = 'not-allowed'
+        self.env.config.set('http-headers', 'Content-Type', content_type)
+        # Control code not allowed.
+        custom1 = '\x00custom1'
+        self.env.config.set('http-headers', 'X-Custom-1', custom1)
+        # Many special characters allowed in header name.
+        custom2 = 'Custom2-!#$%&\'*+.^_`|~'
+        self.env.config.set('http-headers', custom2, 'custom2')
+        # Some special characters not allowed in header name.
+        self.env.config.set('http-headers', 'X-Custom-(3)', 'custom3')
+
+        req = MockRequest(self.env, method='POST')
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+        self.assertRaises(RequestDone, getattr(req, method), self._content())
+
+        self.assertNotEqual('not-allowed', req.headers_sent.get('Content-Type'))
+        self.assertNotIn('x-custom-1', req.headers_sent)
+        self.assertIn(custom2.lower(), req.headers_sent)
+        self.assertNotIn('x-custom-(3)', req.headers_sent)
+
+    def test_send_configurable_headers(self):
+        self._test_configurable_headers('send')
+
+    def test_send_error_configurable_headers(self):
+        self._test_configurable_headers('send_error')
+
+    def test_send_configurable_headers_no_override(self):
+        """Headers in request not overridden by configurable headers."""
+        self.env.config.set('http-headers', 'X-XSS-Protection', '1; mode=block')
+        request_dispatcher = RequestDispatcher(self.env)
+        req1 = MockRequest(self.env)
+        request_dispatcher.set_default_callbacks(req1)
+
+        self.assertRaises(RequestDone, req1.send, self._content())
+
+        self.assertNotIn('X-XSS-protection', req1.headers_sent)
+        self.assertIn('x-xss-protection', req1.headers_sent)
+        self.assertEqual('1; mode=block', req1.headers_sent['x-xss-protection'])
+
+        req2 = MockRequest(self.env, method='POST')
+        request_dispatcher.set_default_callbacks(req2)
+
+        self.assertRaises(RequestDone, req2.send, self._content())
+
+        self.assertNotIn('x-xss-protection', req2.headers_sent)
+        self.assertIn('X-XSS-Protection', req2.headers_sent)
+        self.assertEqual('0', req2.headers_sent['X-XSS-Protection'])
+
 
 class HdfdumpTestCase(unittest.TestCase):
 
