@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,8 +19,8 @@ import trac.tests.compat
 from trac.admin.api import console_date_format
 from trac.test import EnvironmentStub, MockRequest
 from trac.util.datefmt import format_date, to_datetime
-from trac.web.session import DetachedSession, Session, PURGE_AGE, \
-                             UPDATE_INTERVAL, SessionAdmin
+from trac.web.session import DetachedSession, PURGE_AGE, Session, \
+                             SessionAdmin, SessionDict, UPDATE_INTERVAL
 from trac.core import TracError
 from trac.util.datefmt import time_now
 
@@ -59,25 +59,43 @@ def _prep_session_table(env, spread_visits=False):
     all_list = auth_list + anon_list
     return auth_list, anon_list, all_list
 
+def get_session_attrs(env, sid):
+    rows = env.db_query("""
+        SELECT a.sid, a.name, a.value
+        FROM session AS s
+        LEFT JOIN session_attribute AS a
+            ON a.sid=s.sid AND a.authenticated=s.authenticated
+        WHERE s.sid=%s
+        """, (sid,))
+    if rows:
+        return dict((row[1], row[2]) for row in rows if row[0])
+    else:
+        return None
 
 def get_session_info(env, sid):
     """
     :since 1.0: changed `db` input parameter to `env`
     :since 1.1.2: returns `default_handler` as the fourth entry in the tuple.
     """
-    for row in env.db_query("""
-            SELECT DISTINCT s.sid, n.value, e.value, h.value FROM session AS s
-            LEFT JOIN session_attribute AS n
-              ON (n.sid=s.sid AND n.name='name')
-            LEFT JOIN session_attribute AS e
-              ON (e.sid=s.sid AND e.name='email')
-            LEFT JOIN session_attribute AS h
-              ON (h.sid=s.sid AND h.name='default_handler')
-            WHERE s.sid=%s
-            """, (sid,)):
-        return row
-    else:
+    attrs = get_session_attrs(env, sid)
+    if attrs is None:
         return None, None, None, None
+    else:
+        return sid, attrs.get('name'), attrs.get('email'), \
+               attrs.get('default_handler')
+
+
+class SessionDictTestCase(unittest.TestCase):
+
+    def test_initializer(self):
+        """`SessionDict` initializer accepts same arguments as `dict`."""
+        session = SessionDict(kwarg1=1, kwarg2='b')
+        self.assertEqual(1, session['kwarg1'])
+        self.assertEqual('b', session['kwarg2'])
+
+        session = SessionDict([('kwarg1', 1), ('kwarg2', 'b')])
+        self.assertEqual(1, session['kwarg1'])
+        self.assertEqual('b', session['kwarg2'])
 
 
 class SessionTestCase(unittest.TestCase):
@@ -276,19 +294,21 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='123456' AND name='foo'
             """)[0][0])
 
-    def test_purge_anonymous_session(self):
-        """
-        Verify that old sessions get purged.
-        """
+    def _purge_anonymous_session(self):
+        now = int(time_now())
+        lifetime = 90 * 86400  # default lifetime
         with self.env.db_transaction as db:
-            db("INSERT INTO session VALUES ('123456', 0, %s)", (0,))
-            db("INSERT INTO session VALUES ('987654', 0, %s)",
-               (int(time_now() - PURGE_AGE - 3600),))
-            db("""
+            db.executemany("INSERT INTO session VALUES (%s, 0, %s)",
+                           [('123456', 0),
+                            ('987654', now - lifetime - 3600),
+                            ('876543', now - lifetime + 3600),
+                            ('765432', now - 3600)])
+            db.executemany("""
                 INSERT INTO session_attribute
-                VALUES ('987654', 0, 'foo', 'bar')
-                """)
+                VALUES (%s, 0, 'foo', 'bar')
+                """, [('987654',), ('876543',), ('765432',)])
 
+        with self.env.db_transaction as db:
             # We need to modify a different session to trigger the purging
             req = MockRequest(self.env, authname='anonymous')
             req.incookie['trac_session'] = '123456'
@@ -296,9 +316,26 @@ class SessionTestCase(unittest.TestCase):
             session['foo'] = 'bar'
             session.save()
 
-        self.assertEqual(0, self.env.db_query("""
-            SELECT COUNT(*) FROM session WHERE sid='987654' AND authenticated=0
-            """)[0][0])
+        return [row[0] for row in self.env.db_query("""
+            SELECT sid FROM session WHERE authenticated=0 ORDER BY sid
+            """)]
+
+    def test_purge_anonymous_session(self):
+        """
+        Verify that old sessions get purged.
+        """
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432', '876543'], sids)
+
+    def test_purge_anonymous_session_with_short_lifetime(self):
+        self.env.config.set('trac', 'anonymous_session_lifetime', '1')
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432'], sids)
+
+    def test_purge_anonymous_session_disabled(self):
+        self.env.config.set('trac', 'anonymous_session_lifetime', '0')
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432', '876543', '987654'], sids)
 
     def test_delete_empty_session(self):
         """
@@ -541,31 +578,65 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='john' AND name='foo'
             """)[0][0])
 
-    def test_session_set_email(self):
-        """Setting session email invalidates known_users cache."""
-        session = DetachedSession(self.env, 'user')
-        session['email'] = 'user@domain.org'
+    def test_create_new_session(self):
+        """Setting attribute on a new session invalidates known_users cache."""
+        sid = 'user'
+        session = DetachedSession(self.env, sid)
+        session.authenticated = True
 
         self.assertEqual([], list(self.env.get_known_users()))
         session.save()
-        email = None
-        for sid, name, email in self.env.get_known_users():
-            if sid == 'user':
-                break
-        self.assertEqual(session['email'], email)
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertIsNone(known_users[0][1])
+        self.assertIsNone(known_users[0][2])
+
+    def test_create_new_anonymous_session(self):
+        """Setting attribute on a new anonymous session doesn't invalidate
+        known_users cache."""
+        sid = 'anonymous'
+        session = DetachedSession(self.env, sid)
+        session.authenticated = False
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        # insert a session record without invalidating cache
+        self.env.insert_users([('user', 'Name' 'user@example.com', 1)])
+        session.save()
+        self.assertEqual([], list(self.env.get_known_users()))
+
+        self.env.invalidate_known_users_cache()
+        self.assertEqual(1, len(list(self.env.get_known_users())))
+
+    def test_session_set_email(self):
+        """Setting session email invalidates known_users cache."""
+        sid = 'user'
+        email = 'user@domain.org'
+        session = DetachedSession(self.env, sid)
+        session['email'] = email
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        session.save()
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertIsNone(known_users[0][1])
+        self.assertEqual(email, known_users[0][2])
 
     def test_session_set_name(self):
         """Setting session name invalidates known_users cache."""
-        session = DetachedSession(self.env, 'user')
-        session['name'] = 'The User'
+        sid = 'user'
+        name = 'The User'
+        session = DetachedSession(self.env, sid)
+        session['name'] = name
 
         self.assertEqual([], list(self.env.get_known_users()))
         session.save()
-        name = None
-        for sid, name, email in self.env.get_known_users():
-            if sid == 'user':
-                break
-        self.assertEqual(session['name'], name)
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertEqual(name, known_users[0][1])
+        self.assertIsNone(known_users[0][2])
 
     def test_session_admin_list(self):
         auth_list, anon_list, all_list = _prep_session_table(self.env)
@@ -594,24 +665,39 @@ class SessionTestCase(unittest.TestCase):
         self.assertRaises(Exception, sess_admin._do_add, 'name00')
 
         sess_admin._do_add('john')
-        result = get_session_info(self.env, 'john')
-        self.assertEqual(result, ('john', None, None, None))
+        self.assertEqual({}, get_session_attrs(self.env, 'john'))
         self.assertIn(('john', None, None),
                       list(self.env.get_known_users()))
 
         sess_admin._do_add('john1', 'John1')
-        result = get_session_info(self.env, 'john1')
-        self.assertEqual(result, ('john1', 'John1', None, None))
+        self.assertEqual({'name': 'John1'},
+                         get_session_attrs(self.env, 'john1'))
         self.assertIn(('john1', 'John1', None),
                       list(self.env.get_known_users()))
 
         sess_admin._do_add('john2', 'John2', 'john2@example.org')
-        result = get_session_info(self.env, 'john2')
-        self.assertEqual(result,
-                         ('john2', 'John2', 'john2@example.org', None))
+        self.assertEqual({'name': 'John2', 'email': 'john2@example.org'},
+                         get_session_attrs(self.env, 'john2'))
         self.assertIn(('john2', 'John2', 'john2@example.org'),
                       list(self.env.get_known_users()))
 
+        sess_admin._do_add('alice1', None, 'alice1@example.org')
+        self.assertEqual({'email': 'alice1@example.org'},
+                         get_session_attrs(self.env, 'alice1'))
+        sess_admin._do_add('alice2', '', 'alice2@example.org')
+        self.assertEqual({'email': 'alice2@example.org'},
+                         get_session_attrs(self.env, 'alice2'))
+        sess_admin._do_add('bob1', 'Bob 1', None)
+        self.assertEqual({'name': 'Bob 1'},
+                         get_session_attrs(self.env, 'bob1'))
+        sess_admin._do_add('bob2', 'Bob 2', '')
+        self.assertEqual({'name': 'Bob 2'},
+                         get_session_attrs(self.env, 'bob2'))
+        sess_admin._do_add('charlie1', '', '')
+        self.assertEqual({}, get_session_attrs(self.env, 'charlie1'))
+        sess_admin._do_add('charlie2', None, None)
+        self.assertEqual({}, get_session_attrs(self.env, 'charlie2'))
+
     def test_session_admin_set(self):
         _prep_session_table(self.env)
         sess_admin = SessionAdmin(self.env)
@@ -623,21 +709,30 @@ class SessionTestCase(unittest.TestCase):
         self.assertIn(('name00', 'val00', 'val00'),
                       list(self.env.get_known_users()))
         sess_admin._do_set('name', 'name00', 'john')
-        result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, ('name00', 'john', 'val00', None))
+        self.assertEqual({'name': 'john', 'email': 'val00'},
+                         get_session_attrs(self.env, 'name00'))
         self.assertIn(('name00', 'john', 'val00'),
                       list(self.env.get_known_users()))
 
         sess_admin._do_set('email', 'name00', 'john@example.org')
-        result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, ('name00', 'john', 'john@example.org', None))
+        self.assertEqual({'name': 'john', 'email': 'john@example.org'},
+                         get_session_attrs(self.env, 'name00'))
         self.assertIn(('name00', 'john', 'john@example.org'),
                       list(self.env.get_known_users()))
-
         sess_admin._do_set('default_handler', 'name00', 'SearchModule')
-        result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, ('name00', 'john', 'john@example.org',
-                                  'SearchModule'))
+        self.assertEqual({'name': 'john', 'email': 'john@example.org',
+                          'default_handler': 'SearchModule'},
+                         get_session_attrs(self.env, 'name00'))
+
+        sess_admin._do_set('name', 'name00', '')
+        self.assertEqual({'email': 'john@example.org',
+                          'default_handler': 'SearchModule'},
+                         get_session_attrs(self.env, 'name00'))
+        sess_admin._do_set('email', 'name00', '')
+        self.assertEqual({'default_handler': 'SearchModule'},
+                         get_session_attrs(self.env, 'name00'))
+        sess_admin._do_set('default_handler', 'name00', '')
+        self.assertEqual({}, get_session_attrs(self.env, 'name00'))
 
     def test_session_admin_delete(self):
         _prep_session_table(self.env)
@@ -646,14 +741,12 @@ class SessionTestCase(unittest.TestCase):
         self.assertIn(('name00', 'val00', 'val00'),
                       list(self.env.get_known_users()))
         sess_admin._do_delete('name00')
-        result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, (None, None, None, None))
+        self.assertEqual(None, get_session_attrs(self.env, 'name00'))
         self.assertNotIn(('name00', 'val00', 'val00'),
                          list(self.env.get_known_users()))
 
         sess_admin._do_delete('nothere')
-        result = get_session_info(self.env, 'nothere')
-        self.assertEqual(result, (None, None, None, None))
+        self.assertEqual(None, get_session_attrs(self.env, 'nothere'))
 
         auth_list, anon_list, all_list = _prep_session_table(self.env)
         sess_admin._do_delete('anonymous')
@@ -668,10 +761,10 @@ class SessionTestCase(unittest.TestCase):
         sess_admin._do_purge('2010-01-02')
         result = [i for i in sess_admin._get_list(['*'])]
         self.assertEqual(result, auth_list + anon_list)
-        result = get_session_info(self.env, anon_list[0][0])
-        self.assertEqual(result, ('name10', 'val10', 'val10', None))
-        result = get_session_info(self.env, anon_list[1][0])
-        self.assertEqual(result, ('name11', 'val11', 'val11', None))
+        self.assertEqual({'name': 'val10', 'email': 'val10'},
+                         get_session_attrs(self.env, anon_list[0][0]))
+        self.assertEqual({'name': 'val11', 'email': 'val11'},
+                         get_session_attrs(self.env, anon_list[1][0]))
 
         auth_list, anon_list, all_list = \
             _prep_session_table(self.env, spread_visits=True)
@@ -682,8 +775,9 @@ class SessionTestCase(unittest.TestCase):
             SELECT name, value FROM session_attribute WHERE sid = %s
             """, (anon_list[0][0],))
         self.assertEqual([], rows)
-        result = get_session_info(self.env, anon_list[1][0])
-        self.assertEqual(result, ('name11', 'val11', 'val11', None))
+        self.assertEqual(None, get_session_attrs(self.env, anon_list[0][0]))
+        self.assertEqual({'name': 'val11', 'email': 'val11'},
+                         get_session_attrs(self.env, anon_list[1][0]))
 
     def test_session_get_session_with_invalid_sid(self):
         req = MockRequest(self.env, authname='anonymous')
@@ -719,7 +813,10 @@ class SessionTestCase(unittest.TestCase):
 
 
 def test_suite():
-    return unittest.makeSuite(SessionTestCase)
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(SessionDictTestCase))
+    suite.addTest(unittest.makeSuite(SessionTestCase))
+    return suite
 
 
 if __name__ == '__main__':
