@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2012 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -16,6 +16,7 @@
 
 from fnmatch import fnmatchcase
 from itertools import groupby
+import fnmatch
 import inspect
 import os
 import re
@@ -38,15 +39,20 @@ from trac.util.translation import _, dgettext, cleandoc_, tag_
 from trac.web.chrome import chrome_resource_path
 from trac.wiki.api import IWikiMacroProvider, WikiSystem, parse_args
 from trac.wiki.formatter import (
-    MacroError, OutlineFormatter, extract_link, format_to_html,
+    MacroError, OutlineFormatter, ProcessorError, extract_link, format_to_html,
     format_to_oneliner, system_message
-)
+)  # ProcessorError unused, but imported for plugin use.
 from trac.wiki.interwiki import InterWikiMap
 
 
 # TODO: should be moved in .api
 class WikiMacroBase(Component):
-    """Abstract base class for wiki macros."""
+    """Abstract base class for wiki macros and processors.
+
+    On usage error, the `MacroError` or `ProcessorError` exception should be
+    raised, to ensure proper display of the error message in the rendered
+    wiki content.
+    """
 
     implements(IWikiMacroProvider)
     abstract = True
@@ -537,13 +543,15 @@ class ImageMacro(WikiMacroBase):
     _quoted_re = re.compile("(?:[\"'])(.*)(?:[\"'])$")
 
     def expand_macro(self, formatter, name, content):
-        # args will be null if the macro is called without parenthesis.
-        if not content:
+        args = None
+        if content:
+            content = stripws(content)
+            # parse arguments
+            # we expect the 1st argument to be a filename (filespec)
+            args = [stripws(arg) for arg
+                                 in self._split_args_re.split(content)[1::2]]
+        if not args:
             return ''
-        # parse arguments
-        # we expect the 1st argument to be a filename (filespec)
-        args = [stripws(arg) for arg
-                             in self._split_args_re.split(content)[1::2]]
         # strip unicode white-spaces and ZWSPs are copied from attachments
         # section (#10668)
         filespec = args.pop(0)
@@ -609,7 +617,7 @@ class ImageMacro(WikiMacroBase):
         if self._quoted_re.match(filespec):
             filespec = filespec.strip('\'"')
         # parse filespec argument to get realm and id if contained.
-        parts = [i.strip('''['"]''')
+        parts = [i.strip('\'"')
                  for i in self._split_filespec_re.split(filespec)[1::2]]
         realm = parts[0] if parts else None
         url = raw_url = desc = None
@@ -698,6 +706,9 @@ class ImageMacro(WikiMacroBase):
         if style:
             attr['style'] = '; '.join('%s:%s' % (k, escape(v))
                                       for k, v in style.iteritems())
+        if not WikiSystem(self.env).is_safe_origin(raw_url,
+                                                   formatter.context.req):
+            attr['crossorigin'] = 'anonymous'  # avoid password prompt
         result = tag.img(src=raw_url, **attr)
         if link is not None:
             result = tag.a(result, href=link or url,
@@ -772,32 +783,56 @@ class TracIniMacro(WikiMacroBase):
     _description = cleandoc_(
     """Produce documentation for the Trac configuration file.
 
-    Typically, this will be used in the TracIni page.
-    Optional arguments are a configuration section filter,
+    Typically, this will be used in the TracIni page. The macro accepts
+    two ordered arguments and two named arguments.
+
+    The ordered arguments are a configuration section filter,
     and a configuration option name filter: only the configuration
     options whose section and name start with the filters are output.
+
+    The named arguments can be specified:
+
+     section :: a glob-style filtering on the section names
+     option  :: a glob-style filtering on the option names
     """)
 
     def expand_macro(self, formatter, name, content):
         from trac.config import ConfigSection, Option
-        section_filter = key_filter = ''
+
         args, kw = parse_args(content)
-        if args:
-            section_filter = args.pop(0).strip()
-        if args:
-            key_filter = args.pop(0).strip()
+        filters = {}
+        for name, index in (('section', 0), ('option', 1)):
+            pattern = kw.get(name, '').strip()
+            if pattern:
+                filters[name] = fnmatch.translate(pattern)
+                continue
+            prefix = args[index].strip() if index < len(args) else ''
+            if prefix:
+                filters[name] = re.escape(prefix)
+        has_option_filter = 'option' in filters
+        for name in ('section', 'option'):
+            filters[name] = re.compile(filters[name], re.IGNORECASE).match \
+                            if name in filters \
+                            else lambda v: True
+        section_filter = filters['section']
+        option_filter = filters['option']
 
-        registry = ConfigSection.get_registry(self.compmgr)
-        sections = dict((name, section.doc)
-                        for name, section in registry.iteritems()
-                        if name.startswith(section_filter))
-
-        registry = Option.get_registry(self.compmgr)
+        section_registry = ConfigSection.get_registry(self.compmgr)
+        option_registry = Option.get_registry(self.compmgr)
         options = {}
-        for (section, key), option in registry.iteritems():
-            if section.startswith(section_filter):
+        for (section, key), option in option_registry.iteritems():
+            if section_filter(section) and option_filter(key):
                 options.setdefault(section, {})[key] = option
-                sections.setdefault(section, '')
+        if not has_option_filter:
+            for section in section_registry:
+                if section_filter(section):
+                    options.setdefault(section, {})
+        for section in options:
+            options[section] = sorted(options[section].itervalues(),
+                                      key=lambda option: option.name)
+        sections = [(section, section_registry[section].doc
+                              if section in section_registry else '')
+                    for section in sorted(options)]
 
         def default_cell(option):
             default = option.default
@@ -807,20 +842,27 @@ class TracIniMacro(WikiMacroBase):
             else:
                 return tag.td(_("(no default)"), class_='nodefault')
 
+        def options_table(section, options):
+            if options:
+                return tag.table(class_='wiki')(
+                    tag.tbody(
+                        tag.tr(
+                            tag.td(tag.a(tag.code(option.name),
+                                         class_='tracini-option',
+                                         href='#%s-%s-option' %
+                                              (section, option.name))),
+                            tag.td(format_to_html(self.env, formatter.context,
+                                                  option.doc)),
+                            default_cell(option),
+                            id='%s-%s-option' % (section, option.name),
+                            class_='odd' if idx % 2 else 'even')
+                     for idx, option in enumerate(options)))
+
         return tag.div(class_='tracini')(
             (tag.h3(tag.code('[%s]' % section), id='%s-section' % section),
              format_to_html(self.env, formatter.context, section_doc),
-             tag.table(class_='wiki')(tag.tbody(
-                 tag.tr(tag.td(tag.code(option.name)),
-                        tag.td(format_to_html(self.env, formatter.context,
-                                              option.doc)),
-                        default_cell(option),
-                        class_='odd' if idx % 2 else 'even')
-                 for idx, option in
-                    enumerate(sorted(options.get(section, {}).itervalues(),
-                                     key=lambda o: o.name))
-                 if option.name.startswith(key_filter))))
-            for section, section_doc in sorted(sections.iteritems()))
+             options_table(section, options.get(section)))
+            for section, section_doc in sections)
 
 
 class KnownMimeTypesMacro(WikiMacroBase):
