@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -18,12 +18,13 @@ import trac.wiki.api
 import trac.wiki.formatter
 import trac.wiki.parser
 from trac.wiki.tests import (
-    formatter, macros, model, web_api, web_ui, wikisyntax)
+    admin, formatter, macros, model, web_api, web_ui, wikisyntax)
 from trac.wiki.tests.functional import functionalSuite
 
 def test_suite():
 
     suite = unittest.TestSuite()
+    suite.addTest(admin.test_suite())
     suite.addTest(formatter.test_suite())
     suite.addTest(macros.test_suite())
     suite.addTest(model.test_suite())
