@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2018 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -13,13 +13,13 @@
 
 from StringIO import StringIO
 import os
-import shutil
 import tempfile
 import unittest
 
 from trac.attachment import Attachment
-from trac.config import Option, ListOption, IntOption, BoolOption
-from trac.test import locale_en
+from trac.config import BoolOption, ConfigSection, IntOption, ListOption, \
+                        Option
+from trac.test import locale_en, rmtree
 from trac.util.datefmt import datetime_now, format_date, utc
 from trac.wiki.model import WikiPage
 from trac.wiki.tests import formatter
@@ -47,6 +47,7 @@ def image_setup(tc):
     tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
     add_attachment(tc, 'wiki', 'page:fr', 'img.png')
     add_attachment(tc, 'wiki', 'page', 'img.png')
+    add_attachment(tc, 'wiki', 'page', '][img.png')
     tc.env.config.set('interwiki', 'shields', 'https://img.shields.io/')
     tc.env.config.set('interwiki', 'travis',
                       'https://travis-ci.org/$1?branch=$2')
@@ -55,7 +56,7 @@ def image_setup(tc):
     tc.env.config.set('trac', 'htdocs_location', htdocs_location)
 
 def image_teardown(tc):
-    shutil.rmtree(os.path.join(tc.env.path, 'files'))
+    rmtree(os.path.join(tc.env.path, 'files'))
     os.rmdir(tc.env.path) # there was only 'files' below tc.env.path
     tc.env.reset_db()
 
@@ -64,6 +65,26 @@ def image_teardown(tc):
 #       normal ASCII SPACE, second is Unicode NO-BREAK SPACE).
 
 IMAGE_MACRO_TEST_CASES = u"""
+============================== Image, no arguments
+[[Image]]
+------------------------------
+
+============================== Image, no arguments
+[[Image()]]
+------------------------------
+
+============================== Image, multiple no arguments
+[[Image(,)]]
+------------------------------
+
+============================== Image, whitespace argument
+[[Image( )]]
+------------------------------
+
+============================== Image, ZWSP argument
+[[Image(​)]]
+------------------------------
+
 ============================== source: Image, no other arguments
 [[Image(source:« test ».png)]]
 ------------------------------
@@ -84,7 +105,7 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(trac:source:/trunk/doc/images/bkgnd_pattern_« test ».png)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png"><img src="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" /></a>
+<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png"><img src="http://trac.edgewall.org/intertrac/source%3A/trunk/doc/images/bkgnd_pattern_%C2%AB%20test%C2%A0%C2%BB.png%3Fformat%3Draw" alt="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" crossorigin="anonymous" title="source:/trunk/doc/images/bkgnd_pattern_« test ».png in Trac's Trac" /></a>
 </p>
 ============================== source: Image, nolink
 [[Image(source:« test », nolink)]]
@@ -114,13 +135,13 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(http://www.edgewall.com/gfx/shredder_« test ».png, nolink)]]
 ------------------------------
 <p>
-<img src="http://www.edgewall.com/gfx/shredder_« test ».png" alt="http://www.edgewall.com/gfx/shredder_« test ».png" title="http://www.edgewall.com/gfx/shredder_« test ».png" />
+<img src="http://www.edgewall.com/gfx/shredder_« test ».png" alt="http://www.edgewall.com/gfx/shredder_« test ».png" crossorigin="anonymous" title="http://www.edgewall.com/gfx/shredder_« test ».png" />
 </p>
 ============================== http: Image, absolute, many ':'
 [[Image(http://chart.apis.google.com:80/chart?cht=p3&chd=s:hW&chs=250x100&chl=Héllo|Wôrld, title=Google & Charting, link=)]]
 ------------------------------
 <p>
-<img src="http://chart.apis.google.com:80/chart?cht=p3&amp;chd=s:hW&amp;chs=250x100&amp;chl=Héllo|Wôrld" alt="http://chart.apis.google.com:80/chart" title="Google &amp; Charting" />
+<img src="http://chart.apis.google.com:80/chart?cht=p3&amp;chd=s:hW&amp;chs=250x100&amp;chl=Héllo|Wôrld" alt="http://chart.apis.google.com:80/chart" crossorigin="anonymous" title="Google &amp; Charting" />
 </p>
 ============================== // Image, server-relative
 [[Image(//browser/« test »?format=raw, link=)]]
@@ -175,62 +196,62 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(travis:edgewall/trac.svg:1.0-stable, link=trac:source:/branches/1.0-stable)]]
 ------------------------------
 <p>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk"><img src="https://img.shields.io/travis/edgewall/trac.svg" alt="travis/edgewall/trac.svg in shields" title="travis/edgewall/trac.svg in shields" /></a>
-<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/branches/1.0-stable"><img src="https://travis-ci.org/edgewall/trac.svg?branch=1.0-stable" alt="edgewall/trac.svg:1.0-stable in travis" title="edgewall/trac.svg:1.0-stable in travis" /></a>
+<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk"><img src="https://img.shields.io/travis/edgewall/trac.svg" alt="travis/edgewall/trac.svg in shields" crossorigin="anonymous" title="travis/edgewall/trac.svg in shields" /></a>
+<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/branches/1.0-stable"><img src="https://travis-ci.org/edgewall/trac.svg?branch=1.0-stable" alt="edgewall/trac.svg:1.0-stable in travis" crossorigin="anonymous" title="edgewall/trac.svg:1.0-stable in travis" /></a>
 </p>
 ============================== InterWiki, nolink
 [[Image(shields:pypi/dm/trac.svg, nolink)]]
 ------------------------------
 <p>
-<img src="https://img.shields.io/pypi/dm/trac.svg" alt="pypi/dm/trac.svg in shields" title="pypi/dm/trac.svg in shields" />
+<img src="https://img.shields.io/pypi/dm/trac.svg" alt="pypi/dm/trac.svg in shields" crossorigin="anonymous" title="pypi/dm/trac.svg in shields" />
 </p>
 ==============================
 [[Image(notfound.png, nolink)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;notfound.png&#34; attached to WikiStart" title="No image &#34;notfound.png&#34; attached to WikiStart" />
+<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;notfound.png&#34; attached to WikiStart" crossorigin="anonymous" title="No image &#34;notfound.png&#34; attached to WikiStart" />
 </p>
 ==============================
 [[Image(img.png, margin-bottom=-1)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img.png&#34; attached to WikiStart" style="margin-bottom: 1px" title="No image &#34;img.png&#34; attached to WikiStart" />
+<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img.png&#34; attached to WikiStart" title="No image &#34;img.png&#34; attached to WikiStart" style="margin-bottom: 1px" crossorigin="anonymous" />
 </p>
 ==============================
 [[Image(img.png, margin-bottom=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>margin-bottom=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, margin-bottom=--) failed</strong><pre>Invalid macro argument <code>margin-bottom=--</code></pre></div>
 </p>
 ==============================
 [[Image(img.png, margin-top=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>margin-top=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, margin-top=--) failed</strong><pre>Invalid macro argument <code>margin-top=--</code></pre></div>
 </p>
 ==============================
 [[Image(img.png, margin=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>margin=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, margin=--) failed</strong><pre>Invalid macro argument <code>margin=--</code></pre></div>
 </p>
 ==============================
 [[Image(img.png, margin-left=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>margin-left=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, margin-left=--) failed</strong><pre>Invalid macro argument <code>margin-left=--</code></pre></div>
 </p>
 ==============================
 [[Image(img.png, margin-right=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>margin-right=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, margin-right=--) failed</strong><pre>Invalid macro argument <code>margin-right=--</code></pre></div>
 </p>
 ==============================
 [[Image(img.png, border=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>border=--</code></strong></div>
+<div class="system-message"><strong>Macro Image(img.png, border=--) failed</strong><pre>Invalid macro argument <code>border=--</code></pre></div>
 </p>
 ==============================  # Regression test for #12333
 = [[Image]]
@@ -246,7 +267,19 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(wiki:page:img2.png)]]
 ------------------------------
 <p>
-<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img2.png&#34; attached to page" title="No image &#34;img2.png&#34; attached to page" />
+<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;img2.png&#34; attached to page" crossorigin="anonymous" title="No image &#34;img2.png&#34; attached to page" />
+</p>
+==============================  "[" and "]" characters - 1 (#12762)
+[[Image(wiki:page:][img.png,nolink)]]
+------------------------------
+<p>
+<img src="/raw-attachment/wiki/page/%5D%5Bimg.png" alt="image in page" title="image in page" />
+</p>
+==============================  "[" and "]" characters - 2 (#12762)
+[[Image(][img.png,nolink)]]
+------------------------------
+<p>
+<img src="http://assets.example.org/common/attachment.png" alt="No image &#34;][img.png&#34; attached to WikiStart" crossorigin="anonymous" title="No image &#34;][img.png&#34; attached to WikiStart" />
 </p>
 """
 
@@ -285,13 +318,13 @@ TITLEINDEX1_MACRO_TEST_CASES = u"""
 [[TitleIndex(min=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>min=--</code></strong></div>
+<div class="system-message"><strong>Macro TitleIndex(min=--) failed</strong><pre>Invalid macro argument <code>min=--</code></pre></div>
 </p>
 ==============================
 [[TitleIndex(depth=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>depth=--</code></strong></div>
+<div class="system-message"><strong>Macro TitleIndex(depth=--) failed</strong><pre>Invalid macro argument <code>depth=--</code></pre></div>
 </p>
 ------------------------------
 """
@@ -575,7 +608,7 @@ RECENTCHANGES_MACRO_TEST_CASES = u""""
 [[RecentChanges(Trac, --)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>--</code></strong></div>
+<div class="system-message"><strong>Macro RecentChanges(Trac, --) failed</strong><pre>Invalid macro argument <code>--</code></pre></div>
 </p>
 ------------------------------
 """
@@ -599,18 +632,18 @@ def recentchanges_teardown(tc):
     tc.env.reset_db()
 
 
-PAGEOUTLINE_MACRO_TEST_CASES = u""""\
+PAGEOUTLINE_MACRO_TEST_CASES = u""""
 ==============================
 [[PageOutline(a)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>a</code></strong></div>
+<div class="system-message"><strong>Macro PageOutline(a) failed</strong><pre>Invalid macro argument <code>a</code></pre></div>
 </p>
 ==============================
 [[PageOutline(a-b)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <code>a</code></strong></div>
+<div class="system-message"><strong>Macro PageOutline(a-b) failed</strong><pre>Invalid macro argument <code>a</code></pre></div>
 </p>
 ==============================
 [[PageOutline(0)]]
@@ -703,11 +736,16 @@ TRACINI_MACRO_TEST_CASES = u"""\
 <p>
 </p><div class="tracini"><h3 id="section-42-section"><code>[section-42]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><code>option1</code></td><td></td><td class="default"><code>value</code></td></tr>\
-<tr class="odd"><td><code>option2</code></td><td><p>
+<tr class="even" id="section-42-option1-option">\
+<td><a class="tracini-option" href="#section-42-option1-option"><code>option1</code></a></td><td></td>\
+<td class="default"><code>value</code></td></tr>\
+<tr class="odd" id="section-42-option2-option">\
+<td><a class="tracini-option" href="#section-42-option2-option"><code>option2</code></a></td><td><p>
 blah
 </p>
-</td><td class="default"><code>value</code></td></tr><tr class="even"><td><code>option3</code></td><td><p>
+</td><td class="default"><code>value</code></td></tr>\
+<tr class="even" id="section-42-option3-option">\
+<td><a class="tracini-option" href="#section-42-option3-option"><code>option3</code></a></td><td><p>
 Doc for option3
 </p>
 </td><td class="default"><code>value</code></td></tr></tbody></table></div><p>
@@ -720,7 +758,9 @@ Doc for option3
 </p><div class="tracini">\
 <h3 id="section-list-section"><code>[section-list]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><code>option1</code></td><td></td><td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
+<tr class="even" id="section-list-option1-option">\
+<td><a class="tracini-option" href="#section-list-option1-option"><code>option1</code></a></td><td></td>\
+<td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
 </tbody></table>\
 </div><p>
 </p>
@@ -732,20 +772,123 @@ Doc for option3
 </p><div class="tracini">\
 <h3 id="section-def-section"><code>[section-def]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><code>option1</code></td><td></td><td class="nodefault">(no default)</td></tr>\
-<tr class="odd"><td><code>option2</code></td><td></td><td class="nodefault">(no default)</td></tr>\
-<tr class="even"><td><code>option3</code></td><td></td><td class="default"><code>0</code></td></tr>\
-<tr class="odd"><td><code>option4</code></td><td></td><td class="default"><code>disabled</code></td></tr>\
-<tr class="even"><td><code>option5</code></td><td></td><td class="nodefault">(no default)</td></tr>\
+<tr class="even" id="section-def-option1-option">\
+<td><a class="tracini-option" href="#section-def-option1-option"><code>option1</code></a></td><td>\
+</td><td class="nodefault">(no default)</td></tr>\
+<tr class="odd" id="section-def-option2-option">\
+<td><a class="tracini-option" href="#section-def-option2-option"><code>option2</code></a></td><td></td>\
+<td class="nodefault">(no default)</td></tr>\
+<tr class="even" id="section-def-option3-option">\
+<td><a class="tracini-option" href="#section-def-option3-option"><code>option3</code></a></td><td></td>\
+<td class="default"><code>0</code></td></tr>\
+<tr class="odd" id="section-def-option4-option">\
+<td><a class="tracini-option" href="#section-def-option4-option"><code>option4</code></a></td><td></td>\
+<td class="default"><code>disabled</code></td></tr>\
+<tr class="even" id="section-def-option5-option">\
+<td><a class="tracini-option" href="#section-def-option5-option"><code>option5</code></a></td><td></td>\
+<td class="nodefault">(no default)</td></tr>\
 </tbody></table>\
 </div><p>
 </p>
 ------------------------------
+============================== TracIni, option argument
+[[TracIni(,option5)]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-def-section"><code>[section-def]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-def-option5-option">\
+<td><a class="tracini-option" href="#section-def-option5-option"><code>option5</code></a></td><td></td>\
+<td class="nodefault">(no default)</td></tr>\
+</tbody></table>\
+</div><p>
+</p>
+------------------------------
+============================== TracIni, option named argument
+[[TracIni(option=opt?o*[24])]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-42-section"><code>[section-42]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-42-option2-option">\
+<td><a class="tracini-option" href="#section-42-option2-option"><code>option2</code></a></td><td><p>
+blah
+</p>
+</td><td class="default"><code>value</code></td></tr></tbody></table>\
+<h3 id="section-def-section"><code>[section-def]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-def-option2-option">\
+<td><a class="tracini-option" href="#section-def-option2-option"><code>option2</code></a></td><td></td>\
+<td class="nodefault">(no default)</td></tr>\
+<tr class="odd" id="section-def-option4-option">\
+<td><a class="tracini-option" href="#section-def-option4-option"><code>option4</code></a></td><td></td>\
+<td class="default"><code>disabled</code></td></tr>\
+</tbody></table>\
+</div><p>
+</p>
+------------------------------
+============================== TracIni, section and option named argument
+[[TracIni(section=section-*,option=opt*[13])]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-42-section"><code>[section-42]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-42-option1-option">\
+<td><a class="tracini-option" href="#section-42-option1-option"><code>option1</code></a></td><td></td>\
+<td class="default"><code>value</code></td></tr>\
+<tr class="odd" id="section-42-option3-option">\
+<td><a class="tracini-option" href="#section-42-option3-option"><code>option3</code></a></td>\
+<td><p>
+Doc for option3
+</p>
+</td><td class="default"><code>value</code></td></tr>\
+</tbody></table>\
+<h3 id="section-def-section"><code>[section-def]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-def-option1-option">\
+<td><a class="tracini-option" href="#section-def-option1-option"><code>option1</code></a></td><td></td>\
+<td class="nodefault">(no default)</td></tr>\
+<tr class="odd" id="section-def-option3-option">\
+<td><a class="tracini-option" href="#section-def-option3-option"><code>option3</code></a></td><td></td>\
+<td class="default"><code>0</code></td></tr>\
+</tbody></table>\
+<h3 id="section-list-section"><code>[section-list]</code></h3>\
+<table class="wiki"><tbody>\
+<tr class="even" id="section-list-option1-option">\
+<td><a class="tracini-option" href="#section-list-option1-option"><code>option1</code></a></td><td></td>\
+<td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
+</tbody></table>\
+</div><p>
+</p>
+------------------------------
+============================== TracIni, section with no options
+[[TracIni(section=section-no-options)]]
+------------------------------
+<p>
+</p><div class="tracini">\
+<h3 id="section-no-options-section"><code>[section-no-options]</code></h3>\
+<p>
+No options
+</p>
+</div><p>
+</p>
+------------------------------
+============================== TracIni, ordered arguments don't glob
+[[TracIni(section*,option*)]]
+------------------------------
+<p>
+</p><div class="tracini"></div><p>
+</p>
+------------------------------
 """
 
 def tracini_setup(tc):
-    tc._orig_registry = Option.registry
+    tc._orig_registries = ConfigSection.registry, Option.registry
     class Foo(object):
+        section = (ConfigSection)('section-no-options', doc='No options')
         option_a1 = (Option)('section-42', 'option1', 'value', doc='')
         option_a2 = (Option)('section-42', 'option2', 'value', doc='blah')
         option_a3 = (Option)('section-42', 'option3', 'value',
@@ -761,7 +904,7 @@ def tracini_setup(tc):
         option_d5 = (ListOption)('section-def', 'option5', [])
 
 def tracini_teardown(tc):
-    Option.registry = tc._orig_registry
+    ConfigSection.registry, Option.registry = tc._orig_registries
 
 
 def test_suite():
