@@ -395,28 +395,31 @@ class Storage(object):
             self._fs_to_unicode = self._fs_from_unicode = lambda s: s
 
         # simple sanity checking
-        __git_file_path = partial(os.path.join, git_dir)
-        control_files = ['HEAD', 'objects', 'refs']
-        control_files_exist = \
-            lambda p: all(map(os.path.exists, map(p, control_files)))
-        if not control_files_exist(__git_file_path):
-            __git_file_path = partial(os.path.join, git_dir, '.git')
-            if os.path.exists(__git_file_path()) and \
-                    control_files_exist(__git_file_path):
-                git_dir = __git_file_path()
-            else:
-                self.logger.error("GIT control files missing in '%s'"
-                                  % git_dir)
-                raise GitError("GIT control files not found, maybe wrong "
-                               "directory?")
-        # at least, check that the HEAD file is readable
-        head_file = os.path.join(git_dir, 'HEAD')
         try:
-            with open(head_file, 'rb'):
+            os.listdir(git_dir)
+        except EnvironmentError as e:
+            self._raise_not_readable(git_dir, e)
+        if not self._control_files_exist(git_dir):
+            dot_git_dir = os.path.join(git_dir, '.git')
+            try:
+                os.listdir(dot_git_dir)
+            except EnvironmentError:
+                missing = True
+            else:
+                if self._control_files_exist(dot_git_dir):
+                    missing = False
+                    git_dir = dot_git_dir
+                else:
+                    missing = True
+            if missing:
+                raise GitError("Git control files not found in '%s'" % git_dir)
+
+        # at least, check that the HEAD file is readable
+        try:
+            with open(os.path.join(git_dir, 'HEAD'), 'rb'):
                 pass
-        except IOError as e:
-            raise GitError("Make sure the Git repository '%s' is readable: %s"
-                           % (git_dir, to_unicode(e)))
+        except EnvironmentError as e:
+            self._raise_not_readable(git_dir, e)
 
         self.repo = GitCore(git_dir, git_bin, log, git_fs_encoding)
         self.repo_path = git_dir
@@ -595,6 +598,11 @@ class Storage(object):
                           key=lambda (name, rev, head): (not head, name))
         return [(name, rev) for name, rev, head in branches]
 
+    def get_refs(self):
+        for refname, rev in self.rev_cache.refs_dict.iteritems():
+            if refname != 'HEAD':
+                yield refname, rev
+
     def get_commits(self):
         return self.rev_cache.rev_dict
 
@@ -791,27 +799,19 @@ class Storage(object):
                       for name, rev_ in self.rev_cache.iter_tags()
                       if rev is None or rev == rev_)
 
-    def ls_tree(self, rev, path=''):
+    def ls_tree(self, rev, path='', recursive=False):
         rev = rev and str(rev) or 'HEAD' # paranoia
-
-        path = self._fs_from_unicode(path)
-
-        if path.startswith('/'):
-            path = path[1:]
-
-        tree = self.repo.ls_tree('-z', '-l', rev, '--', path).split('\0')
+        path = self._fs_from_unicode(path).lstrip('/') or '.'
+        tree = self.repo.ls_tree('-zlr' if recursive else '-zl',
+                                 rev, '--', path).split('\0')
 
         def split_ls_tree_line(l):
             """split according to '<mode> <type> <sha> <size>\t<fname>'"""
 
             meta, fname = l.split('\t', 1)
             _mode, _type, _sha, _size = meta.split()
-
-            if _size == '-':
-                _size = None
-            else:
-                _size = int(_size)
-
+            _mode = int(_mode, 8)
+            _size = None if _size == '-' else int(_size)
             return _mode, _type, _sha, _size, self._fs_to_unicode(fname)
 
         return [ split_ls_tree_line(e) for e in tree if e ]
@@ -907,10 +907,10 @@ class Storage(object):
         p = []
         change = {}
         next_path = []
-        base_path = self._fs_from_unicode(base_path)
+        base_path = self._fs_from_unicode(base_path) or '.'
 
         def name_status_gen():
-            p[:] = [self.repo.log_pipe('--pretty=format:%n%H',
+            p[:] = [self.repo.log_pipe('--pretty=format:%n%H', '--no-renames',
                                        '--name-status', sha, '--', base_path)]
             f = p[0].stdout
             for l in f:
@@ -1014,49 +1014,63 @@ class Storage(object):
         # diff-tree returns records with the following structure:
         # :<old-mode> <new-mode> <old-sha> <new-sha> <change> NUL <old-path> NUL [ <new-path> NUL ]
 
-        path = self._fs_from_unicode(path).strip('/')
+        path = self._fs_from_unicode(path).strip('/') or '.'
         diff_tree_args = ['-z', '-r']
         if find_renames:
             diff_tree_args.append('-M')
         diff_tree_args.extend([str(tree1) if tree1 else '--root',
-                               str(tree2),
-                               '--', path])
+                               str(tree2), '--', path])
+        result = self.repo.diff_tree(*diff_tree_args)
+        if not result:
+            return
 
-        lines = self.repo.diff_tree(*diff_tree_args).split('\0')
+        def iter_entry(result):
+            start = 0
+            while True:
+                idx = result.find('\0', start)
+                if idx == -1:
+                    return
+                yield result[start:idx]
+                start = idx + 1
 
-        assert lines[-1] == ''
-        del lines[-1]
+        iterate = iter_entry(result)
 
-        if tree1 is None and lines:
+        def next_entry():
+            return iterate.next()
+
+        if not tree1:
             # if only one tree-sha is given on commandline,
             # the first line is just the redundant tree-sha itself...
-            assert not lines[0].startswith(':')
-            del lines[0]
+            entry = next_entry()
+            assert not entry.startswith(':')
 
-        # FIXME: the following code is ugly, needs rewrite
+        while True:
+            try:
+                entry = next_entry()
+            except StopIteration:
+                return
+            assert entry.startswith(':')
+            values = entry[1:].split(' ')
+            assert len(values) == 5
+            old_mode, new_mode, old_sha, new_sha, change = values
+            old_mode = int(old_mode, 8)
+            new_mode = int(new_mode, 8)
+            change = change[:1]
+            old_path = self._fs_to_unicode(next_entry())
+            new_path = None
+            if change in ('R', 'C'):  # renamed or copied
+                new_path = self._fs_to_unicode(next_entry())
+            yield (old_mode, new_mode, old_sha, new_sha, change, old_path,
+                   new_path)
 
-        chg = None
+    def _raise_not_readable(self, git_dir, e):
+        raise GitError("Make sure the Git repository '%s' is readable: %s"
+                       % (git_dir, to_unicode(e)))
 
-        def __chg_tuple():
-            if len(chg) == 6:
-                chg.append(None)
-            else:
-                chg[6] = self._fs_to_unicode(chg[6])
-            chg[5] = self._fs_to_unicode(chg[5])
-
-            assert len(chg) == 7
-            return tuple(chg)
-
-        for line in lines:
-            if line.startswith(':'):
-                if chg:
-                    yield __chg_tuple()
-
-                chg = line[1:].split()
-                assert len(chg) == 5
-            else:
-                chg.append(line)
-
-        # handle left-over chg entry
-        if chg:
-            yield __chg_tuple()
+    def _control_files_exist(self, git_dir):
+        for name in ('HEAD', 'objects', 'refs'):
+            if not os.path.exists(os.path.join(git_dir, name)):
+                self.logger.debug("Missing Git control file '%s' in '%s'",
+                                  name, git_dir)
+                return False
+        return True
